 /*!****************************************************************************
 *    \file csp.cpp
 *    This file contains the implementation of the csp class belonging
 *    to the Star Algorithm.
 *    Date: 08/02/2005
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/
#include <vcl.h>
#include "csp.h"
#include <time>


////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
   \fn csp::csp()*/
csp::csp()
{
}//-----------------------------------------------------------------------------

/** Default Constructor.
   \fn csp::csp()*/
csp::csp(csp &CSP)
{Constraints=CSP.Constraints;
 X=CSP.X;
}//-----------------------------------------------------------------------------

csp& csp::operator=(const csp &CSP)
{
 Constraints=CSP.Constraints;
 X=CSP.X;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

list<fstar>& csp::GetConstraints(void)
{return Constraints;
}//-----------------------------------------------------------------------------

list<fstar>& csp::GetVectConstraints(void)
{return VectConstraints;
}//-----------------------------------------------------------------------------

box& csp::GetX(void)
{return X;
}//-----------------------------------------------------------------------------

void csp::SetConstraints(list<fstar>& CONSTR)
{Constraints=CONSTR;
}//-----------------------------------------------------------------------------

void csp::SetX(box& XBOX)
{X=XBOX;
}//-----------------------------------------------------------------------------

/*
iboolean csp::CheckQSIConsistency(box &X)
{
  iboolean Test=itrue;
  iboolean PTest=itrue;
  bool TTest;
  bool FTest;
  fstream fout;
  twin TW;
  string CName;
  tree* T;
  double LBound,UBound;
  ivalDb Ival;

  list<fstar>::iterator C;
  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
     CName=C->GetModel().GetName();
     T=&C->GetModel().GetTree();

     if(Test==itrue)
     {
       PTest=C->CheckPunctualConsistency(X,true);

       if(PTest==itrue)       // Test all the box
       {
         C->GetModel().AssignXToNodeVars(X,true);
         T->AssignModes(true,C->GetPars());
         C->Run(X,fout);
         TW=C->GetTwin();
         TTest=IsIn(C->GetTwin().GetOut(),Zero());
       }
       else {TTest=false;}
     }
     else {TTest=false;}
     if(TTest)
      {Test=itrue;}
     else if(PTest==ifalse)
      {
       PTest=C->CheckPunctualConsistency(X,false);
       if(PTest==itrue)       // Test all the box
       {
         C->GetModel().AssignXToNodeVars(X,false);
         T->AssignModes(false,C->GetPars());
         C->Run(X,fout);
         TW=C->GetTwin();
         FTest=!IsIn(C->GetTwin().GetInn(),Zero());
       }
       else FTest=false;
       if(FTest)
        {Test=ifalse;
         break;}
       else
        {Test=iperhaps;}
      }
     else {Test=iperhaps;}

  } // End for

  return Test;
}//-----------------------------------------------------------------------------
*/

/*
iboolean csp::CheckQSIConsistency(box &X)
{
  iboolean Test=itrue;
  bool TTest1,TTest2;
  bool FTest;
  fstream fout;
  twin TW;
  string CName;
  tree* T;
  double LBound,UBound;
  ivalDb Ival;

  list<fstar>::iterator C;
  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
     CName=C->GetModel().GetName();
     T=&C->GetModel().GetTree();

     C->AssignRelEps(X,0.1);

     Ival=X[1].GetIval();

     if(Test==itrue)
     {
       X[1].SetIval(Ival.Inf);
       C->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       TTest1=IsIn(C->GetTwin().GetOut(),Zero());

       if(TTest1)
       {
        X[1].SetIval(Ival.Sup);
        C->GetModel().AssignXToNodeVars(X,true);
        T->AssignModes(true,C->GetPars());
        C->Run(X,fout);
        TW=C->GetTwin();
        TTest2=IsIn(C->GetTwin().GetOut(),Zero());
       }
       else
        {TTest2=false;}
     }
     else
      {TTest1=false;
       TTest2=false;}

     if(TTest1 && TTest2)
      {Test=itrue;}
     else if(!TTest1 && !TTest2)
      {
       X[1].SetIval(Ival);

       C->GetModel().AssignXToNodeVars(X,false);
       T->AssignModes(false,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       FTest=!IsIn(C->GetTwin().GetInn(),Zero());
       if(FTest)
        {Test=ifalse;
         break;}
       else
        {Test=iperhaps;}
      }
      else
       {Test=iperhaps;}
     } // End for

  X[1].SetIval(Ival);

  return Test;
}//-----------------------------------------------------------------------------
*/
/*
iboolean csp::CheckQSIConsistency(box &X)
{
  iboolean Test=itrue;
  bool TTest=false;
  bool FTest=false;
  fstream fout;
  twin TW;
  string CName;
  tree* T;
  double LBound,UBound;
  ivalDb Ival;



  list<fstar>::iterator C;
  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
     CName=C->GetModel().GetName();
     T=&C->GetModel().GetTree();

     //C->AssignRelEps(X,0.01);

     if(Test==itrue)
     {
       C->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       TTest=IsIn(C->GetTwin().GetOut(),Zero());

     }
     else {TTest=false;}
     if(TTest)
      {Test=itrue;}
     else
      {
       C->GetModel().AssignXToNodeVars(X,false);
       T->AssignModes(false,C->GetPars());
//       fout.open("salidaQSI.txt",ios::out);
//       C->GetPars().SetDebug(1);
       C->Run(X,fout);
//       fout.close();
       TW=C->GetTwin();
       ivalDb temp=C->GetTwin().GetInn();
       FTest=!IsIn(C->GetTwin().GetInn(),Zero());
       if(FTest)
        {Test=ifalse;
         break;}
       else
        {Test=iperhaps;}

       // Test=iperhaps;
      }
     }

  return Test;
}//-----------------------------------------------------------------------------
*/



iboolean csp::CheckQSIConsistency(box &X, int rule)
{
  iboolean Test=itrue;
  bool TTest=false;
  bool FTest=false;
  fstream fout;
  twin TW;
  string CName;
  tree* T;
  double LBound,UBound;
  ivalDb Ival;


/////////////////////////////
/*  double x0=0,y0=0,z0=10;
  ivalDb t=ivalDb(0,10);
  ivalDb xa,ya,za;
*/
////////////////////////////

  list<fstar>::iterator C;
  C=this->Constraints.begin();

  CName=C->GetModel().GetName();
  T=&C->GetModel().GetTree();
  Test=iperhaps;
  if(rule==1) {
       //verify if not contain
       C->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       if (IsIn(C->GetTwin().GetOut(),Zero())) Test=itrue;
  } else {
       //verify if not contain
       C->GetModel().AssignXToNodeVars(X,false);
       T->AssignModes(false,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       if (!IsIn(C->GetTwin().GetInn(),Zero())) Test=ifalse;
  }


/*  if(rule==1) {
          if (!IsInfinity( trace((*X)[0].GetIval(),(*X)[1].GetIval()).GetInf() )) {
               return ifalse;
          }
  } else {
       //verify if not contain
       C->GetModel().AssignXToNodeVars(X,false);
       T->AssignModes(false,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       if (!IsIn(C->GetTwin().GetInn(),Zero())) Test=ifalse;
  }
  */
  return Test;
}//-----------------------------------------------------------------------------


/*iboolean csp::CheckQSIConsistency1(box &X)
{
  iboolean Test=itrue;
  bool TTest=false;
  bool FTest=false;
  fstream fout;
  twin TW;
  string CName;
  tree* T;
  double LBound,UBound;
  ivalDb Ival;

  list<fstar>::iterator C;
  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
     CName=C->GetModel().GetName();
     T=&C->GetModel().GetTree();

     //C->AssignRelEps(X,0.01);

     if(Test==itrue)
     {
       C->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       TTest=IsIn(C->GetTwin().GetOut(),Zero());
     }
     else {TTest=false;}
     if(TTest)
      {Test=itrue;}
     else
      {
       C->GetModel().AssignXToNodeVars(X,false);
       T->AssignModes(false,C->GetPars());
       C->Run(X,fout);
       TW=C->GetTwin();
       FTest=!IsIn(C->GetTwin().GetInn(),Zero());
       if(FTest)
        {Test=ifalse;
         break;}
       else
        {Test=iperhaps;}
      }
     }

  return Test;

}*/

//-----------------------------------------------------------------------------

iboolean csp::CheckQSIVectConsistency(box &X)
{
  iboolean TTest;
  iboolean Test=iperhaps;
  twin TW;
  tree* T;
  fstream fout;

  Test=this->VectorialConsistency(X);


  if(Test!=itrue)
  {
    list<fstar>::iterator C;
    for (C=this->VectConstraints.begin(); C != this->VectConstraints.end(); C++)
    {
         T=&C->GetModel().GetTree();
         C->GetModel().AssignXToNodeVars(X,false);
         T->AssignModes(false,C->GetPars());
         C->Run(X,fout);
         TW=C->GetTwin();
         TTest=!IsIn(C->GetTwin().GetInn(),Zero());
         if(TTest==itrue)
         {Test=ifalse;
          break;}
    }
  }

  return Test;
}//-----------------------------------------------------------------------------


iboolean csp::VectorialConsistency(box &X)
{
  list<box> List;
  box V(2);
  list<box>::iterator IV;
  V[0].SetName("v1");
  V[0].SetIval(ivalDb(-3,3));
  V[1].SetName("v2");
  V[1].SetIval(ivalDb(-5,5));
  iboolean Test=iperhaps;
  iboolean TTest1=itrue;
  iboolean TTest2=itrue;
  twin TW;
  ivalDb IVAL1,IVAL2;
  fstream fout;
  tree* T;
  list<fstar>::iterator C1,C2;
  C1=this->Constraints.begin();
  C2=--this->Constraints.end();

  T=&C1->GetModel().GetTree();
  C1->GetModel().AssignXToNodeVars(X,true);
  T->AssignModes(true,C1->GetPars());
  C1->Run(X,fout);
  TW=C1->GetTwin();
  TTest1=IsIn(TW.GetOut(),Zero());

  T=&C2->GetModel().GetTree();
  C2->GetModel().AssignXToNodeVars(X,true);
  T->AssignModes(true,C2->GetPars());
  C2->Run(X,fout);
  TW=C2->GetTwin();
  TTest2=IsIn(TW.GetOut(),Zero());


  if(TTest1==itrue && TTest2==itrue)
  {
    List.push_back(V);
    while(!List.empty() && Test==iperhaps)
    {
     IV=List.begin();

     IVAL1=IV->operator [](0).GetIval();
     IVAL2=IV->operator [](1).GetIval();

     C1->GetModel().AssignXToNodeVars(X,false);
     T->AssignModes(false,C1->GetPars());

     C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL1));
     C1->GetModel().GetTree().GetNodeVars()[3].SetIval(Impr(IVAL2));
     C1->Run(X,fout);
     TW=C1->GetTwin();
     TTest1=!IsIn(TW.GetInn(),Zero());

     if(TTest1!=itrue)
     {
     C2->GetModel().AssignXToNodeVars(X,false);
     T->AssignModes(false,C2->GetPars());

     C2->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL1));
     C2->GetModel().GetTree().GetNodeVars()[3].SetIval(Impr(IVAL2));
     C2->Run(X,fout);
     TW=C2->GetTwin();
     TTest2=!IsIn(TW.GetInn(),Zero());
     }

     if(TTest1==itrue || TTest2==itrue)
     {List.pop_front();}

     else
     {

       C1->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C1->GetPars());

       C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL1));
       C1->GetModel().GetTree().GetNodeVars()[3].SetIval(Prop(IVAL2));
       C1->Run(X,fout);
       TW=C1->GetTwin();
       TTest1=IsIn(TW.GetOut(),Zero());

       if(TTest1==itrue)
       {
       C2->GetModel().AssignXToNodeVars(X,true);
       T->AssignModes(true,C2->GetPars());

       C2->GetModel().GetTree().GetNodeVars()[2].SetIval(Prop(IVAL1));
       C2->GetModel().GetTree().GetNodeVars()[3].SetIval(Impr(IVAL2));
       C2->Run(X,fout);
       TW=C2->GetTwin();
       TTest2=IsIn(TW.GetOut(),Zero());
       }

       if(TTest1==itrue && TTest2==itrue)
       {Test=itrue;}

       else
       {
         C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Prop(IVAL1));
         C1->GetModel().GetTree().GetNodeVars()[3].SetIval(Impr(IVAL2));
         C1->Run(X,fout);
         TW=C1->GetTwin();
         TTest1=IsIn(TW.GetOut(),Zero());

         if(TTest1==itrue)
         {
         C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL1));
         C1->GetModel().GetTree().GetNodeVars()[3].SetIval(Prop(IVAL2));
         C2->Run(X,fout);
         TW=C2->GetTwin();
         TTest2=IsIn(TW.GetOut(),Zero());
         }

         if(TTest1==itrue && TTest2==itrue)
         {Test=itrue;}

         else if(Width(Prop(IV->operator [](0).GetIval()))<10 && Width(Prop(IV->operator [](1).GetIval()))<10)
         {List.pop_front();}

         else
         {ivalDb Domain;
         int Index;
         ivalDb Domain1=Prop(IV->operator [](0).GetIval());
         ivalDb Domain2=Prop(IV->operator [](1).GetIval());
         if(Width(Domain1)>Width(Domain2))
          {Domain=Domain1;
           Index=0;}
         else
          {Domain=Domain2;
           Index=1;}
         double Inf=Domain.GetInf();
         double Sup=Domain.GetSup();
         double Middle = Center(Domain);

         IV->operator [](Index).SetIval(Ival(Inf,Middle));
         List.push_back(*IV);
         IV->operator [](Index).SetIval(Ival(Middle,Sup));
         List.splice(List.end(),List,IV);
         }
       }
      }
   }

 }  // End if

 C1->GetModel().GetTree().GetNodeVars()[2].SetIval(ivalDb(3,-3));
 C2->GetModel().GetTree().GetNodeVars()[2].SetIval(ivalDb(3,-3));
 C1->GetModel().GetTree().GetNodeVars()[3].SetIval(ivalDb(5,-5));
 C2->GetModel().GetTree().GetNodeVars()[3].SetIval(ivalDb(5,-5));

 return Test;
}//-----------------------------------------------------------------------------


/*
iboolean csp::VectorialConsistency(box &X)
{
  list<box> List;
  box V(1);
  list<box>::iterator IV;
  V[0].SetName("v1");
  V[0].SetIval(ivalDb(-3,3));
  iboolean Test=iperhaps;
  iboolean TTest1=itrue;
  iboolean TTest2=itrue;
  twin TW;
  ivalDb IVAL;
  fstream fout;
  tree* T;
  list<fstar>::iterator C1,C2;
  C1=this->Constraints.begin();
  C2=--this->Constraints.end();

  T=&C1->GetModel().GetTree();
  C1->GetModel().AssignXToNodeVars(X,true);
  T->AssignModes(true,C1->GetPars());

  T=&C2->GetModel().GetTree();
  C2->GetModel().AssignXToNodeVars(X,true);
  T->AssignModes(true,C2->GetPars());

  List.push_back(V);

  while(!List.empty() && Test==iperhaps)
  {
   IV=List.begin();

   IVAL=IV->operator [](0).GetIval();

   ivalDb Asd=C1->GetModel().GetTree().GetNodeVars()[2].GetIval();
   C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL));
   C1->GetModel().GetTree().AssignModes(true,C1->GetPars());
   C1->Run(X,fout);
   TW=C1->GetTwin();
   TTest1=IsIn(TW.GetOut(),Zero());

   ivalDb Asd1=C2->GetModel().GetTree().GetNodeVars()[3].GetIval();
   C2->GetModel().GetTree().GetNodeVars()[2].SetIval(Prop(IVAL));
   C2->GetModel().GetTree().AssignModes(true,C2->GetPars());
   C2->Run(X,fout);
   TW=C2->GetTwin();
   TTest2=IsIn(TW.GetOut(),Zero());

   if(TTest1==ifalse)
   {List.pop_front();}

   else if(TTest1==itrue && TTest2==itrue)
   {Test=itrue;}

   else
   {
     C1->GetModel().GetTree().GetNodeVars()[2].SetIval(Prop(IVAL));
     C1->GetModel().GetTree().AssignModes(true,C1->GetPars());
     C1->Run(X,fout);
     TW=C1->GetTwin();
     TTest1=IsIn(TW.GetOut(),Zero());

     C2->GetModel().GetTree().GetNodeVars()[2].SetIval(Impr(IVAL));
     C2->GetModel().GetTree().AssignModes(true,C2->GetPars());
     C2->Run(X,fout);
     TW=C2->GetTwin();
     TTest2=IsIn(TW.GetOut(),Zero());

     if(TTest2==ifalse)
     {List.pop_front();}

     else if(TTest1==itrue && TTest2==itrue)
     {Test=itrue;}

     else if(Width(Prop(IV->operator [](0).GetIval()))<0.01)
     {List.pop_front();}

     else
     {
     ivalDb Domain=Prop(IV->operator [](0).GetIval());
     double Inf=Domain.GetInf();
     double Sup=Domain.GetSup();
     double Middle = Center(Domain);

     IV->operator [](0).SetIval(Ival(Inf,Middle));
     List.push_back(*IV);
     IV->operator [](0).SetIval(Ival(Middle,Sup));
     List.splice(List.end(),List,IV);
     }
   }
  }

 C1->GetModel().GetTree().GetNodeVars()[2].SetIval(ivalDb(3,-3));
 C2->GetModel().GetTree().GetNodeVars()[2].SetIval(ivalDb(3,-3));

 return Test;
}//-----------------------------------------------------------------------------

*/

void csp::Delete(void)
{
 list<fstar>::iterator C;
 for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {C->Delete();}
}//-----------------------------------------------------------------------------

void csp::Print(void)
{
 list<fstar>::iterator C;
 for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {C->GetModel().GetTree().PrintTrees();}
}//-----------------------------------------------------------------------------

void csp::CreateModels(box &X,fstream &fout)
{
 list<fstar>::iterator C;
 for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
   C->CreateModel(X,fout);
   C->GetModel().AssignNodeVarsXIndexes(X);
   C->GetModel().AssignXCellIndexes(X);
   C->GetPars().SetPlot(false);
   C->GetPars().SetCells(false);
   C->GetPars().SetSolution(0);
   }

  for (C=this->VectConstraints.begin(); C != this->VectConstraints.end(); C++)
  {
   C->CreateModel(X,fout);
   C->GetModel().AssignNodeVarsXIndexes(X);
   C->GetModel().AssignXCellIndexes(X);
   C->GetPars().SetPlot(false);
   C->GetPars().SetCells(false);
   C->GetPars().SetSolution(0);
   }
}//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Propagate
//------------------------------------------------------------------------------
iboolean csp::Propagate(box &X,fstream &fout,double Eps,int &PCounter)
{
  box AX;
  AX.Vars.resize(X.Vars.size());
  bool Run=true;
  iboolean Consistent=iperhaps;
  list<fstar>::iterator C;

  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
   C->GetModel().AssignXToNodeVars(X,true);
   CreateCellFromTree(C->GetModel().GetCell(),C->GetModel().GetTree());
   AssignCellToTree(C->GetModel().GetTree(),C->GetModel().GetCell(),false);
   C->GetModel().GetTree().InitializeNodes();
  }
  
  while(Run && Consistent==iperhaps)
  {
    PCounter++;
    //fout << "Propagation: " << PCounter << endl << endl;
    AX.AssignIval(X);
    for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
    {//fout << "Constraint: " << C->GetModel().GetName() << endl << endl;                                 // Assign Proper Modality

     int Opt=C->GetPars().GetOptim();
     if(Opt!=2)
      {Consistent=C->Propagate(X,fout,"minmax");}
     if(Opt!=1)
      {Consistent=C->Propagate(X,fout,"maxmin");}

     if(Consistent==ifalse){break;}
    }
    Run=X.IsEpsBigger(AX,Eps);
  }
 return Consistent;
}//-----------------------------------------------------------------------------


bool IsXInConstraint(variable& X, fstar& Constraint)
{
 bool Belongs=false;
 cell* Cell=&Constraint.GetModel().GetCell();

 for(int i=0;i<Cell->GetDim();i++)
 {
  if(X.GetName()==Cell->operator[](i).GetName())
  {Belongs=true;
   break;}
 }
 return Belongs;
}//-----------------------------------------------------------------------------


bool csp::Contract(box &X,double Eps,fstream &fout,bool ModeAlgorithm,double &TimeOut)
{
  ivalDb Ival,Approx;
  double Inf,Mid,Sup;
  bool GConsistency=true;
  bool TTest,FTest,Bisect;
  string NameVar,NameConstraint;
  list<ivalDb> IvalList;
  list<ivalDb>::iterator I;
  list<list<fstar>::iterator> LBuffer,RBuffer;
  list<list<fstar>::iterator>::iterator PC;
  list<list<list<fstar>::iterator>::iterator> Trash;
  list<list<list<fstar>::iterator>::iterator>::iterator T;
  list<fstar>::iterator C;

  double CTime;                                  // Computation Time
  clock_t CStart = clock();                      // Start internal clock.
  clock_t CStop;                                 // Final Time

  for(int i=0; i<X.GetVars().size(); i++)
   {
    NameVar=X.GetVars()[i].GetName();
    LBuffer.clear();
    RBuffer.clear();
    for(C=this->Constraints.begin(); C != this->Constraints.end(); C++)  // Load Constraints to Buffer
    {if(IsXInConstraint(X.GetVars()[i],*C))
     {LBuffer.push_back(C);
      RBuffer.push_back(C);}}

    IvalList.clear();                                            // Clear IvalList
    IvalList.push_back(X.GetVars()[i].GetIval());

    while(!IvalList.empty() && Width(IvalList.back()) > Eps && !LBuffer.empty())    // Constract Right Bound
    {
      Bisect=true;
      Trash.clear();
      Ival=IvalList.back();
      X.GetVars()[i].SetIval(Ival);
      for(PC=LBuffer.begin(); PC != LBuffer.end(); PC++)
      {
         NameConstraint=(*PC)->GetModel().GetName();
         (*PC)->GetModel().AssignXToNodeVars(X,-1);     // Test if the interval can be eliminated
         (*PC)->GetPars().SetInclusion(2);
         (*PC)->Run(X,fout);
         Approx=(*PC)->GetTwin().GetInn();
         if(IsPNaN(Approx.GetInf()) && IsPNaN(Approx.GetSup()))
         {FTest=true;}
         else if(IsNaN(Approx.GetInf()) || IsNaN(Approx.GetSup()))
         {FTest=false;}
         else{FTest=!IsIn(Approx,Zero());}
         if(FTest)
          {Bisect=false;
           IvalList.pop_back();
           break;}

         else if(ModeAlgorithm)           // Test if the constraint can be eliminated
         {
         (*PC)->GetModel().AssignXToNodeVars(X,i);
         (*PC)->GetPars().SetInclusion(1);
         (*PC)->Run(X,fout);
         Approx=(*PC)->GetTwin().GetOut();
         TTest=IsIn(Approx,Zero());
         if(TTest)
          {Trash.push_back(PC);}
         }
         
      } // End For

      for(T=Trash.begin();T!=Trash.end();T++)     // Erase Inactive constraints from the buffer
      {LBuffer.erase(*T);}

      if(Bisect && !LBuffer.empty())
      {Ival=IvalList.back();
       Inf=Ival.GetInf();
       Sup=Ival.GetSup();
       Mid=Ival.GetMid();
       IvalList.pop_back();
       IvalList.push_back(ivalDb(Inf,Mid));
       IvalList.push_back(ivalDb(Mid,Sup));}
      else {}

    CStop = clock();
    CTime=(CStop-CStart)/CLK_TCK;
    if(CTime > TimeOut)
     {break;}

    } // End While (Constract Right)

    while(!IvalList.empty() && Width(IvalList.front()) > Eps && !RBuffer.empty())  //Contract Left Bound
    {
      Bisect=true;
      Trash.clear();
      Ival=IvalList.front();
      X.GetVars()[i].SetIval(Ival);
      for(PC=RBuffer.begin(); PC != RBuffer.end(); PC++)
      {
       NameConstraint=(*PC)->GetModel().GetName();
       (*PC)->GetModel().AssignXToNodeVars(X,-1);
       (*PC)->GetPars().SetInclusion(2);
       (*PC)->Run(X,fout);
       Approx=(*PC)->GetTwin().GetInn();
       if(IsPNaN(Approx.GetInf()) && IsPNaN(Approx.GetSup()))
       {FTest=true;}
       else if(IsNaN(Approx.GetInf()) || IsNaN(Approx.GetSup()))
       {FTest=false;}
       else{FTest=!IsIn(Approx,Zero());}
       if(FTest)
        {Bisect=false;
         IvalList.pop_front();
         break;}

       else if(ModeAlgorithm)
       {
       (*PC)->GetModel().AssignXToNodeVars(X,i);
       (*PC)->GetPars().SetInclusion(1);
       (*PC)->Run(X,fout);
       Approx=(*PC)->GetTwin().GetOut();
       TTest=IsIn(Approx,Zero());
       if(TTest)
        {Trash.push_back(PC);}
       }

      } // End For

      for(T=Trash.begin();T!=Trash.end();T++)     // Erase Inactive constraints from the buffer
      {RBuffer.erase(*T);}

      if(Bisect && !RBuffer.empty())
      {Ival=IvalList.front();
       Inf=Ival.GetInf();
       Sup=Ival.GetSup();
       Mid=Ival.GetMid();
       IvalList.pop_front();
       IvalList.push_front(ivalDb(Mid,Sup));
       IvalList.push_front(ivalDb(Inf,Mid));}
       else {}

    CStop = clock();
    CTime=(CStop-CStart)/CLK_TCK;
    if(CTime > TimeOut){break;}

    } // End while (Constract Left)

    if(IvalList.empty())
    {GConsistency=false;
     break;}
    else
    {Ival.SetInf(IvalList.front().GetInf());
     Ival.SetSup(IvalList.back().GetSup());
     X.GetVars()[i].SetIval(Ival);
     GConsistency=true;}
   }
TimeOut=CTime;

 return GConsistency;
}//-----------------------------------------------------------------------------

iboolean csp::CheckConsistency(void)
{
  iboolean Test=itrue;
  iboolean TTest;
  fstream fout;
  twin TW;
  box X;

  list<fstar>::iterator C;
  for (C=this->Constraints.begin(); C != this->Constraints.end(); C++)
  {
   C->GetPars().SetInclusion(1);
   C->Run(X,fout);
   TW=C->GetTwin();
   TTest=C->GetPars().GetConsistency();
   if(TTest==itrue){}
   else if(TTest==ifalse)
   {Test=ifalse;
    break;}
   else
    {Test=iperhaps;}
  }
  return Test;
}//-----------------------------------------------------------------------------

ivalDb csp::LocalSearch(box &X,int NodeVarIndex,double Eps,fstream &fout)
{
 double Inf,Sup,Mid;
 ivalDb Outer1,Outer2;
 list<fstar>::iterator C;
 tree* T;
 C=this->Constraints.begin();
 T=&C->GetModel().GetTree();
 bool Solution=true;
 ivalDb Ival;

 C->GetModel().AssignXToNodeVars(X,false);
 T->AssignModes(false,C->GetPars());
 C->GetPars().SetOptim(3);
 C->Run(X,fout);
 ivalDb Inner=C->GetTwin().GetInn();
 if(!IsIn(Inner,Zero()))  //no existe
  {Solution=false;
   Ival.SetInf(NaN());
   Ival.SetSup(NaN());}

 else
 {
 C->GetModel().AssignXToNodeVars(X,true);
 C->GetPars().SetInclusion(1);
 Ival=T->GetNodeVars()[NodeVarIndex].GetIval();

 while(Width(Ival)>Eps)
 {
   Inf=Ival.GetInf();
   Sup=Ival.GetSup();
   Mid=Ival.GetMid();

   Ival=ivalDb(Inf,Mid);
   T->GetNodeVars()[NodeVarIndex].SetIval(Ival);
   C->GetPars().SetOptim(3);
   C->Run(X,fout);
   Outer1=C->GetTwin().GetOut();

   if(IsIn(Outer1,Zero())){ //existe

   } else
    {Ival=ivalDb(Mid,Sup);
     T->GetNodeVars()[NodeVarIndex].SetIval(Ival);}
 }

 }
 return Ival;
}//-----------------------------------------------------------------------------





