 /*!****************************************************************************
 *    \file csp.h
 *    This file contains the definition of the csp class belonging
 *    to the Star Algorithm.
 *    Date: 08/02/2005
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include <fstar.h>

#ifndef CSP_HH
#define CSP_HH

/*!*****************************************************************************
*    \class csp
*    \brief Class containig a set of csp used for the Star Algorithm.
*******************************************************************************/
class csp
{
 private:

 list<fstar> Constraints;
 list<fstar> VectConstraints;
 box X;

 public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

csp();
csp(csp &CSP);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~csp() {}

////////////////////////////////////////////////////////////////////////////////
// Copy Operator                                                                //
////////////////////////////////////////////////////////////////////////////////

csp& operator=(const csp &CSP);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

list<fstar>& GetConstraints(void);
list<fstar>& GetVectConstraints(void);
box& GetX(void);

void SetConstraints(list<fstar>&);
void SetX(box&);

iboolean CheckConsistency(box &X);
iboolean CheckQSIConsistency(box &X, int rule);
iboolean CheckQSIVectConsistency(box &X);
void Delete(void);
void Print(void);
void CreateModels(box &X,fstream &fout);
void CreateConstraints(box &X,fstream &fout);
iboolean CheckConsistency(void);
bool Contract(box &X,double Eps,fstream &fout,bool ModeAlgorithm,double &TimeOut);
iboolean Propagate(box &X,fstream &fout,double Eps,int &Counter);
ivalDb LocalSearch(box &X,int NodeVarIndex,double Eps,fstream &fout);

iboolean VectorialConsistency(box &X);


}; // End csp class

bool IsXInConstraint(variable& X, fstar& Constraint);

#endif

