

#include "boost/spirit/spirit.hpp"
#include <iostream>
#include <stack>
#include <functional>
#include <algorithm>

#include "repere.h"

#include "FSParser.h"
#include "plot.h"
//#include "squaltrack.h"
#include "Unit1.h"
#include "TCHAR.H"

#pragma hdrstop
#pragma argsused

////////////////////////////////////////////////////////////////////////////
using namespace std;
using namespace spirit;
////////////////////////////////////////////////////////////////////////////


stack<long> IntegerStack;
stack<double> FloatStack;
stack<string> SymbolStack;
list<string> CommandStack;
stack<ivalDb> IntervalStack;
stack<node*> NodeStack;
list<string> StringList;
//fstar Problem;
model Constraint;
tree* Tree;
rule<> command_line;
bool Parsed=true;
deque<node> NodeVars;
string ModeAlgorithm;
int AuxVarCount=0;
string OutputFile,InFile;
int NameVarCounter=0;
node GNode;
ifstream fin;                        // Input File stream
fstream fout;                       // Output File Stream
extern fstar *Problem;
int pos_fstar=0;
csp* CSP;
extern qsi QSI;
//squaltrack SQT;

repere Repere;
bool G_FirstSqtProblem=true;      // SQT Flag to kwow if it is the first SQT problem
int G_ProblemCounter=0;
////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////

void DeleteSpaces(string& Symbol)
{string::iterator c;
 int Spaces=0;
 for(c=Symbol.begin();c!=Symbol.end();c++)   // Delete front spaces
 {if(std::isspace(*c))
  {Spaces++;}
  else{break;}}
 if(Symbol.size()>0){Symbol.erase(1,Spaces);}

 Spaces=0;
 string::reverse_iterator rc;
 for(rc=Symbol.rbegin();rc!=Symbol.rend();rc++)
 {if(std::isspace(*rc))
  {Spaces++;}
  else{break;}}
 if(Symbol.size()>0){Symbol.erase(Symbol.size()-Spaces,Spaces);}
}//-----------------------------------------------------------------------------

struct save_command
{
  void operator()(char const* first, char const* last) const
    {
    string Symbol(first, last);
    CommandStack.push_back(Symbol);
  }
};
//-----------------------------------------------------------------------------

struct assign_mode
{
  void operator()(char const* first, char const* last) const
    {
    string Symbol(first, last);
    DeleteSpaces(Symbol);
    ModeAlgorithm=Symbol;
  }
};
//-----------------------------------------------------------------------------

struct push_int {

    void operator()(char const* str, char const* ) const
    {
        long n = strtol(str, 0, 10);
        IntegerStack.push(n);
        //cout << "push\t" << long(n) << endl;
    }
};//----------------------------------------------------------------------------

struct push_num {

    typedef double arg_type; //  Required

    void operator()(double n) const
    {
        FloatStack.push(n);
        //cout << "push\t" << n << endl;
    }
};//----------------------------------------------------------------------------

struct push_interval
{
    void operator()(char const*, char const*) const
    {
       double Sup=FloatStack.top();
       FloatStack.pop();
       double Inf=FloatStack.top();
       FloatStack.pop();
       ivalDb Interval=ivalDb(Inf,Sup);
       IntervalStack.push(Interval);
       //cout << "push\t" << Interval << endl;
    }
};//----------------------------------------------------------------------------

struct redecl_var
{
    void operator()(char const* , char const* , double& ) const
    {
        //cout << "Warning. You are attempting to re-declare a var.\n";
    }
};//----------------------------------------------------------------------------

struct push_symbol
{
  void operator()(char const* first, char const* last) const
    {
    string Symbol(first, last);
    DeleteSpaces(Symbol);
    SymbolStack.push(Symbol);
  }
};
//-----------------------------------------------------------------------------

struct store_string
{
  void operator()(char const* first, char const* last) const
    {
    string String(first,last);
    DeleteSpaces(String);
    StringList.push_back(String);
  }
};
//-----------------------------------------------------------------------------
/*
struct create_windows
{
 void operator()(char const*, char const*) const
  {
    int Win;
    int StackSize=IntegerStack.size();
    SQT.WinsVector.resize(StackSize);
    SQT.Fstars.resize(StackSize);
    G_ProblemCounter=0;
    while(!IntegerStack.empty())
    {StackSize--;
     Win=IntegerStack.top();
     SQT.WinsVector[StackSize]=Win;
     IntegerStack.pop();
    }
    Tree=&SQT.Fstars[0].GetModel().GetTree();    // Point the Tree To the first Fstar problem
  }
};//-----------------------------------------------------------------------------
*/
/*
struct create_measurement
{
    void operator()(char const*, char const*) const
    {
      string Name=SymbolStack.top();
      SymbolStack.pop();

      node Node(Infinity(),Name,"M");
      NodeVars.push_back(Node);

      if(G_FirstSqtProblem) {SQT.Data.NameVars.push_back(Name);}
      noise Noise;
      Noise.Absolute=FloatStack.top();
      FloatStack.pop();
      Noise.Relative=FloatStack.top();
      FloatStack.pop();
      if(G_FirstSqtProblem) {SQT.Data.Noise.push_back(Noise);}
  }
};//-----------------------------------------------------------------------------
*/
/*
struct create_state
{
    void operator()(char const*, char const*) const
    {
      string Name=SymbolStack.top();
      SymbolStack.pop();

      node Node(Infinity(),Name,"S");
      NodeVars.push_back(Node);

      if(G_FirstSqtProblem) {SQT.Data.NameVars.push_back(Name);}
      double RelativeNoise=FloatStack.top();
      noise Noise;
      Noise.Relative=RelativeNoise;
      FloatStack.pop();
      if(G_FirstSqtProblem) {SQT.Data.Noise.push_back(Noise);}
  }
};//-----------------------------------------------------------------------------
*/
struct create_parameter
{
    void operator()(char const*, char const*) const
    {
      string Name=SymbolStack.top();
      SymbolStack.pop();

      ivalDb Interval=IntervalStack.top();
      IntervalStack.pop();

      node Node(Interval,Name,"P");

      NodeVars.push_back(Node);
  }
};//-----------------------------------------------------------------------------

struct create_var
{
    void operator()(char const*, char const*) const
    {
      string Name=SymbolStack.top();
      SymbolStack.pop();

      ivalDb Interval=IntervalStack.top();
      IntervalStack.pop();

      node Node(Interval,Name);

      NodeVars.push_back(Node);
  }
};//-----------------------------------------------------------------------------

struct create_quantvar
{
    void operator()(char const*, char const*) const
    {
      string Name=SymbolStack.top();
      SymbolStack.pop();

      string Quantifier=SymbolStack.top();
      SymbolStack.pop();

      ivalDb Interval=IntervalStack.top();
      IntervalStack.pop();

      int Optim=Problem[pos_fstar].GetPars().GetOptim();

      if(Quantifier=="U") {Interval.MakeProp();}
      else if(Quantifier=="E") {Interval.MakeImpr();}
      else if(Quantifier=="F")
       {Interval.MakeProp();
        variable x=variable(0,Interval,Name);
        CSP->GetX().GetVars().push_back(x);
       }

      else
      {  int Optim=Problem[pos_fstar].GetPars().GetOptim();
         if(Optim==1 && Quantifier=="MIN") {Interval.MakeProp();}
         else if(Optim==1 && Quantifier=="MAX") {Interval.MakeImpr();}
         if(Optim==2 && Quantifier=="MIN") {Interval.MakeImpr();}
         else if(Optim==2 && Quantifier=="MAX") {Interval.MakeProp();}
       }

      node Node(Interval,Name);
      NodeVars.push_back(Node);

      //cout << "Create Quantified Var: " << Name << "=" << Interval << endl;
  }
};//-----------------------------------------------------------------------------

struct create_float {

    typedef double arg_type; //  Required

    void operator()(double n) const
    {
      node* Node=GNode.NewNode();
      Node->SetType("const");
      ivalDb Interval=ivalDb(n);
      Node->SetIval(Interval);
      Node->SetConst(n);
      NodeStack.push(Node);

      //cout << "Create Float: " << Interval << endl;
    }
};//-----------------------------------------------------------------------------

struct create_interval {

    void operator()(char const*, char const*) const
    {
      node* Node=GNode.NewNode();
      Node->SetType("var");
      char buffer[20];
      itoa(AuxVarCount,buffer,10);
      string NameVar= "Aux" + string(buffer);
      Node->SetNameVar(NameVar);
      ivalDb Interval=IntervalStack.top();
      IntervalStack.pop();
      Node->SetIval(Interval);
      Tree->AddNodeVar(*Node);
      incidence* Inc=NULL;
      Node->GetPVars()->back().push_back(Inc);
      Node->SetPInc(&Node->GetPVars()->back().back());
      NodeStack.push(Node);
    }
};//-----------------------------------------------------------------------------

struct create_incidence{

  void operator()(char const* first, char const* last, double n) const
  //void operator()(char const* first, char const* last) const
  {
     string Name(first, last);
     DeleteSpaces(Name);

     bool Match=false;

     node* N=GNode.NewNode();
     N->SetType("var");
     N->SetNameVar(Name);
     N->SetPNodeVars(&Tree->GetNodeVars());
     N->SetPVars(&Tree->GetVars());

     incidence* Inc=NULL;
     int i=0;
     while(i < Tree->GetNodeVars().size())       // if the variable is defined, create a new incidence
     {if(N->GetNameVar() == Tree->GetNodeVars()[i].GetNameVar())
        {Match=true;
         N->GetPVars()->operator[](i).push_back(Inc);
         N->SetPInc(&N->GetPVars()->operator[](i).back());
         break;}
      i++;
     }

     if(!Match)     // if the variable is not in the tree, look on the Node List
      {Match=false;
       deque<node>::iterator Node;
       for(Node=NodeVars.begin();Node!=NodeVars.end();Node++)
        {if(N->GetNameVar() == Node->GetNameVar())
          {Tree->AddNodeVar(*Node);
           N->SetQuantified(Node->GetQuantified());
           N->GetPVars()->operator[](i).push_back(Inc);
           N->SetPInc(&N->GetPVars()->operator[](i).back());
           Match=true;
           break;
          }
        }
       if(!Match)
       {node ANode(Infinity(),Name);
        Tree->AddNodeVar(ANode);
        N->GetPVars()->operator[](i).push_back(Inc);
        N->SetPInc(&N->GetPVars()->operator[](i).back());
       }
      }

    NodeStack.push(N);

   //cout << "Create Incidence: " << Name << endl;
  }
};//-----------------------------------------------------------------------------

struct create_operator
{
    void operator()(char const*, char const*) const
    {node* A;
     node* N=GNode.NewNode();
     string Opt=SymbolStack.top();
     SymbolStack.pop();
     N->SetType(Opt);
     N->SetRNode(NodeStack.top());
     NodeStack.pop();
     N->SetLNode(NodeStack.top());
     NodeStack.pop();
     NodeStack.push(N);
     //cout << Opt << "(" << N->GetLNode()->GetType() << "," << N->GetRNode()->GetType() << ")" << endl;
    }
};
//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////
struct create_negate
{
    void operator()(char const*, char const*) const
    {
     node* N=GNode.NewNode();
     N->SetType("minus");
     N->SetLNode(NodeStack.top());
     NodeStack.pop();
     N->SetRNode(NULL);
     NodeStack.push(N);
    }
};//-----------------------------------------------------------------------------

struct create_function {

   void operator()(char const*, char const*) const
    {
     node* N=GNode.NewNode();
     string Type=SymbolStack.top();
     SymbolStack.pop();
     N->SetType(Type);
     N->SetLNode(NodeStack.top());
     NodeStack.pop();
     N->SetRNode(NULL);
     NodeStack.push(N);
    }
};//-----------------------------------------------------------------------------

struct create_power {

   void operator()(char const*, char const*) const
    {
     node* N=GNode.NewNode();
     string Type="pow";
     N->SetType(Type);
     int Integer=IntegerStack.top();
     IntegerStack.pop();
     N->SetConst(Integer);
     N->SetLNode(NodeStack.top());
     NodeStack.pop();
     N->SetRNode(NULL);
     NodeStack.push(N);
    }
};//-----------------------------------------------------------------------------

struct assign_num_param {

   void operator()(char const*, char const*) const
    {
     string Symbol=SymbolStack.top();
     SymbolStack.pop();
     double Value=FloatStack.top();
     FloatStack.pop();

     if(Symbol=="Epsilon") Problem[pos_fstar].GetPars().SetEps(Value);
     else if(Symbol=="Tolerance") Problem[pos_fstar].GetPars().SetTol(Value);
     else if(Symbol=="Bisect") Problem[pos_fstar].GetPars().SetBisect(Value);
     else if(Symbol=="Debug") Problem[pos_fstar].GetPars().SetDebug(Value);
     else if(Symbol=="Optim") Problem[pos_fstar].GetPars().SetOptim(Value);
     else if(Symbol=="ScreenOut") Problem[pos_fstar].GetPars().SetScreenOut(Value);
     else if(Symbol=="Time") Problem[pos_fstar].GetPars().SetTime(Value);
     else if(Symbol=="Solution") Problem[pos_fstar].GetPars().SetSolution(Value);
     else if(Symbol=="Plot") Problem[pos_fstar].GetPars().SetPlot(Value);
     else if(Symbol=="Inclusion") Problem[pos_fstar].GetPars().SetInclusion(Value);
     else if(Symbol=="TOptimal") Problem[pos_fstar].GetModel().SetTOptimal(Value);
     else if(Symbol=="CSPEps") QSI.SetEpsilon(Value);
     else if(Symbol=="QSITime") QSI.SetTime(Value);
  //   else if(Symbol=="SampleTime") SQT.SampleTime=Value;

     //cout << "Assign Parameter: " << Symbol << "=" << Value << endl;
    }
};//-----------------------------------------------------------------------------

struct assign_sym_param {

   void operator()(char const*, char const*) const
    {
     string Value=SymbolStack.top();
     SymbolStack.pop();
     string Symbol=SymbolStack.top();
     SymbolStack.pop();

     if(Symbol=="PlotX") {QSI.SetPlotX(Value);}
     else if(Symbol=="PlotY") {QSI.SetPlotY(Value);}
//     else if(Symbol=="InFile") {Value=Value + ".dat"; SQT.Data.InFile=const_cast<char*> (Value.c_str());}
//     else if(Symbol=="OutFile") {Value=Value + ".dat"; SQT.Data.OutFile=const_cast<char*> (Value.c_str());}
    }
};//-----------------------------------------------------------------------------


struct new_constraint {

   void operator()(char const*, char const*) const
    {
     if(ModeAlgorithm=="MINMAX" || ModeAlgorithm=="FSTAR" || ModeAlgorithm=="LOGIC")
     {Problem[pos_fstar].GetModel().GetConstraints().push_back(Constraint);
      Tree=&Problem[pos_fstar].GetModel().GetConstraints().back().GetTree();}
     else if(ModeAlgorithm=="QSI" || ModeAlgorithm=="CONTRACTOR")
     {CSP->GetConstraints().push_back(Problem[pos_fstar]);
      Tree=&CSP->GetConstraints().back().GetModel().GetTree();}
    }
};//-----------------------------------------------------------------------------

struct assign_constraint {

   void operator()(char const*, char const*) const
    {
     if(!NodeStack.empty())
     {
     if(ModeAlgorithm=="MINMAX" || ModeAlgorithm=="FSTAR" || ModeAlgorithm=="LOGIC")
      {Problem[pos_fstar].GetModel().GetConstraints().back().GetTree().SetRoot(NodeStack.top());}
     else if(ModeAlgorithm=="QSI" || ModeAlgorithm=="CONTRACTOR")
      {CSP->GetConstraints().back().GetModel().GetTree().SetRoot(NodeStack.top());}
      NodeStack.pop();
    }

    }
};//-----------------------------------------------------------------------------

struct assign_function {

   void operator()(char const*, char const*) const
    {node* N;
     if(!NodeStack.empty())
     {N=NodeStack.top();
      Tree->SetRoot(N);
      NodeStack.pop();}
    }
};//-----------------------------------------------------------------------------

struct new_function {

   void operator()(char const*, char const*) const
    {
     if(ModeAlgorithm=="MINMAX" || ModeAlgorithm=="FSTAR" || ModeAlgorithm=="LOGIC")
     {Tree=&Problem[pos_fstar].GetModel().GetTree();}
     else if(ModeAlgorithm=="SQT")
     {G_FirstSqtProblem=false;}
    }
};//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////

void RunParser(char* File, fstar &Problem1)
{
    fstream fin,fout;
    Problem = new fstar[2];
    cout << "/////////////////////////////////////////////////////////\n\n";
    cout << "\t\t Modal Interval Solvers Parser\n\n";
    cout << "\t By Pau Herrero i Vias - MiceLab 2005.\n\n";
    cout << "/////////////////////////////////////////////////////////\n\n";

    //  Start grammar definition

    rule<> statement, var_assign1,var_assign2, var_decl, assignment, expression, term, factor;
    rule<> interval, functor, function,param,num_param_assign,sym_param_assign,integer,power,comment;
    rule<> blank_line,obj_constraint,obj_function, relation,relation_sym, obj_formula;
    rule<> beginning, termination, expression_symbols,mode_algorithm,var_assign_set,qcsp;
    rule<> logic_formula,connectives,constraint,constraints;
    rule<> measurement,parameter,state,windows;

    symbols<double> vars;

    NodeVars.clear();
    AuxVarCount=0;

    CSP=&QSI.GetCSP();

    factor      =   real_p[create_float()]
                |   vars[create_incidence()]
                |   interval[create_interval()]
                |   '(' >> expression >> ')'
                |   ('-' >> factor)[create_negate()]
                |   function[create_function()]
                |   power[create_power()];
    term        =   factor >>
                *(  (strlit<>("*")[push_symbol()] >> factor)[create_operator()]
                |   (strlit<>("/")[push_symbol()] >> factor)[create_operator()]
                );
    expression  =   term >>
                *(  (strlit<>("+")[push_symbol()] >> term)[create_operator()]
                |   (strlit<>("-")[push_symbol()] >> term)[create_operator()]
                );
    relation    = (expression >> relation_sym[push_symbol()] >> expression)[create_operator()];

    function    = functor[push_symbol()] >> '(' >>  expression >> ')';
    functor     = strlit<>("sin") | "Abs" | "cos" | "tan" | "exp" | "log" | "acos" | "asin" | "atan" | "sqrt" | "min" | "max";
    power       = strlit<>("pow") >> '(' >>  expression >> ',' >> integer >> ')';
    relation_sym  = strlit<>("=") | "<=" | ">=" | ">" | "<";
    var_decl    = lexeme[alpha >> *(alnum | '_')];
    interval    = ('[' >> real_p[push_num()] >> ',' >> real_p[push_num()] >> ']' >> !strlit<>("'"))[push_interval()];
    var_assign1 = (var_decl[vars.add][push_symbol()] >> ('=' | strlit<>("in"))  >> interval)[create_var()];
    var_assign2 = ((strlit<>("U") | "E" | "MIN" | "MAX" | "F")[push_symbol()] >> '(' >> var_decl[vars.add][push_symbol()] >> ',' >> interval  >> ')') [create_quantvar()];

    param       = strlit<>("Epsilon") | "InFile" | "OutFile" | "SampleTime" | "Tolerance" | "Bisect" | "Debug" | "Optim" | "PlotX" | "PlotY"
                  | "ScreenOut" | "Time" | "Solution" | "Plot" | "CSPEps" | "QSITime" | "Inclusion";
    num_param_assign= (param[push_symbol()] >> '=' >> real_p[push_num()])[assign_num_param()];
    sym_param_assign= (param[push_symbol()] >> '=' >> var_decl[push_symbol()])[assign_sym_param()];
    integer     = lexeme[ (!ch_p('-') >> +digit)[push_int()] ];
    comment     = '%' >> *(anychar);
    blank_line  = *(space);

    obj_function = (strlit<>("f:=")[new_function()] >> expression)[assign_function()][save_command()];

    obj_formula = (strlit<>("lf:=")[new_function()] >> relation)[assign_function()];
    obj_constraint   = (strlit<>("c:=")[new_constraint()] >> relation)[assign_constraint()];
    statement   =  (var_assign1 | var_assign2 | obj_formula | obj_function | obj_constraint | sym_param_assign |
    num_param_assign | mode_algorithm | measurement | state | parameter | windows) >> ch_p(';');
    command_line = (statement >> *(blank_line >> statement)) | comment | blank_line;
    mode_algorithm = strlit<>("Algorithm=") >> (strlit<>("QSI") | "SQT" | "FSTAR" | "LOGIC" | "CSP" | "MINMAX" | "MAXMIN" | "CONTRACTOR")[assign_mode()];

//    measurement = (strlit<>("Measurement") >> '(' >> var_decl[vars.add][push_symbol()] >> ',' >> real_p[push_num()] >> ',' >> real_p[push_num()] >> ')')[create_measurement()][save_command()];
//    state = (strlit<>("State") >> '(' >> var_decl[vars.add][push_symbol()] >> ',' >> real_p[push_num()]  >> ')')[create_state()][save_command()];
    parameter = (strlit<>("Parameter") >> '(' >> var_decl[vars.add][push_symbol()] >> ',' >> interval >> ')')[create_parameter()][save_command()];
//    windows = (strlit<>("Windows=") >> '{' >> +(integer >> !ch_p(',')) >> '}')[create_windows()];

    //  End grammar definition

     char buffer[256];
     string EveryThing;

     string::reverse_iterator EndString;
//     ShowMessage("inicio");
     InFile= string(File) + ".txt";

     ifstream InputFile(InFile.c_str());
     string Line,AuxLine;

     if (! InputFile.is_open())
     {cout << "Error opening file"; exit (1); }



     /*strcpy(buffer,"f:=[-97.6,-1.2]/[0.12,10];");

     cout << "Parser started" << buffer << "\n";


     if(parse(buffer, command_line, space).full)
              {cout << "Parsing Succeeded\n";
               cout << buffer << "\n";}
     */

     while (! InputFile.eof() && Parsed )
     {
          getline(InputFile,Line);

          DeleteSpaces(Line);

          Line=AuxLine+Line;

          strcpy(buffer, Line.c_str());

          if(Line.empty())
          {cout << "Blank Line\n";}
          else if(*Line.begin()=='%')
          {cout << "Comment \n";
           cout << buffer << "\n";}
          else
          {
            EndString=Line.rbegin()++;
            if(*EndString==';')
            {
              if(parse(buffer, command_line, space).full)
              {cout << "Parsing Succeeded\n";
               cout << buffer << "\n";}
              else
              {cout << "Syntax Error: \n\n";
               cout << buffer << "\n";
               fstream fout;
               OutputFile=string(File) + "Solution.txt";
               fout.open(OutputFile.c_str(),ios::out);
               fout << "Syntax Error: \n\n";
               fout << buffer << "\n";
               fout.close();
               Parsed=false;}
              AuxLine=string();
            }

            else
            {AuxLine=Line;}
          }
    } // End While


    fin.close();

    if(Parsed)
    {
      box X(2);
      ivalDb XAxis=ivalDb(0);
      ivalDb YAxis=ivalDb(0);

      if(ModeAlgorithm=="FSTAR" || ModeAlgorithm=="MINMAX" || ModeAlgorithm=="LOGIC")
      {model* Model=&Problem[pos_fstar].GetModel();
       Tree=&Model->GetTree();
       cell* Cell=&Model->GetCell();
       Problem[pos_fstar].GetPars().SetPlot(true);
       Problem[pos_fstar].CreateModel(X,fout);

      }

      else if(ModeAlgorithm=="QSI")
      {

       QSI.AssignPlotIndexes(CSP->GetX());
       XAxis=Prop(CSP->GetX()[CSP->GetX().GetXIndex()].GetIval());
       if(CSP->GetX().GetVars().size()>1)
       {YAxis=Prop(CSP->GetX()[CSP->GetX().GetYIndex()].GetIval());}
       else
       {YAxis=ivalDb(-1,1);}
       X=CSP->GetX();
       repere R1(Form1,XAxis.GetInf(),XAxis.GetSup(),YAxis.GetInf(),YAxis.GetSup());
       X[0].SetIval(XAxis);
       X[1].SetIval(YAxis);
       R1.DrawBox(X,clWhite,clWhite,bsSolid);
       Repere=R1;}

      // Execution

      if(ModeAlgorithm=="FSTAR")
      {Problem[pos_fstar].GetPars().SetPlot(false);
       Problem[pos_fstar].GetPars().SetCells(false);
       Problem[pos_fstar].GetPars().SetFile(File);
       Problem[pos_fstar].GetPars().SetOptim(0);
       Problem[pos_fstar].GetPars().SetDebug(0);       
       Problem[pos_fstar].GetPars().SetSolution(0);
//       Problem[pos_fstar].GetPars().SetTime(1);

/*       Problem.GetModel().GetTree().GetNodeVars()[0].SetIval(ivalDb(-1,1));
       Problem.GetModel().GetTree().GetNodeVars()[1].SetIval(ivalDb(-2,2));
       Problem.GetModel().GetTree().GetNodeVars()[2].SetIval(ivalDb(-3,3));
       */
       //Problem1=Problem[pos_fstar];
       //Tree->DeleteTree();
//       Problem[pos_fstar].Run(X,fout);
//       delete [] Problem;
       //Problem[pos_fstar].Delete();
       }



      else if(ModeAlgorithm=="QSI")
      {
       CSP->CreateModels(X,fout);
       CSP->Print();
       QSI.GetList().push_back(X);
       QSI.Run(File,Repere);
       //R.ConvertToBitmap(Form1->Image1,File);
       Repere.ConvertToBitmap(Form1,File);
       }


    }

//  OutputFile=string(File) + "Solution.txt";
//  ShellExecute(NULL, "open",OutputFile.c_str(), "", "", SW_SHOWDEFAULT);
//  OutputFile=string(File) + "Image.bmp";
//  ShellExecute(NULL, "open",OutputFile.c_str(), "", "", SW_SHOWDEFAULT);

}//-----------------------------------------------------------------------------



    /*
    if(parse(buffer, blank_line, space).full){}
    else if(parse(buffer, comment, space).full){}
    else
    {EveryThing=EveryThing+buffer;}
    */

    /*
    if(parse(EveryThing.c_str(),command_line, space).full)
    {Parsed=true;
     cout << "parsing succeeded\n";}
    else
    {
     cout << "parsing failed\n\n";
     fstream fout;
     OutputFile=string(File) + "Solution.txt";
     fout << "parsing failed\n\n";
     fout.close();
     Parsed=false;
    }
    */

      /*
    constraint   = relation[assign_constraint()];
    constraints =  ch_p('{') >> constraint >> *(connectives >> constraint) >> ch_p('}');
    qcsp= *(var_assign2) >> constraints;
    logic_formula = *(var_assign2) >> constraint;
    connectives = strlit<>("") | "&" | "|" | ",";
    */

