 /*!****************************************************************************
 *    \file qsi.cpp
 *    This file contains the implementation of the QSI class
 *    Date: 15/02/2005
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "qsi.h"
//#include "functions.h"
#include "matrix.h"
#include "Unit1.h"

bool GlobalStop;
extern box Lbox[10000];
extern int maxbox;
extern double x0;
extern double y0;
extern double z0;
extern vector< vector<int> > buffer;
extern double factor;
extern ivalDb screen;
extern int box_in;
extern int box_out;
extern int pixel_in;
extern int pixel_out;
extern matrix mv;
extern double par1,par2,par3,par4;

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

qsi::qsi()
{
 this->Epsilon=0.1;
 this->OnLinePlot=true;
 Time=PInfinity();
}//-----------------------------------------------------------------------------

qsi::qsi(box& X,double Eps)
{
 this->List.push_back(X);
 this->Epsilons.resize(X.GetVars().size());
 for(int i=0;i<this->Epsilons.size();i++)
 {this->Epsilons[i]=Eps;}
 this->Epsilon=Eps;
 Time=PInfinity();
}//-----------------------------------------------------------------------------

qsi::qsi(qsi &QSI)
{
 this->CSP=QSI.CSP;
 this->List=QSI.List;
 this->TList=QSI.TList;
 this->FList=QSI.FList;
 this->UList=QSI.UList;
 this->Epsilons.resize(QSI.Epsilons.size());
 this->Epsilons=QSI.Epsilons;
 this->Epsilon=QSI.Epsilon;
 this->Time=QSI.Time;
 PlotX=QSI.PlotX;
 PlotY=QSI.PlotY;
}//-----------------------------------------------------------------------------

csp& qsi::GetCSP(void)
{
 return CSP;
}//-----------------------------------------------------------------------------

list<box>& qsi::GetList(void)
{
 return List;
}//-----------------------------------------------------------------------------

list<box>& qsi::GetTList(void)
{
 return TList;
}//-----------------------------------------------------------------------------

list<box>& qsi::GetFList(void)
{
 return FList;
}//-----------------------------------------------------------------------------

list<box>& qsi::GetUList(void)
{
 return this->UList;
}//-----------------------------------------------------------------------------

vector<double>& qsi::GetEpsilons(void)
{
 return this->Epsilons;
}//-----------------------------------------------------------------------------

double qsi::GetEpsilon(void)
{
 return this->Epsilon;
}//-----------------------------------------------------------------------------

bool qsi::GetOnLinePlot(void)
{
 return this->OnLinePlot;
}//-----------------------------------------------------------------------------

double qsi::GetTime(void)
{return this->Time;
}//-----------------------------------------------------------------------------

string& qsi::GetPlotX(void)
{return PlotX;
}//-----------------------------------------------------------------------------

string& qsi::GetPlotY(void)
{return PlotY;
}//-----------------------------------------------------------------------------

void qsi::SetEpsilon(double Eps)
{
 this->Epsilon=Eps;
}//-----------------------------------------------------------------------------

void qsi::SetOnLinePlot(bool OLP)
{
 this->OnLinePlot=OLP;
}//-----------------------------------------------------------------------------

void qsi::SetTime(double T)
{
 this->Time=T;
}//-----------------------------------------------------------------------------

void qsi::SetPlotX(string& X)
{PlotX=X;
}//-----------------------------------------------------------------------------

void qsi::SetPlotY(string& Y)
{PlotY=Y;
}//-----------------------------------------------------------------------------

int gencol;

ivalDb traceint(ivalDb x, ivalDb y, ivalDb t1) {
     double z,Inf,Sup,Mid;
     ivalDb t,approx,approx1,approx2;
     ivalDb xa,ya,za;
     list<ivalDb> T;
//     ivalDb screen=ivalDb(-2,2);
    // x0=0.1;y0=0.1;z0=20;
//         x0=-20;y0=0.1;z0=8;
     z=3;
     int size=300,x1,y1,x2,y2;
//     vect l1;
     T.clear();
     t=ivalDb(0,30);
//     t=t1;
/*     if (t.GetInf()!=0) {
          ShowMessage("menor");
     }
*/
     T.push_back(t);
//     xp=x*(Width(screen)/size)+screen.GetInf();
//     yp=y*(Width(screen)/size)+screen.GetInf();
//     zp=z;
     xa=x;
     ya=y;
     za=z;

/*        a.mat[0][0]=x;
        a.mat[1][0]=y;
        a.mat[2][0]=-17;
        a.mat[3][0]=1;
*/
        xa=mv.mat[0][0]*x+mv.mat[0][1]*y+mv.mat[0][2]*(-17)+mv.mat[0][3];
        ya=mv.mat[1][0]*x+mv.mat[1][1]*y+mv.mat[1][2]*(-17)+mv.mat[1][3];
        za=mv.mat[2][0]*x+mv.mat[2][1]*y+mv.mat[2][2]*(-17)+mv.mat[2][3];

/*        b=mv*a;
        xa=b.mat[0][0];
        ya=b.mat[1][0];
        za=b.mat[2][0];
*/


/*    x1=(xa.GetInf()-screen.GetInf())*(size/Width(screen));
    y1=size-(ya.GetInf()-screen.GetInf())*(size/Width(screen));
    x2=(xa.GetSup()-screen.GetInf())*(size/Width(screen));
    y2=size-(ya.GetSup()-screen.GetInf())*(size/Width(screen));
*/
     do {
               t=T.back();

               T.pop_back();

/*               approx1=pow(pow(x0+(xa-x0)*t,2)*pow(y0+(ya-y0)*t,2)+
                      pow(y0+(ya-y0)*t,2)*pow(z0+(za-z0)*t,2)+
                      pow(z0+(za-z0)*t,2)*pow(x0+(xa-x0)*t,2),2)+
                      (x0+(xa-x0)*t)*(y0+(ya-y0)*t)*(z0+(za-z0)*t);
*/
              //strange 4 spikes 45 degree
/*               approx1=(pow(x0+(xa-x0)*t,2)+3)*
                      (pow(y0+(ya-y0)*t,2)+3)*
                      (pow(z0+(za-z0)*t,2)+3)-32*
                      ((x0+(xa-x0)*t)*(y0+(ya-y0)*t)*(z0+(za-z0)*t)+1);
*/
               //sphere
//               approx1=pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)-1.5;

//             crosscap
//               approx1=4*pow(x0+(xa-x0)*t,2)*(pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)+(z0+(za-z0)*t))+pow(y0+(ya-y0)*t,2)*(pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)-1);

//             mitchell
//               approx1=4*(pow(x0+(xa-x0)*t,4)+pow(pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2),2))+17*pow(x0+(xa-x0)*t,2)*(pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2))-20*(pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2))+17;
//               approx1=pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)-1;

               //blobby
//               approx1=pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)+sin(4*(x0+(xa-x0)*t))+sin(4*(y0+(ya-y0)*t))+sin(4*(z0+(za-z0)*t))-1;

               //bath
               approx1=4*(2.618*pow(x0+(xa-x0)*t,2)-pow(y0+(ya-y0)*t,2))*(2.618*pow(y0+(ya-y0)*t,2)-pow(z0+(za-z0)*t,2))*(2.618*pow(z0+(za-z0)*t,2)-
                       pow(x0+(xa-x0)*t,2))-4.236*pow(pow(x0+(xa-x0)*t,2)+pow(y0+(ya-y0)*t,2)+pow(z0+(za-z0)*t,2)-1,2);

               //deformations
               //def 1: x^2 - y^2 - z^2 = a
//               approx1=pow(x0+(xa-x0)*t,2)-pow(y0+(ya-y0)*t,2)-pow(z0+(za-z0)*t,2)-par1;
               //def 2: x^2 y - y^3 - z^2 + a x^2 + b y^2 + c (x^2-y^2) + d y z = 0
//               approx1=pow(x0+(xa-x0)*t,2)*(y0+(ya-y0)*t)-pow(y0+(ya-y0)*t,3)-pow(z0+(za-z0)*t,2)+par1*pow(x0+(xa-x0)*t,2)+par2*pow(y0+(ya-y0)*t,2)+par3*(pow(x0+(xa-x0)*t,2)-pow(y0+(ya-y0)*t,2))+par4*(y0+(ya-y0)*t)*(z0+(za-z0)*t);

//               if (!IsIn(Zero(),approx1) || !IsIn(Zero(),approx2)) {
                if (!IsIn(Zero(),approx1)) {

                } else {
                    //if (x==50 && y==50) {
                    //     approx1=PutCoords(xa,ya,za,t);
                    //}

                    Inf=t.GetInf();
                    Sup=t.GetSup();
                    Mid=t.GetMid();
                    T.push_back(ivalDb(Mid,Sup));
                    T.push_back(ivalDb(Inf,Mid));
                }
                double temp=Width(T.back());
     } while (!T.empty() && Width(T.back()) > 0.001);
     if (!T.empty()) {
          ivalDb ta=ivalDb(T.back().GetInf(),T.front().GetSup());
          return ta;
     } else {
          return ivalDb(PInfinity(),PInfinity());
     }

}


void qsi::Run(string& File, repere &R)
{

  fstream fout;
  int PCounter=0;
  iboolean Consistency,Consist;
  int DebugCounter=0;
  int p1,p2,p3,p4,i,j;

  list<box>::iterator X=this->List.begin();

  box AUX(*X);

  this->CSP.CreateModels(*X,fout);

//  this->CSP.Print();
//  this->GetCSP().GetConstraints().front().GetModel().GetTree().ToLatex("TexTree.tex");

  this->TList.clear();
  this->FList.clear();
  this->UList.clear();

  this->AssignPlotIndexes(*X);
  this->AssignRelativeEps(*X);

  clock_t Start = clock();
  clock_t Stop;
  double PTime=0;

//  ivalDb C1a,C2a,C1b,C2b;

//  int NodeVarIndex=this->CSP.GetConstraints().begin()->GetModel().GetTree().GetNodeVarIndex("ROs");
//  ivalDb Ival=this->CSP.LocalSearch(*X,NodeVarIndex,0.00001,fout);

  Stop = clock();
  PTime=(Stop-Start)/CLK_TCK;

/*  (*X).p[0][0]=trace((*X)[0].GetIval().GetInf(),(*X)[1].GetIval().GetInf());
  (*X).p[1][0]=trace((*X)[0].GetIval().GetSup(),(*X)[1].GetIval().GetInf());
  (*X).p[0][1]=trace((*X)[0].GetIval().GetInf(),(*X)[1].GetIval().GetSup());
  (*X).p[1][1]=trace((*X)[0].GetIval().GetSup(),(*X)[1].GetIval().GetSup());
*/

  (*X).t=ivalDb(0,30);
  for (int i=1;i<128;i++) {
       X=this->List.begin();
       this->Bisect(X,R);
//       Application->ProcessMessages();
  }
  maxbox=0;
  while(!this->List.empty() && PTime<this->Time )
  {
   /*
   fout.open("DebugCounter.txt",ios::out);
   fout << DebugCounter << endl;
   fout.close();
   */

//   if(DebugCounter==21)
//   {GlobalStop=true;}

    DebugCounter++;

    X=this->List.begin();

/*
    p1=(*X).p[0][0];
    p2=(*X).p[1][0];
    p3=(*X).p[0][1];
    p4=(*X).p[1][1];
*/


    ivalDb res=traceint((*X)[0].GetIval(),(*X)[1].GetIval(),(*X).t);


    if(Consist==ifalse)
     {this->List.pop_front();}

    else
    {
////////////////////////////////
//      if (p1==2 && p2==2 && p3==2 && p4==2) {
      if (IsInfinity( res.GetInf() ) ) {
///////////////////////////////

//          Consistency=this->CSP.CheckQSIConsistency(*X,2);
//         ivalDb res=traceint((*X)[0].GetIval(),(*X)[1].GetIval(),(*X).t);
//         if (IsInfinity( res.GetInf() ) ) {
               Consistency=ifalse;
//         }


//      } else
//      if (p1==1 && p2==1 && p3==1 && p4==1) {
          //Consistency=this->CSP.CheckQSIConsistency(*X,1);
//          Consistency=itrue;
      } else {
          (*X).t=res;
//          if(this->IsEpsSmall(*X))
//               Consistency=this->CSP.CheckQSIConsistency(*X,2);
 //         else
               Consistency=iperhaps;
      }

      //Consistency=this->CSP.CheckQSIVectConsistency(*X);

      if(Consistency==itrue)
        {if(this->OnLinePlot) {
//               if (gencol==0)
                    R.DrawBox(*X,clRed,clBlack,bsSolid);
//               else
//                    R.DrawBox(*X,clGray,clBlack,bsSolid);
        }
         this->TList.splice(this->TList.end(),this->List,X);}

      else if(Consistency==ifalse)
        {if(this->OnLinePlot) {
//               if (gencol==0)
                    R.DrawBox(*X,clBlue,clBlack,bsSolid);
                    box_out++;
//               else
//                    R.DrawBox(*X,clGreen,clBlack,bsSolid);
         }
         this->FList.splice(this->FList.end(),this->List,X);}

      else if(this->IsEpsSmall(*X))
        {if(this->OnLinePlot) {
//               if (gencol==0)
                    R.DrawBox(*X,clYellow,clBlack,bsSolid);
                    box_in++;
//               else
//                    R.DrawBox(*X,clGray,clBlack,bsSolid);
         }
         this->UList.splice(this->UList.end(),this->List,X);
         //Lbox[maxbox++]=(*X);
         ivalDb temp=(*X)[0].GetIval();
         p1=((*X)[0].GetIval().GetInf()-screen.GetInf())*factor;
         p3=((*X)[1].GetIval().GetInf()-screen.GetInf())*factor;
         p2=((*X)[0].GetIval().GetSup()-screen.GetInf())*factor;
         p4=((*X)[1].GetIval().GetSup()-screen.GetInf())*factor;
//         if (p1>=0&& p3>=0 && p2<=300 && p3<=300)
         for (i=p1;i<=p2;i++)
              for (j=p3;j<=p4;j++)
                    buffer[i][j]=1;

         }

      else  {this->Bisect(X,R);}
    }



  Stop = clock();
  PTime=(Stop-Start)/CLK_TCK;

  if(PTime>this->Time)
  {break;}

  } // End While

  //this->Print(File,PTime);

  //this->PlotPolarDiagram();

  //this->CSP.Delete();
}//-----------------------------------------------------------------------------

//Bisect Box
void qsi::Bisect(list<box>::iterator X,repere& R)
{
 int Dim=0;
 double width=MInfinity();
 double Length;
 double Eps;
 int Dim1;
 double p1,p2,p3,p4,p1a,p2a,p3a,p4a,p1b,p2b,p3b,p4b;

 for(int i=0;i<X->GetVars().size();i++)
  {Length=Width(X->operator [](i).GetIval());
   if(this->Epsilons.size()>0)
    {Eps=this->Epsilons[i];}
   else {Eps=this->Epsilon;}
   if(Length >=width && Length > Eps) {Dim=i; width=Length;}
  }

 Dim1=1-Dim;
 ivalDb Domain=X->operator [](Dim).GetIval();
 double Inf=Domain.GetInf();
 double Sup=Domain.GetSup();
 double Middle = Center(Domain);

 X->operator [](Dim).SetIval(Ival(Inf,Middle));
 /////////
 /*p1a=(*X).p[0][0];
 p2a=(*X).p[1][0];
 p3a=(*X).p[0][1];
 p4a=(*X).p[1][1];

 if (Dim==0) {
     p1=trace((*X)[Dim].GetIval().GetSup(),(*X)[Dim1].GetIval().GetInf());
     p2=trace((*X)[Dim].GetIval().GetSup(),(*X)[Dim1].GetIval().GetSup());
 }
 if (Dim==1) {
     p1=trace((*X)[Dim].GetIval().GetInf(),(*X)[Dim1].GetIval().GetSup());
     p2=trace((*X)[Dim].GetIval().GetSup(),(*X)[Dim1].GetIval().GetSup());
 }

 if (Dim==0) {
     (*X).p[0][0]=p1a;
     (*X).p[1][0]=p1;
     (*X).p[0][1]=p3a;
     (*X).p[1][1]=p2;
 } else {
     (*X).p[0][0]=p1a;
     (*X).p[1][0]=p2a;
     (*X).p[0][1]=p1;
     (*X).p[1][1]=p2;
 }

/* p1b=(*X).p[0][0];
 p2b=(*X).p[1][0];
 p3b=(*X).p[0][1];
 p4b=(*X).p[1][1];
*/

 /////////
 this->List.push_back(*X);

 if(this->OnLinePlot) {R.DrawCadre(*X,clBlack,psSolid,1);}
 /////////
/* if (Dim==0) {
     (*X).p[0][0]=p1;
     (*X).p[1][0]=p2a;
     (*X).p[0][1]=p2;
     (*X).p[1][1]=p4a;
 } else {
     (*X).p[0][0]=p1;
     (*X).p[1][0]=p2;
     (*X).p[0][1]=p3a;
     (*X).p[1][1]=p4a;
 }

/* p1b=(*X).p[0][0];
 p2b=(*X).p[1][0];
 p3b=(*X).p[0][1];
 p4b=(*X).p[1][1];
*/
 /////////
 X->operator [](Dim).SetIval(Ival(Middle,Sup));
 this->List.splice(this->List.end(),this->List,X);
 if(this->OnLinePlot) {R.DrawCadre(*X,clBlack,psSolid,1);}
}//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Is Smaller than Epsilon
//------------------------------------------------------------------------------
bool qsi::IsEpsSmall(box &X)
{
 bool IsEps=true;
 for(int i=0;i<X.GetVars().size();i++)
 {
  if(this->Epsilons.size()>0)
  {if (Width(X[i].GetIval())>this->Epsilons[i]){IsEps=false; break;}}
  else
  {if(Width(X[i].GetIval())>this->Epsilon){IsEps=false;}}
 }
 return IsEps;
}//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Print Solution
//------------------------------------------------------------------------------
void qsi::Print(string& File,double Time)
{
 fstream fout;
 list<box>::iterator Box;

 fout.precision(4);

 string SFile=File + "Solution.txt";

 fout.open(SFile.c_str(),ios::out);

 fout << "Solution File:" << "\n\n";

 fout << "Computation Time:" << Time << "\n\n";

 if(!this->TList.empty())
 {
 box B(*this->TList.begin());
 this->JoinTList(B);
 fout << "Solution Hull:" << "\n\n";
 B.Print(fout);
 }

 fout << "\n\n" << "Solution Boxes:" << "\n\n";

  for (Box=this->TList.begin(); Box != this->TList.end(); Box++)
    {Box->Print(fout);}

 fout << "\n\n" << "Non Solution Boxes:" << "\n\n";
  for (Box=this->FList.begin(); Box != this->FList.end(); Box++)
    {Box->Print(fout);}

 fout << "\n\n" << "Undefined Boxes:" << "\n\n";
  for (Box=this->UList.begin(); Box != this->UList.end(); Box++)
    {Box->Print(fout);}

  fout.close();
}//-----------------------------------------------------------------------------

void qsi::PlotPolarDiagram(void)
{
 fstream fout;
 fout.precision(4);
 fout.open("PolarPlot.ps",ios::out);

 fout << "%!" << endl;

 fout << "0.01 setlinewidth"  << endl;

 bool Symmetry=false;
 this->PlotPolarDiagram(fout,Symmetry);
 Symmetry=true;
 this->PlotPolarDiagram(fout,Symmetry);

 fout << "showpage" << endl;
 fout.close();
}//-----------------------------------------------------------------------------

void PlotPolar(box& X,fstream& fout,bool Symmetry)
{
 ivalDb Teta=X[0].GetIval();
 ivalDb v=X[1].GetIval();

 if(Symmetry) Teta=3.1416-Teta;

 double x0=100;
 double y0=100;

 double TetaInf=Teta.Inf*360/(2*3.14159);
 double TetaSup=Teta.Sup*360/(2*3.14159);

 double x2=v.Inf*FDLIBM::cos(Teta.Sup)+x0;
 double y2=v.Inf*FDLIBM::sin(Teta.Sup)+y0;
 double x3=v.Sup*FDLIBM::cos(Teta.Inf)+x0;
 double y3=v.Sup*FDLIBM::sin(Teta.Inf)+y0;

 fout << endl;
 fout << x2 << "\t" << y2 << "\t" << "moveto" << endl;
 fout << x0 << "\t" << y0 << "\t" << v.Inf << "\t" << TetaSup << "\t" << TetaInf << "\t" << "arcn" << endl;
 fout << x3 << "\t" << y3 << "\t" << "lineto" << endl;
 fout << x0 << "\t" << y0 << "\t" << v.Sup << "\t" << TetaInf << "\t" << TetaSup << "\t" << "arc" << endl;
 fout << "closepath" << endl;


}//-----------------------------------------------------------------------------

//------------------------------------------------------------------------------
// Plot Polar Chart
//------------------------------------------------------------------------------
void qsi::PlotPolarDiagram(fstream &fout,bool Symmetry)
{
 list<box>::iterator Box;

 if(Symmetry)  {fout << endl << "%Symmetry:" << endl << endl;}

 fout << endl << "%Solution Boxes:" << endl << endl;

 for (Box=this->TList.begin(); Box != this->TList.end(); Box++)
 {
  //fout << "1 0 0 setrgbcolor" << endl << endl;
  fout << "1 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "fill" << endl;
  fout << "0 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "stroke" << endl;
  }

 fout << endl << endl << "%Non Solution Boxes:" << endl << endl;

 for (Box=this->FList.begin(); Box != this->FList.end(); Box++)
 {
  //fout << "0 1 1 setrgbcolor" << endl << endl;
  fout << "0.5 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "fill" << endl;
  fout << "0 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "stroke" << endl;
 }

 fout << endl << endl << "%Undefined Boxes:" << endl << endl;

 for (Box=this->UList.begin(); Box != this->UList.end(); Box++)
 {
  //fout << "1 1 0 setrgbcolor" << endl << endl;
  fout << "0 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "fill" << endl;
  fout << "0 setgray" << endl << endl;
  PlotPolar(*Box,fout,Symmetry);
  fout << "stroke" << endl;
 }

}//-----------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Join Solutions
//------------------------------------------------------------------------------
void qsi::JoinTList(box &B)
{
  list<box>::iterator Box;
  B=*this->TList.begin();
  for (Box=this->TList.begin(); Box != this->TList.end(); Box++)
  {B = B || *Box;}
}//-----------------------------------------------------------------------------


//------------------------------------------------------------------------------
// Plot Solution
//------------------------------------------------------------------------------
void qsi::Plot(repere &R)
{
 list<box>::iterator Box;

  for (Box=this->TList.begin(); Box != this->TList.end(); Box++)
    {R.DrawBox(*Box,clRed,clBlack,bsSolid);}

  for (Box=this->FList.begin(); Box != this->FList.end(); Box++)
    {R.DrawBox(*Box,clBlue,clBlack,bsSolid);}

  for (Box=this->UList.begin(); Box != this->UList.end(); Box++)
    {R.DrawBox(*Box,clYellow,clBlack,bsSolid);}
}//-----------------------------------------------------------------------------

void qsi::AssignPlotIndexes(box& X)
{
 for(int i=0;i<X.GetVars().size();i++)
 {if(this->PlotX==X.GetVars()[i].GetName())
  {X.SetXIndex(i);
   break;}}
 for(int i=0;i<X.GetVars().size();i++)
 {if(this->PlotY==X.GetVars()[i].GetName())
  {X.SetYIndex(i);
  break;}}
}//-----------------------------------------------------------------------------

void qsi::AssignRelativeEps(box& X)
{
 this->Epsilons.resize(X.GetVars().size());
 for(int i=0;i<X.GetVars().size();i++)
 {this->Epsilons[i]=Width(X.GetVars()[i].GetIval())*this->Epsilon;}
}//-----------------------------------------------------------------------------
