 /*!****************************************************************************
 *    \file qsi.h
 *    This file contains the definition of the QSI clas belonging
 *    Date: 15/02/2005
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include <time.h>
#include "repere.h"
#include "csp.h"

#ifndef QSI_HH
#define QSI_HH

/*!*****************************************************************************
*    \class qsi
*    \brief Class containig a set of qsi used for the Star Algorithm.
*******************************************************************************/
class qsi
{
 private:

 csp CSP;                     // Constraint Satisfaction Problem
 list<box> List;              // Main Box List
 list<box> TList;             // True Box List
 list<box> FList;             // False Box List
 list<box> UList;             // Undefined Box List
 double Epsilon;              // Epsilon
 vector<double> Epsilons;     // Epsilons' Vector
 bool OnLinePlot;             // On-line Plot flag
 double Time;                 // Time to do the computations
 string PlotX;
 string PlotY;

 public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

qsi();
qsi(box& X,double Eps);
qsi(qsi &QSI);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~qsi() {}

////////////////////////////////////////////////////////////////////////////////
// Copy Operator                                                                //
////////////////////////////////////////////////////////////////////////////////

qsi& operator=(const qsi &QSI);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

csp& GetCSP(void);
list<box>& GetList(void);
list<box>& GetTList(void);
list<box>& GetFList(void);
list<box>& GetUList(void);
vector<double>& GetEpsilons(void);
double GetEpsilon(void);
bool GetOnLinePlot(void);
double GetTime(void);
string& GetPlotX(void);
string& GetPlotY(void);

void SetEpsilon(double);
void SetOnLinePlot(bool);
void SetTime(double);
void SetPlotX(string&);
void SetPlotY(string&);

void Run(string& File,repere& R);
void Bisect(list<box>::iterator X,repere& R);
bool IsEpsSmall(box& X);
void Print(string& ,double);
void Plot(repere &R);
void AssignPlotIndexes(box& X);
void AssignRelativeEps(box& X);
void JoinTList(box &B);

void PlotPolarDiagram(void);
void PlotPolarDiagram(fstream& fout,bool Symmetry);

}; // End qsi class

#endif

 