 /*!****************************************************************************
 *    \file cell.cpp
 *    This file contains the mplementation of the cell class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "cell.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
   \fn cell::cell(void)*/
cell::cell(void)
{XpStrip=NULL;
 XiDim=-1;
 Twin=twin();
 Consistent=itrue;
 TOptimal=false;
 ZeroIncluded=false;
 ZeroExcluded=false;
 //Penalty=0;
}//-----------------------------------------------------------------------------

/** Constructor 1.
    \fn cell::cell(vector<variable>& PV,vector<variable>& IV)
    \param PV Proper varibles vector.
    \param IV Imroper varibles vector. */
cell::cell(vector<variable>& PV,vector<variable>& IV)
{Xi.resize(IV.size());
 Xp.resize(PV.size());
 Xi=IV;
 Xp=PV;
 XpStrip=NULL;
 XiDim=-1;
 Twin=twin();
 Consistent=itrue;
 TOptimal=false;
 //Penalty=0;
 ZeroIncluded=false;
 ZeroExcluded=false;
}//-----------------------------------------------------------------------------

/** Copy Constructor.
    \fn cell::cell(const cell& Cell)*/
cell::cell(const cell& Cell)
{Xi.resize(Cell.Xi.size());
 Xi=Cell.Xi;
 Xp.resize(Cell.Xp.size());
 Xp=Cell.Xp;
 XpStrip=Cell.XpStrip;
 XiDim=Cell.XiDim;
 Twin=Cell.Twin;
 Consistent=Cell.Consistent;
 TOptimal=Cell.TOptimal;
 ZeroIncluded=false;
 ZeroExcluded=false;
 //Penalty=0;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator                                                        //
////////////////////////////////////////////////////////////////////////////////

/** \fn cell::cell(const cell& Cell)
    Overloaded Access Operator([])*/
variable& cell::operator[] (int i)
{
 int PropSize;
 
 bool LocalXp;
 if(Xp.size()>0)
  {PropSize=Xp.size();
   LocalXp=true;}
 else
  {if(XpStrip!=NULL){PropSize=XpStrip->size();}
   else{PropSize=0;}
   LocalXp=false;}
 if(i< PropSize)
  {
   if(LocalXp) {Xp[i];}
   else {return XpStrip->operator [](i);}
  }
 else
 {int j=i-PropSize;
  return Xi[j];}
}//-----------------------------------------------------------------------------

/** \fn cell& cell::operator=(const cell &Cell)
    Overloaded Assignation Operator(=)*/
cell& cell::operator=(const cell &Cell)
{Xi.resize(Cell.Xi.size());
 Xi=Cell.Xi;
 Xp.resize(Cell.Xp.size());
 Xp=Cell.Xp;
 XpStrip=Cell.XpStrip;
 XiDim=Cell.XiDim;
 Twin=Cell.Twin;
 Consistent=Cell.Consistent;
 TOptimal=Cell.TOptimal;
 ZeroIncluded=false;
 ZeroExcluded=false;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn vector<variable>& cell::GetXi()
    \return Improper Vars*/
vector<variable>& cell::GetXi()
{return Xi;
}//-----------------------------------------------------------------------------

/** \fn vector<variable>& cell::GetXp()
    \return Proper variables Vector*/
vector<variable>& cell::GetXp()
{
 //if(XpStrip!=NULL) return *XpStrip;
 //else
 return Xp;
}//-----------------------------------------------------------------------------

/** \fn vector<variable>& cell::GetXpStrip()
    \return pointer to the proper variables vector of the strip container*/
vector<variable>& cell::GetXpStrip()
{return *XpStrip;
}//-----------------------------------------------------------------------------

/** \fn int& cell::GetXiDim()
    \return selected improper dimention for bisection*/
int cell::GetXiDim()
{return XiDim;
}//-----------------------------------------------------------------------------

/** \fn twin& cell::GetTwin(void)
    \return cell twin*/
twin& cell::GetTwin(void)
{return Twin;
}//-----------------------------------------------------------------------------

/** \fn  int cell::GetDim() const
    \return cell dimention*/
int cell::GetDim() const
{if(this->XpStrip!=NULL) return XpStrip->size()+Xi.size();
 else return Xp.size()+Xi.size();
}//-----------------------------------------------------------------------------

/** \fn iboolean cell::GetConsist(void)
    \return consistency flag (itrue==Consistent; ifalse==incosistent; ipehaps=?)*/
iboolean cell::GetConsist(void)
{return Consistent;
}//-----------------------------------------------------------------------------

/** \fn bool cell::GetMono(void)
    \return monotonic flag (true==monotonic; false==non-nonotonic)*/
bool cell::GetMono(void)
{return Monotonic;
}//-----------------------------------------------------------------------------

/** \fn int cell::GetOptimal(void)
    \return Optima flag (0==NonOptimal; 1==Optimal)*/
bool cell::GetTOptimal(void)
{return TOptimal;
}//-----------------------------------------------------------------------------
/*
int cell::GetPenalty()
{return Penalty;
}//-----------------------------------------------------------------------------
*/

bool cell::GetZeroIncluded()
{return ZeroIncluded;
}//-----------------------------------------------------------------------------

bool cell::GetZeroExcluded()
{return ZeroExcluded;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetXi(vector<variable>& Ivars)
    \brief Set vector of improper variables.
    \param Ivars vector of improper variables.*/
void cell::SetXi(vector<variable>& Ivars)
{Xi.resize(Ivars.size());
 Xi=Ivars;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetXp(vector<variable>& Pvars)
    \brief Set vector of proper variables.
    \param Pvars vector of proper variables.*/
void cell::SetXp(vector<variable>& Pvars)
{Xp.resize(Pvars.size());
 Xp=Pvars;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetXpStrip(vector<variable>& Pvars)
    \brief Set cell XpStrip pointer to Xp strip vector.
    \param Pvars vector of proper variables (of the strip container)*/
void cell::SetXpStrip(vector<variable>& Pvars)
{XpStrip=&Pvars;
}//-----------------------------------------------------------------------------

void cell::SetXpStrip(vector<variable>* Pvars)
{XpStrip=Pvars;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetXiDim(int& Dim)
    \brief Set selected improper dimention.
    \param */
void cell::SetXiDim(int& Dim)
{XiDim=Dim;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetTwin(twin &TW)
    \brief Set cell twin.
    \param TW twin.*/
void cell::SetTwin(twin &TW)
{Twin=TW;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetConsist(iboolean &CST)
    \brief Set consistency flag.
    \param CST Consistency flag (itrue==Consistent; ifalse==incosistent; ipehaps=?)*/
void cell::SetConsist(iboolean &CST)
{Consistent=CST;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetMono(bool &MNT)
    \brief Set monotony flag.
    \param MNT monotony flag (true==monotonic; false==non-nonotonic)*/
void cell::SetMono(bool &MNT)
{Monotonic=MNT;
}//-----------------------------------------------------------------------------

/** \fn void cell::SetOptimal(bool &OPT)
    \brief Set Optimal flag.
    \param OPT Optimal flag (0==NonOptimal; 1==Optimal)*/
void cell::SetTOptimal(bool TOPT)
{TOptimal=TOPT;
}//-----------------------------------------------------------------------------
/*
void cell::SetPenalty(int P)
{Penalty=P;
}//-----------------------------------------------------------------------------

void cell::Penalize()
{Penalty++;
}//-----------------------------------------------------------------------------
*/

void cell::SetZeroIncluded(bool ZI)
{ZeroIncluded=ZI;
}//-----------------------------------------------------------------------------

void cell::SetZeroExcluded(bool ZE)
{ZeroExcluded=ZE;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Boolean Member Functions                                                   //
////////////////////////////////////////////////////////////////////////////////

/** \fn bool IsUniInci(void)
    \return if it is uniincident*/
bool cell::IsUniInci(void)
{bool aux=true;
 int i=0;
 while(i<this->GetDim() && aux==true)
 {if(this->operator[](i).GetNumIncs()==1) aux=true;
  else aux=false;
  i++;}
  return aux;
}//-----------------------------------------------------------------------------

/** \fn bool IsProp(void)
    \return Return if it is proper*/
bool cell::IsProp(void)
{if(this->Xi.size()==0) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsImpr(void)
    \return if it is Improper*/
bool cell::IsImpr(void)
{if(this->Xp.size()==0) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsUniMod(void)
    \return if it is unimodal*/
bool cell::IsUniMod(void)
{return (this->IsProp() || this->IsImpr());
}//-----------------------------------------------------------------------------

/** \fn bool IsMono(void)
    \return true if it is monotonic respect all variables and incidence.*/
bool cell::IsTotalyMono(void)
{bool aux=true;
 int i=0;
 
 while(i<this->GetDim() && aux==true)
 {if(this->operator[](i).IsTotalyMono()==true)aux=true;
  else aux=false;
  i++;}
 return aux;
}//-----------------------------------------------------------------------------

/** \fn bool IsXiTotalyMono(void)
    \return true if it is monotonic respect all improper variables and incidence.*/
bool cell::IsXiTotalyMono(void)
{bool aux=true;
 int i=0;
 
 while(i<this->GetXi().size() && aux==true)
 {if(this->GetXi()[i].IsTotalyMono()==true)aux=true;
  else aux=false;
  i++;}
 return aux;
}//-----------------------------------------------------------------------------

/** \fn iboolean cell::IsConsist(void)
    \return if it is consistent (MiniMax algortithm*/
iboolean cell::IsConsist(void)
{return Consistent;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn double cell::GetXBigWidth()
    \return Biggest Width.*/
double cell::GetXBigWidth()
{int Dim=-1;
 double MaxWidth=MInfinity();
 ivalDb Dom;
 for(int i=0 ;i<this->GetDim();i++)
 {Dom=this->operator [](i).GetIval();
  if(Width(Dom) > MaxWidth)
        {MaxWidth=Width(Dom);
         Dim=i;}
 }
 return MaxWidth;
}//-----------------------------------------------------------------------------

/** \fn double cell::GetImprBigWidth()
    \return Improper Biggest Width.*/
void cell::AssignXiBigDim()
{int Dim=-1;
 double MaxWidth=MInfinity();
 ivalDb Dom;
 for(int i=0 ;i<this->Xi.size();i++)
 {Dom=Xi[i].GetIval();
  if(Width(Dom) > MaxWidth)
        {MaxWidth=Width(Dom);
         Dim=i;}
 }
 this->XiDim=Dim;
}//-----------------------------------------------------------------------------

/*
void cell::AssignNonMonoXiBigDim(void)
{double MaxWidth=MInfinity();
 int Dim=-1;
 ivalDb Dom;
 bool Mono,UniInci;
 int BPenalty=-1;
 int Penalty;

 for(int i=0 ;i<this->Xi.size();i++)
 {
  Dom=Xi[i].GetIval();
  Mono=Xi[i].IsTotalyMono();
  UniInci=this->Xi[i].IsUniInci();
  Penalty=this->Xi[i].GetPenalty();
  if(this->Consistent==itrue && ((this->TOptimal && UniInci) || Mono)){}
  else if(Penalty > BPenalty)
   {BPenalty=Penalty;
    Dim=i;}
  else if((Penalty == BPenalty) && (Width(Dom) > MaxWidth))            // Select the Non Monotonic biggest width.
   {MaxWidth=Width(Dom);
    Dim=i;}
 }
 this->XiDim=Dim;
}//-----------------------------------------------------------------------------
*/

void cell::AssignNonMonoXiBigDim(param& Pars)
{double MaxWidth=MInfinity();
 int Dim=-1;
 ivalDb Dom;
 bool Mono,UniInci;
 int XpDim=this->Xp.size();
 double XWidth;

 for(int i=0 ;i<this->Xi.size();i++)
 {
  Dom=Xi[i].GetIval();
  Mono=Xi[i].IsTotalyMono();
  UniInci=this->Xi[i].IsUniInci();
  //XWidth=Pars.GetXWidths()[XpDim+i];
  if(this->Consistent==itrue && ((this->TOptimal && UniInci) || Mono)){}
  //else if(Width(Dom)/XWidth > MaxWidth)            // Select the Non Monotonic biggest width.
  else if(Width(Dom) > MaxWidth)
        {
        //MaxWidth=Width(Dom)/XWidth;
        MaxWidth=Width(Dom);
        Dim=i;}
 }
 this->XiDim=Dim;
}//-----------------------------------------------------------------------------


/** \fn double GetNonMonoImprBigWidth(void)
    \return Non Monotonic Improper Biggest Width.
    \param TOptim: Tree optimality flag.*/
double cell::GetNonMonoXiBigWidth(void)
{if(this->XiDim>=0)
 {ivalDb Interval=this->Xi[this->XiDim].GetIval();
  return Width(Interval);}
 else {return MInfinity();}
}//-----------------------------------------------------------------------------

/** \fn void cell::ComputeTwin(tree &T)
    \brief Compute Cell Twin*/
void cell::ComputeTwin(tree &T,bool StripTOptim,param& Pars)
{
 AssignCellToTree(T,*this,true);

 this->ComputeMonos(T,"Xi");

 this->AssignNonMonoXiBigDim(Pars);

 this->AssignTreeOptim(T); 

 this->Twin.SetInn(Du(T.EvaluateFunct("Inn",StripTOptim,this->TOptimal,this->Consistent,'c')));   // Compute Inner  Approximation

 /*
 ivalDb Inn=T.EvaluateFunct("Inn",StripTOptim,this->TOptimal,this->Consistent,'c');
 Inn=Inn || T.EvaluateFunct("Inn",StripTOptim,this->TOptimal,this->Consistent,'i');
 Inn=Inn || T.EvaluateFunct("Inn",StripTOptim,this->TOptimal,this->Consistent,'s');
 this->Twin.SetInn(Du(Inn));   // Compute Inner  Approximation
 */
 
 AssignCellToTree(T,*this,false);       // No Assignar inecessariament Xi!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 this->ComputeMonos(T,"Xp");

 if(this->Consistent==itrue)
  {this->Twin.SetOut(T.EvaluateFunct("Out",StripTOptim,this->TOptimal,this->Consistent,'c'));}
 else if(this->Consistent==iperhaps)
  {this->Twin.SetOut(T.EvaluateFunct("Prop",StripTOptim,this->TOptimal,this->Consistent,'c'));}

 this->Twin.AssignTol();
 /*
 if(IsIn(this->Twin.GetOut(),Zero()))    // For Testing
 {this->ZeroIncluded=true;}
 else
 {this->ZeroIncluded=false;}

 if(!IsIn(this->Twin.GetInn(),Zero()))    // For Testing
 {this->ZeroExcluded=true;}
 else
 {this->ZeroExcluded=false;}
 */
}//-----------------------------------------------------------------------------

/** \fn void cell::ComputeTwinProp(tree &T)
    \brief Compute Cell Twin*/
void cell::ComputeTwinProp(tree &T,string &Opt,bool StripTOptim,param& Pars)
{
 this->MakeProp();
 AssignCellToTree(T,*this,false);
 this->ComputeMonos(T,"Full");

 this->AssignNonMonoXiBigDim(Pars);

 if(Opt=="MinMax")
  {this->AdaptCellToMinMax();}
 else if(Opt=="MaxMin")
  {this->AdaptCellToMaxMin();}

 this->Twin.SetInn(Du(T.EvaluateFunct("Inn",this->TOptimal,StripTOptim,this->Consistent,'c')));   // Compute Inner  Approximation
 this->Twin.SetOut(T.EvaluateFunct("Out",this->TOptimal,StripTOptim,this->Consistent,'c'));
 this->Twin.AssignTol();

 if(IsIn(this->Twin.GetOut(),Zero()))    // For Testing
 {this->ZeroIncluded=true;}
 else
 {this->ZeroIncluded=false;}

 if(!IsIn(this->Twin.GetInn(),Zero()))    // For Testing
 {this->ZeroExcluded=true;}
 else
 {this->ZeroExcluded=false;}
}//-----------------------------------------------------------------------------

/** \fn void ComputeMonos(tree &Tree)
    \brief Compute monotonies.
    \param  void (*Derivatives)(cell&) pointer to derivatives function*/
void cell::ComputeMonos(tree &T,const string &Mode)
{
 T.EvaluateDers(Mode);       // Mode="Xi" Only Xi vector

 ivalDb MonoDom=0;
 ivalDb MonoDomInc;
 int index;

if(Mode!="Xi")
{
 for(int i=0;i<this->GetXp().size();i++)
 {MonoDom=0;
  index=T.GetXpIndexes()[i];
   for(int j=0;j<this->GetXp()[i].GetNumIncs();j++)
    {
     MonoDomInc=T.GetVars()[index][j]->GetMonoDom();
     if(IsIn(0,MonoDomInc))
      {MonoDom=Infinity(); break;}
     else
      {MonoDom=MonoDom + MonoDomInc;}
     }
   this->GetXp()[i].SetMonoDom(MonoDom);
 }
}

if(Mode!="Xp")
{
 for(int i=0;i<this->GetXi().size();i++)
 { MonoDom=0;
   index=T.GetXiIndexes()[i];
   for(int j=0;j<this->GetXi()[i].GetNumIncs();j++)
    {
     MonoDomInc=T.GetVars()[index][j]->GetMonoDom();
     if(IsIn(0,MonoDomInc))
      {MonoDom=Infinity(); break;}
     else
      {MonoDom=MonoDom + MonoDomInc;}
     }
   this->GetXi()[i].SetMonoDom(MonoDom);
 }
}
  this->AssignXMonos(Mode,T);
}//-----------------------------------------------------------------------------

void cell::AssignXMonos(const string &Mode,tree &T)
{
 int XpDim=this->GetXp().size();
 int XiDim=this->GetXi().size();

if(Mode!="Xi")
{
   for(int i=0 ;i<XpDim;i++)
   {this->Xp[i].AssignMono();
    for(int j=0 ;j<this->Xp[i].GetNumIncs();j++)
    {this->Xp[i][j].AssignMono();}
     this->Xp[i].AssignTotalMono();
   }
 }

if(Mode!="Xp")
{
 for(int i=0 ;i<XiDim;i++)
 {this->Xi[i].AssignMono();
  for(int j=0 ;j<this->Xi[i].GetNumIncs();j++)
  {this->Xi[i][j].AssignMono();}
   this->Xi[i].AssignTotalMono();
 }
}
}//-----------------------------------------------------------------------------

void cell::AssignTreeOptim(tree &T)
{
 if(T.GetTOptimal())
 {this->TOptimal=true;}
 else
 {
   if(T.GetUniIncident())
   {this->TOptimal=T.IsDomainTOptimal();}
   else
   {this->TOptimal=false;}
 }
}//-----------------------------------------------------------------------------

void cell::Join(cell &Cell)
{
  for(int i=0 ;i<this->GetXp().size();i++)
  {this->GetXp()[i].SetIval(this->GetXp()[i].GetIval() || Cell.GetXp()[i].GetIval());}
  for(int i=0 ;i<this->GetXi().size();i++)
  {this->Xi[i].SetIval(this->Xi[i].GetIval() && Cell.GetXi()[i].GetIval()) ;}
}//-----------------------------------------------------------------------------

void cell::Meet(cell &Cell)
{
  for(int i=0 ;i<this->GetXp().size();i++)
  {this->Xp[i].SetIval(this->Xp[i].GetIval() && Cell.GetXp()[i].GetIval()) ;}
  for(int i=0 ;i<this->GetXi().size();i++)
  {this->Xi[i].SetIval(this->Xi[i].GetIval() || Cell.GetXi()[i].GetIval()) ;}
}//-----------------------------------------------------------------------------
//
bool operator<(const cell &C1, const cell &C2)
{
 bool Small=false;
 int XpDim;

 int i=0;
 if(C1.XpStrip!=NULL) {XpDim=C1.GetXp().size();}
 else {XpDim=C1.GetXp().size();}

 while(i<XpDim)
 {
   if(C1.operator [](i).GetIval().GetInf() < C2.operator [](i).GetIval().GetInf())
    {Small=true; break;}
   else if(C1.operator [](i).GetIval().GetInf() > C2.operator [](i).GetIval().GetInf())
    {Small=false; break;}
   else
   {
    if(C1.operator [](i).GetIval().GetSup() < C2.operator [](i).GetIval().GetSup())
     {Small=true; break;}
    else if(C1.operator [](i).GetIval().GetSup() > C2.operator [](i).GetIval().GetSup())
     {Small=false; break;}
   }
 i++;
 }
return Small;
}//------------------------------------------------------------------------------

cell operator&&(const cell &c1, const cell &c2)
{ cell Cell;
  for(int i=0;i<c1.GetDim();i++)
  {Cell[i].SetIval(c1.operator [](i).GetIval() && c2.operator [](i).GetIval());}
  return Cell;
}//------------------------------------------------------------------------------

cell operator||(const cell &c1, const cell &c2)
{
  cell Cell;
  for(int i=0;i<c1.GetDim();i++)
  {Cell[i].SetIval(c1.operator [](i).GetIval() || c2.operator [](i).GetIval());}
  return Cell;
}//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// No Funtion Classes                                                        //
////////////////////////////////////////////////////////////////////////////////

// Assign Cell incidence pointers
void AssignCellToTree(tree &Tree,cell& Cell,bool XpStrip)
{int index;

if(XpStrip)
{
 for(int i=0;i<Cell.GetXpStrip().size();i++)
 { index=Tree.GetXpIndexes()[i];
   for(int j=0;j<Cell.GetXpStrip()[i].GetNumIncs();j++)
    {Tree.GetVars()[index][j]=&Cell.GetXpStrip()[i][j];}
 }
}

else
{
  for(int i=0;i<Cell.GetXp().size();i++)
 { index=Tree.GetXpIndexes()[i];
   for(int j=0;j<Cell.GetXp()[i].GetNumIncs();j++)
    {Tree.GetVars()[index][j]=&Cell.GetXp()[i][j];}
 }
}

 for(int i=0;i<Cell.GetXi().size();i++)
 { index=Tree.GetXiIndexes()[i];
   for(int j=0;j<Cell.GetXi()[i].GetNumIncs();j++)
    {Tree.GetVars()[index][j]=&Cell.GetXi()[i][j];}
 }
}//-----------------------------------------------------------------------------

// Assign Cell incidence pointers
void AssignCellToConstraint(tree &Tree,cell& Cell)
{
 int size=Tree.GetVars().size();
 for(int i=0;i<size;i++)
 {
   for(int j=0;j<Tree.GetVars()[i].size();j++)
   {Tree.GetVars()[i][j]=&Cell[i][0];}
 }
}//-----------------------------------------------------------------------------


// Create Cell From Tree (Modificar per accpetar l'ordre al.leatori de les variables)
void CreateCellFromTree(cell &Cell,tree &Tree)
{
 vector<variable> Xp,Xi;
 node* Node;
 incidence* Inc;

 Tree.GetXpIndexes().clear();
 Tree.GetXiIndexes().clear();

 for(int i=0;i<Tree.GetNodeVars().size();i++)             // Count the dimention of Xp & Xi
  {if(Tree.GetNodeVars()[i].GetIval().IsProp())
   {Tree.GetXpIndexes().push_back(i);}
   else
   {Tree.GetXiIndexes().push_back(i);}
  }

 Xp.resize(Tree.GetXpIndexes().size());
 Xi.resize(Tree.GetXiIndexes().size());

 for(int i=0;i<Tree.GetXpIndexes().size();i++)
 {Node=&Tree.GetNodeVars()[Tree.GetXpIndexes()[i]];
  Xp[i]=variable(Tree.GetVars()[Tree.GetXpIndexes()[i]].size(),Node->GetIval(),Node->GetNameVar());}

 for(int i=0;i<Tree.GetXiIndexes().size();i++)
 {Node=&Tree.GetNodeVars()[Tree.GetXiIndexes()[i]];
  Xi[i]=variable(Tree.GetVars()[Tree.GetXiIndexes()[i]].size(),Node->GetIval(),Node->GetNameVar());}

 Cell.SetXp(Xp);
 Cell.SetXi(Xi);
}//-----------------------------------------------------------------------------

void AssignUniIncidence(tree &Tree, cell& Cell)
{
 int XpUni=0;
 int XiUni=0;
 list<int> Indexes;

 for(int i=0;i<Cell.GetXp().size();i++)
  {if(Cell.GetXp()[i].GetNumIncs()==1)
    {XpUni++;
     Indexes.push_back(i);}
  }
 Tree.GetXpUniIndexes().resize(XpUni);
 for(int i=0; i<XpUni;i++)
 {Tree.GetXpUniIndexes()[i]=Indexes.front();
  Indexes.pop_front();}

 for(int i=0;i<Cell.GetXi().size();i++)
  {if(Cell.GetXi()[i].GetNumIncs()==1)
   {XiUni++;
    Indexes.push_back(i);}
  }
 Tree.GetXiUniIndexes().resize(XiUni);
 for(int i=0; i<XiUni;i++)
 {Tree.GetXiUniIndexes()[i]=Indexes.front();
  Indexes.pop_front();}

 if(XpUni!=0 || XiUni!=0)
 {Tree.SetUniIncident(true);}
}//-----------------------------------------------------------------------------

void AssignUniModal(tree &Tree, cell& Cell)
{
 if(Cell.GetXp().empty() || Cell.GetXi().empty())
 {Tree.SetUniModal(true);}
 else
 {Tree.SetUniModal(false);}
}//-----------------------------------------------------------------------------

void AssignXCrossDers(tree &T, cell& C)
{
 int index;
 int XpDim=C.GetXp().size();
 int XiDim=C.GetXi().size();
 T.GetXCrossDers().resize(XpDim+XiDim);
 list<node*>::iterator N;
 bool Match=false;
 string Name1,Name2;

 for(int i=0;i<C.GetDim();i++)
 {Match=false;
  for(int ii=0;ii<C.GetDim();ii++)
  {
   if(i!=ii)
   {
    if(C[ii].GetIval().IsProp()){index=T.GetXpIndexes()[ii];}
    else {index=T.GetXiIndexes()[ii-XpDim];}
    for(int j=0;j<T.GetDersLists()[index].size();j++)
    {
     for(int k=0;k< T.GetDersLists()[index][j].size();k++)
     {
       for(N=T.GetDersLists()[index][j].begin(); N!=T.GetDersLists()[index][j].end();N++)
       {
        if((*N)->GetType()=="var")
        {Name1=C[i].GetName();
         Name2=(*N)->GetNameVar();
         if(Name1==Name2)
         {T.GetXCrossDers()[i].push_back(ii);
          Match=true;
          break;}
        }
        if(Match){break;}
       }
       if(Match){break;}
     }
     if(Match){break;}
    }
   } //End if(i!=ii)
  }
 }

}//-----------------------------------------------------------------------------



bool IsIntersect(cell &c1,cell &c2)
{
  int i=0;
  bool Intersect=true;
  ivalDb Meet;
  while(i<c1.GetDim() && Intersect)
  {
   Meet=Prop(c1[i].GetIval()) && Prop(c2[i].GetIval());
   if(Meet.IsProp())
    {Intersect=true;}
   else
    {Intersect=false;
     break;}
   i++;
  }
 return Intersect;
}//-----------------------------------------------------------------------------

void cell::MakeProp(void)
{
 for(int i=0;i<this->GetXi().size();i++)
 {
  this->GetXi()[i].SetIval(Prop(this->GetXi()[i].GetIval()));
 }
}//-----------------------------------------------------------------------------


/** \fn void cell::PrintCell(fstream &fout,int PrintFlag)
    \brief Print cell structure to an output stream.
    \param fout output stream;
    \param PrintFlag 1==Print; 0==No Print).*/
void cell::Print(fstream &fout,int PrintFlag)
{if(PrintFlag==1)
 {
 //fout << "cell---------------------------------------------------" << endl;
 fout << "box---------------------------------------------------" << endl;

  fout << "Xp vector:" << endl;

  for(int i=0 ;i<this->GetXp().size();i++)
    {

     fout << this->GetXp()[i].GetName() << "[" << i << "].Dom=" << this->GetXp()[i].GetIval()  << "\t";
     fout << "Mono(Strip)=" << this->GetXpStrip()[i].GetMono() << " MonoDom=" << this->GetXpStrip()[i].GetMonoDom() << endl;
     //this->GetXpStrip()[i].Print(i,fout,PrintFlag);

     //fout << this->GetXp()[i].GetName() << "[" << i << "]=" << this->GetXp()[i].GetIval()  << endl;
     }

  fout << "Xi vector:" << endl;

 for(int i=0 ;i<this->GetXi().size();i++)
  {

   fout << this->GetXi()[i].GetName() << "[" << i << "].Dom=" << this->GetXi()[i].GetIval()  << "\t";
   fout << "Mono(Cell)=" << this->GetXi()[i].GetMono() << " MonoDom=" << this->GetXi()[i].GetMonoDom() <<  endl;
   //this->GetXi()[i].Print(i,fout,PrintFlag);

   //fout << this->GetXi()[i].GetName() << "[" << i << "]=" << this->GetXi()[i].GetIval()  << endl;
   }

  fout << endl;
  this->Twin.Print(fout,PrintFlag);
  //fout << "Consistency:" << this->Consistent  << endl;
  //fout << "Penalty: " << this->Penalty << endl << endl;
  
  //fout << "---------------------------------------------------cell" << endl;
  fout << "---------------------------------------------------box" << endl;
}
}//-----------------------------------------------------------------------------


// MinMax Functions

void cell::AdaptCellToMinMax()
{
 ivalDb Dom;
 for(int i=0; i<this->GetXp().size();i++)
 {

   if(this->Consistent==itrue && this->GetXp().size()>0)
   {
     Dom=this->GetXp()[i].GetIval();
     if(this->GetXp().operator [](i).IsMonoIncr())
            {this->GetXp()[i].SetIval(Ival(Dom.GetInf(),AddULPs(Dom.GetInf(),2)));}
     else if (this->GetXp().operator [](i).IsMonoDecr())
            {this->GetXp()[i].SetIval(Ival(RestULPs(Dom.GetSup(),2),Dom.GetSup()));}
     else{}
   }
 }

 for(int i=0; i<this->GetXi().size();i++)
 {
   if(this->Consistent==itrue)
   {
     Dom=this->GetXi()[i].GetIval();
     if(this->GetXi()[i].IsMonoIncr())
            this->GetXi()[i].SetIval(Ival(RestULPs(Dom.GetSup(),2),Dom.GetSup()));
     else if (this->GetXi()[i].IsMonoDecr())
            this->GetXi()[i].SetIval(Ival(Dom.GetInf(),AddULPs(Dom.GetInf(),2)));
     else{}
   }
 }
}//-----------------------------------------------------------------------------

void cell::AdaptCellToMaxMin()
{
 ivalDb Dom;
 for(int i=0; i<this->GetXp().size();i++)
 {
   if(this->Consistent==itrue)
   {
     Dom=this->GetXp()[i].GetIval();
     if(this->GetXp().operator [](i).IsMonoIncr())
            this->GetXp()[i].SetIval(Ival(RestULPs(Dom.GetSup(),2),Dom.GetSup()));
     else if (this->GetXp().operator [](i).IsMonoDecr())
            this->GetXp()[i].SetIval(Ival(Dom.GetInf(),AddULPs(Dom.GetInf(),2)));
     else{}
   }
 }

 for(int i=0; i<this->GetXi().size();i++)
 {
   if(this->Consistent==itrue)
   {
     Dom=this->GetXi()[i].GetIval();
     if(this->GetXi()[i].IsMonoIncr())
            this->GetXi()[i].SetIval(Ival(Dom.GetInf(),AddULPs(Dom.GetInf(),2)));
     else if (this->GetXi()[i].IsMonoDecr())
            this->GetXi()[i].SetIval(Ival(RestULPs(Dom.GetSup(),2),Dom.GetSup()));
     else{}
   }
 }
}//-----------------------------------------------------------------------------

//##############################################################################

