/*!****************************************************************************
 *    \file fstar.cpp
 *    This file contains the mplementation of the fstar class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

 /*
 10/01/05 Eliminar tires per inclusi.
 */

extern bool GlobalStop;

#include "Plot.h"
#include "fstar.h"
#include "functions.h"
#include <time>

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/**\fn fstar::fstar()
   Default Constructor.*/
fstar::fstar()
{Twin=twin("dn");
 AccTwin=twin("dd");
 Update=true;
 SelectedStrip=NULL;
 }//-----------------------------------------------------------------------------

/** \fn fstar::fstar(const fstar &FS)
    Copy Constructor.*/
fstar::fstar(const fstar &FS)
{Twin=FS.Twin;
 AccTwin=FS.AccTwin;
 Update=FS.Update;
 List=FS.List;
 SelectedStrip=FS.SelectedStrip;
 Model=FS.Model;
 Pars=FS.Pars;
}//-----------------------------------------------------------------------------


/** \fn model& model::operator=(const model &Model).
    \brief Assignation (=) overloaded operator.*/
fstar& fstar::operator=(const fstar &FS)
{Twin=FS.Twin;
 AccTwin=FS.AccTwin;
 Update=FS.Update;
 List=FS.List;
 SelectedStrip=FS.SelectedStrip;
 Model=FS.Model;
 Pars=FS.Pars;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn list<strip>& fstar::GetList()
    \return list of strips. */
list<strip>& fstar::GetList()
{return List;
}//-----------------------------------------------------------------------------

/** \fn twin& fstar::GetTwin()
    \return global twin.*/
twin& fstar::GetTwin()
{return Twin;
}//-----------------------------------------------------------------------------

/** \fn twin& fstar::GetAccTwin()
    \return Accumulated twin.*/
twin& fstar::GetAccTwin()
{return AccTwin;
}//-----------------------------------------------------------------------------

/** \fn bool fstar::GetUpdate()
    \return Update flag. */
bool fstar::GetUpdate()
{return Update;
}//-----------------------------------------------------------------------------

/** \fn list<strip>::iterator& fstar::GetSelectStrip(void)
    \return Selected strip to bisect.*/
list<strip>::iterator& fstar::GetSelectStrip(void)
{return SelectedStrip;
}//-----------------------------------------------------------------------------


/** \fn int param& fstar::GetPars(void)
    \return Parameters structure.*/
param& fstar::GetPars(void)
{return Pars;
}//-----------------------------------------------------------------------------

/** \fn model& fstar::GetModel(void)
    \return Model.*/
model& fstar::GetModel(void)
{return Model;
}//-----------------------------------------------------------------------------

list<strip>& fstar::GetBuffer(void)
{return Buffer;
}//-----------------------------------------------------------------------------

list< list<strip>::iterator >& fstar::GetInsertedStrips(void)
{return InsertedStrips;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetList(list<strip>& LST)
    \brief Set List od strips.*/
void fstar::SetList(list<strip>& LST)
{List=LST;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetTwin(twin& TW)
    \brief Set Global twin.*/
void fstar::SetTwin(twin& TW)
{Twin=TW;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetAccTwin(twin& ACCTW)
    \brief Accumulate twin. */
void fstar::SetAccTwin(twin& ACCTW)
{AccTwin=ACCTW;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetUpdate(bool UD)
    \brief Set Update flag.*/
void fstar::SetUpdate(bool UD)
{Update=UD;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetSelectStrip(list<strip>::iterator &SEL)
    \brief Set selected strip to bisect.*/
void fstar::SetSelectStrip(list<strip>::iterator &SEL)
{SelectedStrip=SEL;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetPars(param& P)
    \brief Set Parameters structure.*/
void fstar::SetPars(param& P)
{Pars=P;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SetModel(model& M)
    \brief Set Model.*/
void fstar::SetModel(model& M)
{Model=M;
}//-----------------------------------------------------------------------------

void fstar::SetBuffer(list<strip>& B)
{Buffer=B;
}//-----------------------------------------------------------------------------

/** \fn list<cell>& fstar::GetMinMaxList(void)
    \return MinMax candidates list.*/
list<cell>& fstar::GetMinMaxList(void)
{return MinMaxList;
}//-----------------------------------------------------------------------------

/** \fn list<cell>& fstar::GetMaxMinList(void)
    \return MaxMin candidates list.*/
list<cell>& fstar::GetMaxMinList(void)
{return MaxMinList;
}//-----------------------------------------------------------------------------

/** \fn list<cell> fstar::GetMinMaxs(void)
    \return MinMaxs cell.*/
list<cell>& fstar::GetMinMaxs(void)
{return MinMaxs;
}//-----------------------------------------------------------------------------

/** \fn list<cell> fstar::GetMaxMins(void)
    \return Maxmin cells.*/
list<cell>& fstar::GetMaxMins(void)
{return MaxMins;
}//-----------------------------------------------------------------------------

void fstar::SetInsertedStrips(list< list<strip>::iterator >& IS)
{InsertedStrips=IS;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Members functions                                                          //
////////////////////////////////////////////////////////////////////////////////


/** \fn void fstar::SplitXToXpXi(vector<variable> &Xp,vector<variable> &Xi)
    Split the vector of variable to a vector of proper variables and a vector
    of improper variables.*/
void fstar::SplitXToXpXi(vector<variable> &Xp,vector<variable> &Xi)
{
 int XpDim=0;
 int XiDim=0;
 vector<variable> X=this->Model.GetX();  // Get vector of variables

 for(int i=0;i<X.size();i++)             // Count the dimention of Xp & Xi
  {if(X[i].IsProp()) XpDim++;
   else XiDim++;}
 Xp.resize(XpDim);
 Xi.resize(XiDim);

 int iXp=0;                             // Xp Index
 int iXi=0;                             // Xi Index
 for(int i=0;i<X.size();i++)            // Assign X to Xp & Xi
  {if(X[i].IsProp()) {Xp[iXp]=X[i]; iXp++;}
   else {Xi[iXi]=X[i]; iXi++;}
  }
}//-----------------------------------------------------------------------------

/** \fn void fstar::AccumTwin(list<strip>::iterator Strip)
    \brief Accumulate Strip twin to the Accumulated Strip.*/
void fstar::AccumTwin(list<strip>::iterator Strip)
{
   if(Strip->GetJoinFlag()==true)
      {this->AccTwin.SetInn(this->AccTwin.GetInn() || Strip->GetTwin().GetInn());}  // Join Inner Approxs.
   this->AccTwin.SetOut(this->AccTwin.GetOut() || Strip->GetTwin().GetOut());    // Join Outer Approxs.
   
   this->Buffer.splice(this->Buffer.end(),this->List,Strip); // Store Strip to Buffer (MinMax optimization).
}//-----------------------------------------------------------------------------


// Inclusion Stop Conditions
bool fstar::IsStripInclusionStop(twin& Twin)
{
  if(this->Pars.GetInclusion()==1)
  {
   if(Twin.GetInn().IsInterval() && !IsIn(Twin.GetInn(),Zero()))
    {return true;}
   else
    {return false;}
  }
  else
    {return false;}
}//-----------------------------------------------------------------------------

/** \fn  void fstar::UpdateTwinAndSelectStrip(fstream &fout)
    \brief Update global twin approximation.*/
bool fstar::UpdateTwinAndSelectStrip(fstream &fout)
{
 bool Stop=false;
 list<list<strip>::iterator>::iterator IStrip;

 int Print=Pars.GetDebug();

 if(this->Update)
 {
  double Bigger=MInfinity();
  list< list<strip>::iterator> Match;
  list<list<strip>::iterator> Trash;

  PrintMsg("Action: Update Global Twin",fout,Print);
  PrintMsg("Previous Update Global Twin",fout,Print);
  this->Twin.Print(fout,Print);
  this->Twin=this->AccTwin;                 // Initialize Twin=Accumulated Twin.
  PrintMsg("Accumulated Twin",fout,Print);
  this->AccTwin.Print(fout,Print);
  PrintMsg("Strip's List",fout,Print);

  list<strip>::iterator Strip;              // Make Join of Strip Twins in List.
  for (Strip=this->List.begin(); Strip != this->List.end(); Strip++)
  {
    if(Strip->GetJoinFlag())
    {this->Twin.SetInn(this->Twin.GetInn() || Strip->GetTwin().GetInn());}  // Join Inner Approxs.
     this->Twin.SetOut(this->Twin.GetOut() || Strip->GetTwin().GetOut());    // Join Outer Approxs.
    /*
    Strip->Print(fout,Print,false);
    if(IsStripInclusionStop(Strip->GetTwin()))
    {Stop=true;
     break;}
    */

  } // End For

  this->Twin.AssignTol();                     // Assign Tolerance to the new Twin.
  PrintMsg("Updated Global Twin:",fout,Print);
  this->Twin.Print(fout,Print);

 for (IStrip=Trash.begin(); IStrip != Trash.end(); IStrip++)
 {if(this->Pars.GetPlot()) PlotStripCadre(**IStrip,clFuchsia,psSolid,2,Pars);
  List.erase(*IStrip);}
 Trash.clear();

 } // if(this->Update==true)

 else
  {PrintMsg("No Update Global Twin:",fout,Print);}

 this->SelectStrip(fout);                // Select Strip
// return Stop;
}//-----------------------------------------------------------------------------

/** \fn  void fstar::SelectStrip(fstream &fout)
    \brief Select Strip To Bisect.*/
void fstar::SelectStrip(fstream &fout)
{twin TW;
 double Bigger=MInfinity();
 list<strip>::iterator Strip;              // Make Join of Strip Twins in List.
 int Print=this->GetPars().GetDebug();
 //int AuxPenalty=99999999999999;

 list<list<strip>::iterator> Trash;

 if(this->Update || this->Stock.empty())
 {
   this->Stock.clear();
   for (Strip=this->List.begin(); Strip != this->List.end(); Strip++)
    {
      if(Pars.GetOptim() ==1) {Strip->GetTwin().AssignInfTol();}
      else if(Pars.GetOptim() ==2) {Strip->GetTwin().AssignSupTol();}

      if (Strip->GetConsist()==ifalse)       // Store Strips to the Trash List
        {PrintMsg("Eliminate Strip: Consistency",fout,Print);
         Trash.push_back(Strip);}

      else if(this->IsIncludedStrip(*Strip,fout))              // If Strip is Included Not bisect Xp vector
        {PrintMsg("Eliminate Strip: Inclusion",fout,Print);
         Strip->Print(fout,Print,false);
         Trash.push_back(Strip);
        }
      /*
      else if(this->Pars.GetInclusion()==1 && Strip->GetZeroIncluded())
       {PrintMsg("Eliminate Strip: Logic",fout,Print);
        Trash.push_back(Strip);}
      */

      else if(this->Pars.GetOptim()!=2 && (Strip->GetTwin().GetInn().GetInf() == this->Twin.GetInn().GetInf() ||
         Strip->GetTwin().GetOut().GetInf() == this->Twin.GetOut().GetInf()))
        {this->Stock.push_back(Strip);}
      else if(this->Pars.GetOptim()!=1 && (Strip->GetTwin().GetInn().GetSup() == this->Twin.GetInn().GetSup() ||
              Strip->GetTwin().GetOut().GetSup() == this->Twin.GetOut().GetSup()))
         {this->Stock.push_back(Strip);}


      //else{this->Stock.push_back(Strip);} // New!!!!!!!!!!!!!!!!!!
      
     }

   list<list<strip>::iterator>::iterator TStrip;                 // Erase Strips Stored In the Trash List
   for (TStrip=Trash.begin(); TStrip != Trash.end(); TStrip++)
   {if(this->Pars.GetPlot()) PlotStripCadre(**TStrip,clNavy,psSolid,2,Pars);
    List.erase(*TStrip);}
   Trash.clear();

 }  // end if(this->Update || this->Stock.empty())

  // Choose the Strip with the biggest Tolerance from the Stock list

  list< list<strip>::iterator>::iterator IStrip;
  if(this->Stock.empty())
   {this->SelectedStrip=NULL;}
  else
  {
    for (IStrip=this->Stock.begin(); IStrip != this->Stock.end(); IStrip++)
    {

      //if(((*IStrip)->GetPenalty() <= AuxPenalty) && (IStrip->operator->()->GetTwin().GetTol() > Bigger))  // Choose the Strip with the biggest Tolerance.
      if((IStrip->operator->()->GetTwin().GetTol() > Bigger))  // Choose the Strip with the biggest Tolerance.
          {this->SelectedStrip=*IStrip;
           Bigger=IStrip->operator->()->GetTwin().GetTol();
           this->PStock=IStrip;
           //AuxPenalty=(*IStrip)->GetPenalty();
          }                             // Point to Stock element
    }
  }
}//-----------------------------------------------------------------------------

// Create Model
void fstar::CreateModel(box &X,fstream &fout)
{
  tree* Tree=&this->Model.GetTree();
  cell* Cell=&this->Model.GetCell();

 if(this->Model.Function!=NULL){(*this->Model.Function)(X,this->Model,this->Pars);}   // Initalize Model & Parameters.

  Tree->NormalizeConstraint(this->Pars);

  CreateCellFromTree(*Cell,*Tree);
  AssignCellToTree(*Tree,*Cell,false);
  Tree->Diff();                // Symbolic Differentiation
//  Tree->ToLatex("arbre.tex");
  Tree->ConvertTreesToLists();

  list<model>::iterator Constraint;
  for (Constraint=this->Model.GetConstraints().begin(); Constraint != this->Model.GetConstraints().end(); Constraint++)
  {
   Tree=&Constraint->GetTree();
   Cell=&Constraint->GetCell();

   CreateCellFromTree(*Cell,*Tree);
   AssignCellToTree(*Tree,*Cell,false);
   Tree->Diff();                // Symbolic Differentiation
   Tree->ConvertTreesToLists();
  }
  Model.AssignXToConstraints();

  AssignUniIncidence(*Tree,*Cell);
  Tree->AssignNonUnifNodes();

  //AssignXCrossDers(*Tree,*Cell);
}//-----------------------------------------------------------------------------

/** \fn void fstar::Init(box &X,fstream &fout)
    \brief Initialize Fstar problem.*/
void fstar::Initialize(box &X,fstream &fout)
{
 this->List.clear();                         // Clear List of strips.
 this->Twin=twin("dn");                      // Initialize twin
 this->AccTwin=twin("dd");                   // Initialize Accumulated twin
 this->Update=true;                          // Update Twin Flag = true.

 this->Twin.SetInn(NaN());
 this->Twin.SetOut(NaN());

 this->AccTwin.SetInn(NaN());
 this->AccTwin.SetOut(NaN());

 /*
 if(this->Pars.GetOptim()==1)
   {this->AccTwin.SetSup(PInfinity());}
 else if(this->Pars.GetOptim()==2)
   {this->AccTwin.SetInf(MInfinity());}
 */
 
 int Print=this->Pars.GetDebug();             // Print Flag

 PrintMsg("########################################################",fout,Print);
 PrintMsg("Fstar Algorithm",fout,Print);
 PrintMsg("By Pau Herrero & M.A. Sainz - MiceLab - 2004",fout,Print);
 PrintMsg("########################################################",fout,Print);

 PrintMsg("Objective Function:",fout,Print);
 PrintMsg(this->Model.GetName(),fout,Print);

 CreateCellFromTree(Model.GetCell(),Model.GetTree()); 

 AssignUniModal(Model.GetTree(),Model.GetCell());       // Assign Modality

 Model.GetTree().AssignTOptimality();

 strip Strip(Model.GetCell());                        // Create a Strip from the Cell.

 AssignPlotIndexes(Strip,Pars);

 this->List.push_back(Strip);
 list<strip>::iterator PStrip=this->List.begin();

 //this->AssignXWidths();

 this->InsertStrip(PStrip,false,fout);           // Insert Strip to main List.

 this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

 this->Twin;
}//-----------------------------------------------------------------------------

void fstar::EvaluateProj(box& X,ivalDb &Ival,fstream& fout)
{
tree* T=&this->GetModel().GetTree();
cell Cell;

CreateCellFromTree(Cell,*T);

AssignCellToTree(*T,Cell,false);

ivalDb Outer=T->EvaluateFunct("Out",true,true,itrue,'i');
ivalDb Inner=T->EvaluateFunct("Inn",false,false,itrue,'i');
this->Twin.SetOut(Outer);
this->Twin.SetInn(Inner);

}//-----------------------------------------------------------------------------



/** \fn  void Run(box &X, fstream &fout)
    \brief Run main algoritm. */
void fstar::Run(box& X,fstream& fout)
{
 int Step=0;                                    // Step Counter
 double CTime;                                  // Computation Time
 clock_t CStart = clock();                      // Start internal clock.
 clock_t CStop;                                 // Final Time

 this->Initialize(X,fout);

 int Print=Pars.GetDebug();          // Get Print Flag
 this->DetermineModality();

 while (!this->List.empty())      // While List is not empty do.
 {
  PrintMsg("######################################################",fout,Print);

  if(Print) {fout << "Step Number:"; PrintNum(Step,fout,Print);}

  this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

  if(this->SelectedStrip==NULL)                // If No Strip Matching => Break
  {PrintMsg("No Strip Matching",fout,Print);
   this->Pars.SetFinish("Matching");
   break;}

   this->Twin;

  if(this->IsToleralceReached())
   {PrintMsg("Distance < Tol",fout,Print);
    this->Pars.SetFinish("Tolerance");
    break;}

  CStop = clock();                               // Stop Internal Clock.
  CTime=(CStop-CStart)/CLK_TCK;
  if (CTime > this->Pars.GetTime())              // Break: Time Limit Exceded.
   {PrintMsg("TimeLimit Exceded",fout,Print);
    this->Pars.SetFinish("Time");
    break;}

   if(this->Pars.GetInclusion()!=0 && IsInclusionStop())
   {PrintMsg("Stop By Inclusion",fout,Print);
    break;}

   else
    {this->Bisect(fout);}                       // Bisect

  Step++;                                       // Increase step counter

  if(this->Pars.GetScreenOut())
   {this->PrintScreen(Step);}

 }  // End While

 this->Update=true;
 this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

 if(this->List.empty())
  {PrintMsg("Empty Agenda",fout,Print);
   this->Pars.SetFinish("Empty Agenda");}      // Set Stop Criterion

 if(Pars.GetCells())  
 {this->AddListBufferToMinMaxLists();}           // Add List and Buffer to the minmax lists.

 if(this->Pars.GetSolution())
  {this->PrintSolution(CTime);}                  // Print Solution to a file.
}//-----------------------------------------------------------------------------


/** \fn void fstar::AssignBisectDim(double &XpWidth,double &XiWidth)
    \brief Assign Bisection Dimention and Get Biggest Proper and Improper widths.
    \param XpWidth: Biggest Proper width of the selected Strip.
    \param XiWidth: Biggest Improper width of the selected Cell.*/
void fstar::AssignBisectDim(double &XpWidth,double &XiWidth)
{ list<strip>::iterator Strip=this->SelectedStrip;
  list<cell>::iterator  Cell=Strip->GetSelectCell();

//  if(Strip==NULL || Strip->GetZeroIncluded())
  if(Strip==NULL)
   {XpWidth=MInfinity();}
  else
  {XpWidth=Strip->GetNonMonoXpBigWidth();
   //XpWidth=XpWidth/this->Pars.GetXWidths()[Strip->GetXpDim()];
   } // Get non monotonic biggest proper width.

//  if(Cell==NULL || Cell->GetZeroExcluded() || Strip->GetZeroExcluded())
  if(Cell==NULL)
   {XiWidth=MInfinity();}
  else
   {XiWidth=Cell->GetNonMonoXiBigWidth();
    //XiWidth=XiWidth/this->Pars.GetXWidths()[Cell->GetXp().size()+Cell->GetXiDim()];
    }  // Get non monotonic biggest improper width.
}//-----------------------------------------------------------------------------

void fstar::Bisect(fstream &fout)
{
 double XpWidth,XiWidth;
 int Print=this->Pars.GetDebug();
 bool Improve=true;

 list<strip>::iterator Strip=this->SelectedStrip;  // Pointer to the selected strip.
 list<cell>::iterator Cell=Strip->GetSelectCell();

 if(Strip!=NULL)
  {PrintMsg("Selected Strip",fout,Print);
   Strip->Print(fout,Print,false);}
 if(Cell!=NULL)
   {PrintMsg("Selected Cell",fout,Print);
    Cell->Print(fout,Print);}

 this->AssignBisectDim(XpWidth,XiWidth);    // Assign Bisection Dimention.

 if(XpWidth > this->Pars.GetEps() || XiWidth > this->Pars.GetEps())
  {
   //int CellPenalty=Cell->GetPenalty();
   //int StripPenalty=Strip->GetPenalty();

   if(XpWidth >= XiWidth)
   //if(StripPenalty <= CellPenalty && XpWidth > this->Pars.GetEps())
    {PrintMsg("Action: Bisect Strip Xp",fout,Print);
     this->BisectStripXp(Strip,fout);      // Bisect Strip Proper Dimention
     //Improve=this->IsThereImprovement();
     //if(Improve){this->List.erase(Strip); this->Update=true;}
     //else {this->Update=false;}
     this->Stock.erase(this->PStock);

   }
   else
   {PrintMsg("Action: Bisect Cell Xi",fout,Print);
    this->BisectCellXi(Cell,true,fout);         // Bisect Improper Dimention (Cell)
    twin TW=Strip->GetTwin();
    Strip->ComputeUpdateTwin(false,fout,this->Model,this->Pars);    // Compute Strip Twin.
    //Improve=Strip->IsThereImprovement(TW);
    //if(Improve) {Strip->GetCells().erase(Cell); this->Update=true;}
    //else {this->Update=false;}
    PrintMsg("Updated Selected Strip:",fout,Print);
    Strip->Print(fout,Print,true);
    }
  }

 else
 {
   if(Cell==NULL) // No Cell Matching
   {
     PrintMsg("Accumulate Strip: No Cell Matching",fout,Print);
     this->AccumTwin(Strip);              // Accumulate Twin and Store it to the Buffer.
     this->Update=false;
     this->Stock.erase(this->PStock);
   }
   else     // Width < Epsilon
   {
     PrintMsg("Accumulate for Epsilon (Width < Epsilon)",fout,Print);
     if(this->Pars.GetPlot()) PlotCell(*Cell,clYellow,clBlack,bsDiagCross,Pars);
     Strip->GetWareHouse().splice(Strip->GetWareHouse().end(),Strip->GetCells(),Cell); // Store to WareHouse
     Strip->SetUpdate(false);
     Strip->GetStock().erase(Strip->GetPStock());
     Strip->AssignCell(fout,this->Pars); // Assign Cell to bisect.
     this->Update=false;
   }
 }
}//-----------------------------------------------------------------------------

/** \fn  void fstar::BisectStripXp(strip &Strip,fstream &fout)
    \brief Bisect Strip by its proper vector (Xp) by the assigned proper dimention
    into "n" Strips.*/
void fstar::BisectStripXp(list<strip>::iterator Strip,fstream &fout)
{
 int Dim=Strip->GetXpDim();                             // Get Assigned proper dimention to bisect.
 //int Penalty=Strip->GetXp()[Dim].GetPenalty();

 PrintMsg("Bisected Strip Dim:",fout,this->Pars.GetDebug());
 PrintNum(Dim,fout,this->Pars.GetDebug());
 ivalDb Ivalue=Strip->GetXp()[Dim].GetIval();
 double width=Width(Ivalue);
 double Point=width/this->Pars.GetBisect();
 double Inf=Ivalue.GetInf();
 double Sup=Ivalue.GetSup();

 for(int i=0;i<this->Pars.GetBisect()-1;i++)
 {
  Strip->GetXp()[Dim].SetIval(Ival(Inf,Inf+Point)); // Set New Domains to the bisected dimention.
  this->InsertStrip(Strip,true,fout);                      // Insert Strip to the List.
  Inf=Inf+Point;
 }                                                     // Set New Domains to the bisected dimention.
  Strip->GetXp()[Dim].SetIval(Ival(Inf,Sup));
  this->InsertStrip(Strip,false,fout);                      // Insert Strip to the List.
  //Strip->GetXp()[Dim].SetIval(Ivalue);
}//-----------------------------------------------------------------------------

/** \fn  void fstar::BisectCellXi(cell &Cell,fstream &fout)
    \brief Bisect Cell improper vector (Xi) by the chosen dimention.*/
void fstar::BisectCellXi(list<cell>::iterator Cell,bool ComputeCell,fstream &fout)
{
 int Dim=Cell->GetXiDim();                             // Get Assigned improper dimention to bisect.
 //int Penalty=Cell->GetXi()[Dim].GetPenalty();

 int Print=this->Pars.GetDebug();
 PrintMsg("Bisected Cell Dim:",fout,Print);
 PrintNum(Cell->GetXpStrip().size()+Dim,fout,Print);
 ivalDb Ivalue=Cell->GetXi()[Dim].GetIval();
 double width=Width(Ivalue);
 double Point=width/this->Pars.GetBisect();
 double Inf=Ivalue.GetInf();
 double Sup=Ivalue.GetSup();

 for(int i=0;i<this->Pars.GetBisect()-1;i++)
 {
  Cell->GetXi()[Dim].SetIval(Ival(Inf,Inf-Point));   // Set New Domains to the bisected dimention.
  this->InsertCell(Cell,true,ComputeCell,fout);                        // Insert Cell to the Selected Strip.
  Inf=Inf-Point;
 }
 Cell->GetXi()[Dim].SetIval(Ival(Inf,Sup));         // Set New Domains to the bisected dimention.
 this->InsertCell(Cell,false,ComputeCell,fout);                        // Insert Cell to the Selected Strip.
 //Cell->GetXi()[Dim].SetIval(Ivalue);
}//-----------------------------------------------------------------------------

/** \fn  void fstar::InsertStrip(strip &Strip,fstream &fout)
    \brief Insert Strip to the List. The Strip is first inserted and then evaluated
    in order to eliminate it. This strategy is more convenient.*/
void fstar::InsertStrip(list<strip>::iterator Strip,bool Insert,fstream &fout)
{
 list<strip>::iterator PStrip;
 int Print=this->Pars.GetDebug();

 PrintMsg("Action: Strip Insertion?",fout,Print);

 if(Insert)
 {this->List.push_back(*Strip);             // Insert Strip to the List.
  PStrip=(--this->List.end());             // Pointer to the last Strip.
  PStrip->SetUpdate(true);}                 // Set Update flag to true.
 else
 {PStrip=Strip;}

 PStrip->ComputeUpdateTwin(true,fout,this->Model,this->Pars);    // Compute Strip Twin.

 PStrip->Print(fout,Print,false);               // Print Strip.

 iboolean Consistent=PStrip->GetConsist();  // Get Consistency (Constrained optimization)

 if(Consistent==ifalse)                             // Erase Strip from List.
    {PrintMsg("Eliminate Strip: Inclusion",fout,Print);
     this->List.erase(PStrip);}

 else if(this->IsIncludedStrip(*PStrip,fout))       // Is Included -> Eliminate
    {PrintMsg("Eliminate Strip: Inclusion",fout,Print);
     this->List.erase(PStrip);}

 else if(Consistent==itrue && PStrip->IsTotalyMono()) // Is Monotonic.
     {
     PrintMsg("Accumulate Strip: Monotony",fout,Print);
     this->AccumTwin(PStrip);             // Accumulate Twin and Store it to the Buffer.
     if(this->Pars.GetPlot()) {PlotStripCadre(*PStrip,clPurple,psSolid,2,Pars);}
     }
     
 else    // Do nothing because the Strip is Inside
     {
      PrintMsg("Insert Strip",fout,Print);
      if(this->Pars.GetPlot()) {PlotStripCadre(*PStrip,clBlack,psSolid,0.3,Pars);}
       //if(Insert) {this->InsertedStrips.push_back(PStrip);}
     }
}//-----------------------------------------------------------------------------

/** \fn  void fstar::InsertCell(cell &Cell,fstream &fout)
    \brief InsertCell to the Selected Strip.*/
void fstar::InsertCell(list<cell>::iterator Cell,bool Insert,bool ComputeCell,fstream &fout)
{
 int Print=this->Pars.GetDebug();
 list<strip>::iterator Strip=this->SelectedStrip;   // Pointer to the Selected Strip.
 tree* Tree=&this->Model.GetTree();

 PrintMsg("Action: Cell Insertion?",fout,Print);

 Cell->SetConsist(this->Model.ConstraintsConsistency(*Cell));
 iboolean Consistent=Cell->GetConsist();     // Get Consistency.

 if(Consistent==ifalse)  // Elimination Criteria (Inconsistent)
   {PrintMsg("Eliminate Cell: Inconsistent",fout,Print);}

 else
 {
   if(ComputeCell)
    {Cell->ComputeTwin(this->Model.GetTree(),Strip->GetTOptimal(),this->Pars);}   // Compute Twin with Tree
   
   /*
   if(ComputeCell && Strip->IsIncluded(*Cell,fout,this->Pars))
    {
     PrintMsg("Accumulate Cell: Inclusion",fout,Print);        // Add Cell To the WareHouse
     if(Insert)
     {Strip->AddCellToWareHouse(*Cell);}
     else
     {Strip->GetWareHouse().splice(Strip->GetWareHouse().end(),Strip->GetCells(),Cell);}
    }

   // Insert to the WareHouse: Is Monotonic
   else if(ComputeCell && Consistent==itrue && Cell->IsXiTotalyMono())
    {
     if(this->Pars.GetPlot()) {PlotCell(*Cell,clPurple,clBlack,bsSolid,Pars);}
     PrintMsg("Accumulate Cell: Monotony",fout,Print);
     if(Insert)
     {Strip->AddCellToWareHouse(*Cell);}
     else
     {Strip->GetWareHouse().splice(Strip->GetWareHouse().end(),Strip->GetCells(),Cell);}
     Strip->SetUpdate(true);
    }
   */

   //  Else: Insert to the Cells list
   //else
    {
     PrintMsg("Insert Cell",fout,Print);
     if(this->Pars.GetPlot()) {PlotCellCadre(*Cell,clBlack,psSolid,0.1,Pars);}
     if(Insert)
      {Strip->AddCellToCells(*Cell);
       //Strip->GetInsertedCells().push_back(--Strip->GetCells().end());
      }
    }
 }
}//-----------------------------------------------------------------------------

/** \fn  bool fstar::IsIncluded(strip &Strip,fstream &fout)
    \brief Return If The Strip is Included.*/
bool fstar::IsIncludedStrip(strip &Strip,fstream &fout)
{
 int OptimLevel=this->Pars.GetOptim();
 int Print=this->Pars.GetDebug();

 // Inclusion Criterion
 if((OptimLevel==0 || OptimLevel==3) && IsIn(Strip.GetTwin().GetOut(),this->Twin.GetInn()))

     {
     //PrintMsg("Inclusion Criterion",fout,Print);
     if(this->Pars.GetPlot()) PlotStripCadre(Strip,clRed,psSolid,2,Pars);
     return true;
     }

 // Minimization Criterion (MinMax Optimization)
 else if(OptimLevel==1 && !IsNaN(this->Twin.GetInn().GetInf()) && (Strip.GetTwin().GetOut().GetInf() > this->Twin.GetInn().GetInf()))
     {
     //PrintMsg("Minimization Criterion",fout,Print);
     if(this->Pars.GetPlot()) PlotStripCadre(Strip,clSilver,psSolid,2,Pars);
     return true;
     }

 // Maximization Criterion (MinMax Optimization)
 else if(OptimLevel==2 && !IsNaN(this->Twin.GetInn().GetSup()) && (Strip.GetTwin().GetOut().GetSup() < this->Twin.GetInn().GetSup()))
     {
     //PrintMsg("Maximization Criterion",fout,Print);
     if(this->Pars.GetPlot()) PlotStripCadre(Strip,clTeal,psSolid,2,Pars);
     return true;
     }

 else
  {return false; }
}//-----------------------------------------------------------------------------

bool fstar::IsThereImprovement(void)
{
 list< list<strip>::iterator >::iterator Strip;
 ivalDb Ival=ivalDb(NaN());
 twin NEWTW=twin(Ival,Ival);
 bool Improved;

 for(Strip=this->InsertedStrips.begin(); Strip != this->InsertedStrips.end(); Strip++)
 {NEWTW= NEWTW || (*Strip)->GetTwin();}

 if(this->InsertedStrips.size()>0){NEWTW.AssignTol();}
 else{NEWTW.SetTol(MInfinity());}

 twin OLDTWIN=this->SelectedStrip->GetTwin();

 if(NEWTW.GetTol() < OLDTWIN.GetTol())
 {Improved=true;}
 else
 {
  for(Strip=this->InsertedStrips.begin(); Strip != this->InsertedStrips.end(); Strip++)
  {this->List.erase(*Strip);}
  Improved=false;
  //this->SelectedStrip->Penalize();
  //this->SelectedStrip->GetXp()[this->SelectedStrip->GetXpDim()].Penalize();
 }

 this->InsertedStrips.clear();

 return Improved;
}//-----------------------------------------------------------------------------


// Inclusion Stop Conditions
bool fstar::IsInclusionStop(void)
{

  if(this->Pars.GetInclusion()==1)
  {
   if(this->Twin.GetOut().IsInterval() && IsIn(this->Twin.GetOut(),Zero()))
    {this->Pars.SetConsistency(itrue);
     return true;}
   else if (this->Twin.GetInn().IsInterval() && !IsIn(this->Twin.GetInn(),Zero()))
    {this->Pars.SetConsistency(ifalse);
     return true;}
  }
  else if(this->Pars.GetInclusion()==2)
  {
   if(this->Twin.GetInn().IsInterval() && !IsIn(this->Twin.GetInn(),Zero()))
    return true;
   else if(this->Twin.GetOut().IsInterval() && IsIn(this->Twin.GetOut(),Zero()))
    return true;
   else
    {return false;}
  }
  else if(this->Pars.GetInclusion()==3)
  {
   if(this->Twin.GetOut().IsProp())
   {return true;}
   else return false;
  }
  else if(this->Pars.GetInclusion()==4)
  {
   if(this->Twin.GetOut().IsImpr())
   {return true;}
   else return false;
  }

  else if(this->Pars.GetInclusion()==5)        // Squaltrack condition
  {
   if(IsOut(this->Twin.GetOut(),this->AuxIval) || !IsOut(this->Twin.GetInn(),this->AuxIval))
    {return true;}
   else
    {return false;}
  }
}//-----------------------------------------------------------------------------

/** \fn  bool fstar::IsProp()
    \brief .*/
bool fstar::IsProp()
{bool Prop=true;
 int i=0;
 while(i<this->Model.GetTree().GetNodeVars().size() && Prop)
 {Prop=this->Model.GetTree().GetNodeVars()[i].IsProp();
  i++;}
  return Prop;
}//-----------------------------------------------------------------------------

/** \fn  bool fstar::IsImpr()
    \brief .*/
bool fstar::IsImpr()
{bool Impr=true;
 int i=0;
 while(i<this->Model.GetTree().GetNodeVars().size() && Impr)
 {Impr=this->Model.GetTree().GetNodeVars()[i].IsImpr();
  i++;}
  return Impr;
}//-----------------------------------------------------------------------------

bool fstar::IsZeroIncluded(void)
{
 if(IsIn(this->Twin.GetOut(),Ival(0,0)))
  return true;
 else
  return false;
}//-----------------------------------------------------------------------------

bool fstar::IsZeroExcluded(void)
{
 if (!IsIn(this->Twin.GetInn(),Ival(0,0)))
  return true;
 else
  return false;
}//-----------------------------------------------------------------------------

bool fstar::IsToleralceReached(void)
{
  if((this->Pars.GetOptim()==1 && this->Twin.GetInfTol() <= this->Pars.GetTol()) ||  // Tolerance Stop Criteria
     (this->Pars.GetOptim()==2 && this->Twin.GetSupTol() <= this->Pars.GetTol()) ||
     (this->Twin.GetTol() <= this->Pars.GetTol()))
  {return true;}
  else
  {return false;}
}//-----------------------------------------------------------------------------

bool fstar::PrintScreen(int Step)
{
 string Modality=this->Model.GetModality();

 cout  << "Step:" << Step << " ";
 if(this->Pars.GetOptim()==1)
   {if(Modality=="Prop") {cout << "Min: ";}
    else if(Modality=="Impr") {cout << "Max: ";}
    else  {cout << "MinMax: ";}
    cout << this->Twin.GetInf() << " Tol: " << this->Twin.GetInfTol() << endl;}
 else if(this->Pars.GetOptim()==2)
   {if(Modality=="Prop") {cout << "Max: ";}
    else if(Modality=="Impr") {cout << "Min: ";}
    else{cout << "MaxMin: ";}
    cout << this->Twin.GetSup() << " Tol: " << this->Twin.GetSupTol() << endl;}
 else {cout << "Twin: " << this->Twin << endl;}
}//-----------------------------------------------------------------------------

void fstar::DetermineModality()
{
  if(this->IsProp()) {this->Model.SetModality("Prop");}
  else if(this->IsImpr()) {this->Model.SetModality("Impr");}
  else {this->Model.SetModality("Du");}
}//-----------------------------------------------------------------------------

void fstar::Delete()
{
  this->GetModel().GetTree().DeleteTree();
  list<model>::iterator Constraint;
  for(Constraint=this->GetModel().GetConstraints().begin();Constraint!=this->GetModel().GetConstraints().end();Constraint++)
  {Constraint->GetTree().DeleteTree();}
}//-----------------------------------------------------------------------------

void fstar::AssignRelEps(box& X,double Percentage)
{double BSize=MInfinity();
 double Size;
 for(int i=0;i<X.Vars.size();i++)
 {Size=Width(X[i].GetIval());
  if(Size>BSize)
  BSize=Size;
 }
 this->Pars.SetEps(BSize*Percentage);
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// MinMax Member functions                                                    //
////////////////////////////////////////////////////////////////////////////////

double GetMaxCellWidth(list<cell> &List)
{
  list<cell>::iterator Cell;
  double MaxWidth,Width;
  MaxWidth=MInfinity();

  for(Cell=List.begin(); Cell != List.end(); Cell++)
  {
   Width=Cell->GetXBigWidth();
   if(Width>=MaxWidth){MaxWidth=Width;}
  }
 return MaxWidth;
}//-----------------------------------------------------------------------------

/** \fn void fstar::SelectStrip(void)
    \brief Add List Buffer to MinMax Lists*/
void fstar::AddListBufferToMinMaxLists(void)
{
  list<strip>::iterator Strip;
  ivalDb StripTwInf,StripTwSup,GlobTwInf,GlobTwSup;
  int Optim=Pars.GetOptim();  // 1==[MinMax]; 2==[MaxMin]; 3==[MinMax,MaxMin]

  GlobTwInf=this->Twin.GetInf();
  GlobTwSup=this->Twin.GetSup();

  for (Strip=List.begin(); Strip != List.end(); Strip++) // Iterates List.
  {
     StripTwInf=Strip->GetTwin().GetInf();
     StripTwSup=Strip->GetTwin().GetSup();

     if(!IsIn(Strip->GetTwin().GetOut(),Twin.GetInn()))
     {this->AddStripToMinMaxLists(*Strip);}         // Add Strip To MinMax Lists.
  }

  for (Strip=Buffer.begin(); Strip != Buffer.end(); Strip++)  // Iterates Buffer.
  {
     StripTwInf=Strip->GetTwin().GetInf();
     StripTwSup=Strip->GetTwin().GetSup();

     if(!IsIn(Strip->GetTwin().GetOut(),Twin.GetInn()))
     {this->AddStripToMinMaxLists(*Strip);}         // Add Strip To MinMax Lists.
  }

 if(this->MinMaxList.size()>1){this->MinMaxList.sort();}                                // Sort MinMax List
 if(this->MaxMinList.size()>1){this->MaxMinList.sort();}                                // Sort MaxMin List
}//-----------------------------------------------------------------------------

/** \fn void fstar::AddStripToMinMaxLists(strip &Strip)
    \brief Add Strip To MinMax Lists*/
void fstar::AddStripToMinMaxLists(strip &Strip)
{ list<cell>::iterator Cell,PCell;
  int Optim=Pars.GetOptim();  // 1==[MinMax]; 2==[MaxMin]; 3==[MinMax,MaxMin]
  ivalDb CellTwInf,CellTwSup,GlobTwInf,GlobTwSup;
  list<list<cell>::iterator> MinMaxBuffer, MaxMinBuffer;
  list<list<cell>::iterator>::iterator ICell;

  GlobTwInf=this->Twin.GetInf();
  GlobTwSup=this->Twin.GetSup();

  // Add Cells from Cells list to MinMax MaxMin Lists.
  for(Cell=Strip.GetCells().begin(); Cell != Strip.GetCells().end(); Cell++)
  {
     CellTwInf=Cell->GetTwin().GetInf();
     CellTwSup=Cell->GetTwin().GetSup();
     
     if(Optim!=2  && (CellTwInf <<= GlobTwInf))       // Correcte
      {
       this->MinMaxList.push_back(*Cell);
       PCell=this->MinMaxList.end()--;
       PCell->SetXp(Strip.GetXp());
       PCell->SetXpStrip(NULL);
       }

     if(Optim!=1 && (CellTwSup <<= GlobTwSup))      // Correcte
      {
       this->MaxMinList.push_back(*Cell);
       PCell=this->MaxMinList.end()--;
       PCell->SetXp(Strip.GetXp());
       PCell->SetXpStrip(NULL);
       }

  }
  // Add Cells from WareHouse list to MinMax MaxMin Lists.
  for(Cell=Strip.GetWareHouse().begin(); Cell != Strip.GetWareHouse().end(); Cell++)
  {
     CellTwInf=Cell->GetTwin().GetInf();
     CellTwSup=Cell->GetTwin().GetSup();

     if(Optim!=2  && (CellTwInf <<= GlobTwInf))
      {
       this->MinMaxList.push_back(*Cell);
       PCell=this->MinMaxList.end()--;
       PCell->SetXp(Strip.GetXp());
       PCell->SetXpStrip(NULL);
       }

     if(Optim!=1 && (CellTwSup <<= GlobTwSup))
      {
       this->MaxMinList.push_back(*Cell);
       PCell=this->MaxMinList.end()--;
       PCell->SetXp(Strip.GetXp());
       PCell->SetXpStrip(NULL);
       }
  }
}//-----------------------------------------------------------------------------

/** \fn cell fstar::JoinMeetMinMaxList(void)
    \brief Create a Cell from the MinMax List which is the union of them.*/
void fstar::JoinMinMaxList(void)
{
  list<cell>::iterator Cell,ICell;
  tree* Tree=&this->GetModel().GetTree();
  ivalDb CellTwInf, GlobTwInf;

  GlobTwInf=this->Twin.GetInf();

  while(!this->MinMaxList.empty())
  {
   ICell=this->MinMaxList.begin();
   ICell->SetXp(ICell->GetXpStrip());
   
   Cell=this->MinMaxList.begin();
   Cell++;

   while(Cell!=this->MinMaxList.end())
   {
     for(Cell=this->MinMaxList.begin(); Cell != this->MinMaxList.end(); Cell++)
     {
      if(Cell!=ICell && IsIntersect(*ICell,*Cell))
       {ICell->Join(*Cell);
        this->MinMaxList.erase(Cell);
        Cell=this->MinMaxList.begin();
        Cell++;
        break;}
     }
   }

   ICell->SetConsist(Model.ConstraintsConsistency(*ICell));
   ICell->ComputeTwinProp(*Tree,"MinMax",false,this->Pars);
   
   ivalDb CellOut=ICell->GetTwin().GetOut();
   if(CellOut <<= GlobTwInf)
   {
    this->MinMaxs.splice(this->MinMaxs.end(),this->MinMaxList,ICell);}
   else {this->MinMaxList.erase(ICell);}
  }

 this->MinMaxs.sort();
}//-----------------------------------------------------------------------------

/** \fn cell fstar::JoinMeetMinMaxList(void)
    \brief Create a Cell from the MaxMin List which is the union of them.*/
void fstar::JoinMaxMinList(void)
{
 list<cell>::iterator Cell,ICell;
 tree* Tree=&this->GetModel().GetTree();
 ivalDb CellTwSup, GlobTwSup;

 GlobTwSup=this->Twin.GetSup();

 while(!this->MaxMinList.empty())
 {
   ICell=this->MaxMinList.begin();
   ICell->SetXp(ICell->GetXpStrip());     // Copy Xp Strip Vector

   Cell=this->MaxMinList.begin();
   Cell++;

   while(Cell!=this->MaxMinList.end())
   {
     for(Cell=this->MaxMinList.begin(); Cell != this->MaxMinList.end(); Cell++)
     {
      if(Cell!=ICell && IsIntersect(*ICell,*Cell))
       {ICell->Join(*Cell);
        this->MaxMinList.erase(Cell);
        Cell=this->MaxMinList.begin();
        Cell++;
        break;}
     } // End For
   }

   ICell->SetConsist(Model.ConstraintsConsistency(*ICell));
   ICell->ComputeTwinProp(*Tree,"MaxMin",false,this->Pars);
   ivalDb CellOut=ICell->GetTwin().GetOut();
   if(CellOut <<= GlobTwSup)
    {this->MaxMins.splice(this->MaxMins.end(),this->MaxMinList,ICell);}
   else
     {this->MaxMinList.erase(ICell);}
 }

 this->MaxMins.sort();
}//-----------------------------------------------------------------------------

/** \fn  void fstar::PrintSolution(double &CTime)
    \brief Print Solution To File.*/
void fstar::PrintSolution(double &CTime)
{fstream fout;
 list<cell>::iterator Cell;
 cell Cella;
 bool Proper=this->IsProp();
 bool Improper=this->IsImpr();


 string File=this->Pars.GetFile() + "Solution.txt";

 fout.open(File.c_str(),ios::out);
 
 /*
 fout << "########################################################" << endl;
 fout << "Fstar Algorithm - Solution File " << endl;
 fout << "By Pau Herrero & M.A. Sainz - MiceLab - 2004" << endl;
 fout << "########################################################" << endl;

 fout << "Function Name:" << Model.GetName() << "\n\n";
 fout << "Stop Condition:" << Pars.GetFinish() << "\n\n";
 */

 fout.precision(3);
 if(CTime>this->Pars.GetTime())
 {fout << "WARNING: TimeOut => Partial solution provided" << "\n\n";}
 else
 {fout << "Computation Time (s):" << CTime << "\n\n";}

 double Tol=this->Pars.GetTol();
 int n=0;
 while(int(Tol)==0)
 {Tol=Tol*10;
  n++;}
 fout.precision(n);
 fout << scientific;

 if(this->Pars.GetInclusion()==1)
 {fout << "Consistency: ";
  if(IsIn(this->Twin.GetOut(),Zero()))
  {fout << "True" << endl << endl;}
  else if(!IsIn(this->Twin.GetInn(),Zero()))
  {fout << "False" << endl << endl;}
  else {fout << "Perhaps" << endl << endl;}
 }
 
 if(Pars.GetOptim()==1)
 {
  if(this->IsProp()) fout << "Min Approximation:";
  else if(this->IsImpr()) fout  << "Max Approximation:";
  else fout << "MinMax Approximation:";
  fout <<  this->Twin.GetInf() << endl;
  fout << "Tol: " << this->Twin.GetInfTol() << "\n\n";
 }
 else if(Pars.GetOptim()==2)
 {
  if(Proper) fout << "Max Approximation:";
  else if(Improper) fout << "Min Approximation:";
  else fout << "MaxMin Approximation:";
  fout <<  this->Twin.GetSup() <<  endl;
  fout << "Tol: " << this->Twin.GetSupTol() << "\n\n";
 }
 else
 {
  fout << "Inner Approximation:" <<  this->Twin.GetInn() << "\n";
  fout << "Outer Approximation:" <<  this->Twin.GetOut() << endl;
  fout << "Tol: " << this->Twin.GetTol() << "\n\n";
 }

if(Pars.GetOptim()!=0)
{
 if(Pars.GetOptim()!=2 && !this->MinMaxList.empty())
 {

    if(Proper) fout << "Boxes Min List:" << endl << endl;
    else if(Improper) fout  << "Boxes Max List:" << endl << endl;
    else fout << "Boxes MinMax List:" << endl << endl;
    fout  << "List Size: " << this->MinMaxList.size() << endl;
    fout  << "Biggest Width:" << GetMaxCellWidth(this->MinMaxList) << endl << endl;
    for (Cell=this->MinMaxList.begin(); Cell != this->MinMaxList.end(); Cell++)
    {Cell->Print(fout,1);}


    this->JoinMinMaxList();
    if(Proper) fout << "Joined Boxes Min List:" << endl << endl;
    else if(Improper) fout  << "Joined Boxes Max List:" << endl << endl;
    else fout  << "\n" << "Joined Boxes MinMax List:" << "\n";
    for (Cell=this->MinMaxs.begin(); Cell != this->MinMaxs.end(); Cell++)
    {Cell->Print(fout,1);
     if(this->Pars.GetPlot()) {PlotCellCadre(*Cell,clRed,psSolid,3,Pars);}}
 }

 if(Pars.GetOptim()!=1 && !this->MaxMinList.empty())
 {

   if(Proper) fout << "Boxes Max List:" << endl << endl;
   else if(Improper) fout  << "Boxes Min List:" << endl << endl;
   else fout << "Boxes MaxMin List:" << endl << endl;
   fout  << "List Size: " << this->MaxMinList.size() << endl << endl;
   fout  << "Biggest Width:" << GetMaxCellWidth(this->MaxMinList) << endl;
   for (Cell=this->MaxMinList.begin(); Cell != this->MaxMinList.end(); Cell++)
   {Cell->Print(fout,1);}

   
   this->JoinMaxMinList();
   if(Proper) fout << "Joined Boxes Max List:" << endl << endl;
   else if(Improper) fout  << "Joined Boxes Min List:" << endl << endl;
   else fout  << "\n" << "Joined Boxes MaxMin Cells:" << "\n";
    for (Cell=this->MaxMins.begin(); Cell != this->MaxMins.end(); Cell++)
    {Cell->Print(fout,1);
     if(this->Pars.GetPlot()) {PlotCellCadre(*Cell,clGreen,psSolid,3,Pars);}}
  }
}

 fout.close();
}//-----------------------------------------------------------------------------


//##############################################################################
 /*
void fstar::RunProj(box& X,ivalDb &Ival,fstream& fout)
{
 int Step=0;                                    // Step Counter
 double CTime;                                  // Computation Time
 clock_t CStart = clock();                      // Start internal clock.
 clock_t CStop;                                 // Final Time

 this->Initialize(X,fout);

 int Print=Pars.GetDebug();          // Get Print Flag
 this->DetermineModality();

 while (!this->List.empty())      // While List is not empty do.
 {
  PrintMsg("######################################################",fout,Print);

  if(Print) {fout << "Step Number:"; PrintNum(Step,fout,Print);}
  
  this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

  if(this->SelectedStrip==NULL)                // If No Strip Matching => Break
  {PrintMsg("No Strip Matching",fout,Print);
   this->Pars.SetFinish("Matching");
   break;}

   this->Twin;

  if(this->IsToleralceReached())
   {PrintMsg("Distance < Tol",fout,Print);
    this->Pars.SetFinish("Tolerance");
    break;}

  CStop = clock();                               // Stop Internal Clock.
  CTime=(CStop-CStart)/CLK_TCK;
  if (CTime > this->Pars.GetTime())              // Break: Time Limit Exceded.
   {PrintMsg("TimeLimit Exceded",fout,Print);
    this->Pars.SetFinish("Time");
    break;}

   if(this->Pars.GetInclusion()!=0 && IsInclusionStop())
   {PrintMsg("Stop By Inclusion",fout,Print);
    break;}
   
   ivalDb Inn=this->Twin.GetInn();
   if(IsEmpty(this->Twin.GetOut(),Ival) || IsIntersect(Inn,Ival))
   {PrintMsg("Proj Stop",fout,Print);
    break;}

   else
    {this->Bisect(fout);}                       // Bisect

  Step++;                                       // Increase step counter

  if(this->Pars.GetScreenOut())
   {this->PrintScreen(Step);}

 }  // End While

 this->Update=true;
 this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

 if(this->List.empty())
  {PrintMsg("Empty Agenda",fout,Print);
   this->Pars.SetFinish("Empty Agenda");}      // Set Stop Criterion

 if(Pars.GetCells())  
 {this->AddListBufferToMinMaxLists();}           // Add List and Buffer to the minmax lists.

 if(this->Pars.GetSolution())
  {this->PrintSolution(CTime);}                  // Print Solution to a file.
}//-----------------------------------------------------------------------------
*/

void fstar::AssignXWidths(void)
{
 cell* Cell=&this->Model.GetCell();
 //this->Pars.GetEpsilons().resize(Cell->GetDim());
 this->Pars.GetXWidths().resize(Cell->GetDim());
 for(int i=0;i<Cell->GetDim();i++)
 {//this->Pars.GetEpsilons()[i]=this->Pars.GetEps()*Width((Cell->operator[](i)).GetIval());
  this->Pars.GetXWidths()[i]=Width((Cell->operator[](i)).GetIval());}
}//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
iboolean fstar::Propagate(box& X,fstream &fout,string &Mode)
{

 this->GetModel().AssignXToCell(X);

 int Counter=1;

 this->GetModel().GetCell().ComputeMonos(this->GetModel().GetTree(),"Full");
 
 this->GetModel().GetTree().ForwardPropagation(fout,Counter,Mode);
 Counter--;
 this->GetModel().GetTree().BackwardPropagation(fout,Counter);

 this->GetModel().AssignCellToX(X);

 return X.GetConsistent();
}//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
iboolean fstar::CheckPunctualConsistency(box &X,bool Mode)
{
   twin TW;
   ivalDb Ival0,Ival1;
   bool Consistency=true;
   double Mid0,Inf0,Sup0,Mid1,Inf1,Sup1;
   fstream fout;
   iboolean GConsistency=itrue;

   Ival0=X[0].GetIval();
   Ival1=X[1].GetIval();

   Mid0=Center(Ival0);
   Inf0=Ival0.GetInf();
   Sup0=Ival0.GetSup();

   Mid1=Center(Ival1);
   Inf1=Ival1.GetInf();
   Sup1=Ival1.GetSup();

   this->GetModel().GetTree().AssignModes(Mode,this->GetPars());

   X[0].SetIval(Mid0);
   X[1].SetIval(Mid1);
   this->GetModel().AssignXToNodeVars(X,Mode);
   this->Run(X,fout);
   TW=this->GetTwin();
   if(Mode)Consistency=IsIn(this->GetTwin().GetOut(),Zero());
   else Consistency=!IsIn(this->GetTwin().GetInn(),Zero());
   if(!Consistency) {GConsistency=ifalse;}

   if(GConsistency==itrue)
   {
     X[0].SetIval(Inf0);
     X[1].SetIval(Inf1);
     this->GetModel().AssignXToNodeVars(X,Mode);
     this->Run(X,fout);
     TW=this->GetTwin();
     if(Mode)Consistency=IsIn(this->GetTwin().GetOut(),Zero());
     else Consistency=!IsIn(this->GetTwin().GetInn(),Zero());
     if(!Consistency) {GConsistency=iperhaps;}
   }

   if(GConsistency==itrue)
   {
     X[0].SetIval(Sup0);
     X[1].SetIval(Sup1);
     this->GetModel().AssignXToNodeVars(X,Mode);
     this->Run(X,fout);
     TW=this->GetTwin();
     if(Mode)Consistency=IsIn(this->GetTwin().GetOut(),Zero());
     else Consistency=!IsIn(this->GetTwin().GetInn(),Zero());
     if(!Consistency) {GConsistency=iperhaps;}
   }

   if(GConsistency==itrue)
   {
     X[0].SetIval(Inf0);
     X[1].SetIval(Sup1);
     this->GetModel().AssignXToNodeVars(X,Mode);
     this->Run(X,fout);
     TW=this->GetTwin();
     if(Mode)Consistency=IsIn(this->GetTwin().GetOut(),Zero());
     else Consistency=!IsIn(this->GetTwin().GetInn(),Zero());
     if(!Consistency) {GConsistency=iperhaps;}
   }

   if(GConsistency==itrue)
   {
     X[0].SetIval(Sup0);
     X[1].SetIval(Inf1);
     this->GetModel().AssignXToNodeVars(X,Mode);
     this->Run(X,fout);
     TW=this->GetTwin();
     if(Mode)Consistency=IsIn(this->GetTwin().GetOut(),Zero());
     else Consistency=!IsIn(this->GetTwin().GetInn(),Zero());
     if(!Consistency) {GConsistency=iperhaps;}
   }

 X[0].SetIval(Ival0);
 X[1].SetIval(Ival1);

 return GConsistency;
}//-----------------------------------------------------------------------------



















/*
void fstar::ComputeMonotonies(box& X)
{
 this->Model.AssignXToNodeVars(X,true);
 CreateCellFromTree(this->Model.GetCell(),C->GetModel().GetTree());
 AssignCellToTree(this->Model.GetTree(),this->Model.GetCell(),false);
 this->Model.GetCell().ComputeMonos(this->Model.GetTree(),"Full");
}//-----------------------------------------------------------------------------
*/

/*
void fstar::RunProj(box& X,ivalDb &Ival,fstream& fout)
{
 int Step=0;                                    // Step Counter
 double CTime;                                  // Computation Time
 clock_t CStart = clock();                      // Start internal clock.
 clock_t CStop;                                 // Final Time

 this->Initialize(X,fout);

 int Print=Pars.GetDebug();          // Get Print Flag
 this->DetermineModality();

 while (!this->List.empty())      // While List is not empty do.
 {
  PrintMsg("######################################################",fout,Print);

  if(Print) {fout << "Step Number:"; PrintNum(Step,fout,Print);}
  
  this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

  if(this->SelectedStrip==NULL)                // If No Strip Matching => Break
  {PrintMsg("No Strip Matching",fout,Print);
   this->Pars.SetFinish("Matching");
   break;}

   this->Twin;

   if(Step==3)
   {this->UpdateTwinAndSelectStrip(fout);}         // Update Twin and Select Strip.

  if(this->IsToleralceReached())
   {PrintMsg("Distance < Tol",fout,Print);
    this->Pars.SetFinish("Tolerance");
    break;}

  CStop = clock();                               // Stop Internal Clock.
  CTime=(CStop-CStart)/CLK_TCK;
  if (CTime > this->Pars.GetTime())              // Break: Time Limit Exceded.
   {PrintMsg("TimeLimit Exceded",fout,Print);
    this->Pars.SetFinish("Time");
    break;}

   if(this->Pars.GetInclusion()!=0 && IsInclusionStop())
   {PrintMsg("Stop By Inclusion",fout,Print);
    break;}

   ivalDb Inn=this->Twin.GetInn();
   if(IsEmpty(this->Twin.GetOut(),Ival) || IsIntersect(Inn,Ival))
   {PrintMsg("Proj Stop",fout,Print);
    break;}

   else
    {this->Bisect(fout);}                       // Bisect

  Step++;                                       // Increase step counter

  if(this->Pars.GetScreenOut())
   {this->PrintScreen(Step);}

 }  // End While

 this->Update=true;
 this->UpdateTwinAndSelectStrip(fout);         // Update Twin and Select Strip.

 if(this->List.empty())
  {PrintMsg("Empty Agenda",fout,Print);
   this->Pars.SetFinish("Empty Agenda");}      // Set Stop Criterion

 if(Pars.GetCells())  
 {this->AddListBufferToMinMaxLists();}           // Add List and Buffer to the minmax lists.

 if(this->Pars.GetSolution())
  {this->PrintSolution(CTime);}                  // Print Solution to a file.
}//-----------------------------------------------------------------------------
*/


/** \fn  void fstar::Bisect(fstream &fout)
    \brief Bisect the Strip for its proper dimention or the first Cell for
    its improper dimention.*/
    /*
void fstar::Bisect(fstream &fout)
{
 double XpWidth,XiWidth;
 int Print=this->Pars.GetDebug();
 bool ComputeCell=true;

 list<strip>::iterator Strip=this->SelectedStrip;  // Pointer to the selected strip.
 list<cell>::iterator Cell=Strip->GetSelectCell();

 if(Strip!=NULL)
  {PrintMsg("Selected Strip",fout,Print);
   Strip->Print(fout,Print,true);}
 if(Cell!=NULL)
   {PrintMsg("Selected Cell",fout,Print);
    Cell->Print(fout,Print);}

 this->AssignBisectDim(XpWidth,XiWidth);    // Assign Bisection Dimention.

 if(XpWidth > this->Pars.GetEps() || XiWidth > this->Pars.GetEps())
  {
   if(XiWidth > this->Pars.GetEps())
   {PrintMsg("Action: Bisect Cell Xi",fout,Print);
    if(XpWidth > this->Pars.GetEps()) {ComputeCell=false;}
    else {ComputeCell=true;}
    this->BisectCellXi(Cell,ComputeCell,fout);         // Bisect Improper Dimention (Cell)
    Strip->GetStock().erase(Strip->GetPStock());
    this->Update=true;}

   if(XpWidth > this->Pars.GetEps())
    {PrintMsg("Action: Bisect Strip Xp",fout,Print);
     this->BisectStripXp(Strip,fout);      // Bisect Strip Proper Dimention
     this->Stock.erase(this->PStock);
     this->Update=true;}

   else
     {Strip->SetUpdate(true);
      Strip->ComputeUpdateTwin(false,fout,this->Model,this->Pars);}
  }

 else
 {
   if(Cell==NULL) // No Cell Matching
   {
     PrintMsg("Accumulate Strip: No Cell Matching",fout,Print);
     this->AccumTwin(Strip);              // Accumulate Twin and Store it to the Buffer.
     this->Update=false;
     this->Stock.erase(this->PStock);
   }
   else     // Width < Epsilon
   {
     PrintMsg("Accumulate for Epsilon (Width < Epsilon)",fout,Print);
     if(this->Pars.GetPlot()) PlotCell(*Cell,clYellow,clBlack,bsDiagCross,Pars);
     Strip->GetWareHouse().splice(Strip->GetWareHouse().end(),Strip->GetCells(),Cell); // Store to WareHouse
     Strip->SetUpdate(false);
     Strip->GetStock().erase(Strip->GetPStock());
     Strip->AssignCell(fout,this->Pars); // Assign Cell to bisect.
     this->Update=false;
   }
 }
}//-----------------------------------------------------------------------------
*/

      /*
      if(this->Pars.GetOptim()!=2 && (Strip->GetTwin().GetInf() <<= this->Twin.GetInf()))
       {this->Stock.push_back(Strip);}
      else if(this->Pars.GetOptim()!=1 && (Strip->GetTwin().GetSup() <<= this->Twin.GetSup()))
      {this->Stock.push_back(Strip);}
      */

/*
    // Contract ------------------------------------------------------------
    ivalDb inter=Strip->operator [](0).GetIval();
    if(this->Pars.GetContract() && IsEmpty(Prop(this->Twin.GetOut()),Prop(Strip->operator [](0).GetIval())))
      {Trash.push_back(Strip);
       PrintMsg("Eliminate Strip: No Intersects X",fout,Print);}

    else if(this->Pars.GetContract() && !IsIn(Prop(Strip->operator [](0).GetIval()),Prop(this->Twin.GetOut())))
     {Strip->Contract(this->Twin.GetOut(),0);
      Strip->ComputeUpdateTwin(true,fout,this->Model,this->Pars);
      if(Strip->GetJoinFlag()==true)
      {this->Twin.SetInn(this->Twin.GetInn() || Strip->GetTwin().GetInn());}  // Join Inner Approxs.
       this->Twin.SetOut(this->Twin.GetOut() || Strip->GetTwin().GetOut());    // Join Outer Approxs.
      }
    // Contract ------------------------------------------------------------
    */




