 /*!****************************************************************************
 *    \file incidence.cpp
 *    This file contains the mplementation of the incidence class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "incidence.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
  \fn incidence(void)*/
incidence::incidence(void)
{MonoDom=Infinity();
 Mono=0;
 IvalVar=NULL;
 MonoVar=NULL;
 TotalMonoVar=NULL;
}//-----------------------------------------------------------------------------

/** Copy Constructor.
    \fn incidence(const incidence &Inc)*/
incidence::incidence(const incidence &Inc)
{MonoDom=Inc.MonoDom;
 Mono=Inc.Mono;
 IvalVar=Inc.IvalVar;
 MonoVar=Inc.MonoVar;
 TotalMonoVar=Inc.TotalMonoVar;
}//-----------------------------------------------------------------------------

/*!*****************************************************************************
* \defgroup Members Members' class acces functions
*******************************************************************************/

/** \fn int GetMono()const
    \return Monotony.
    (0 => Non monotonic; 1 => Monotonic increasing; -1 => Monotonic decreasing)*/
int incidence::GetMono()
{return Mono;
}//-----------------------------------------------------------------------------

/** \fn ivalDb GetMonoDom()const
    \return Monotony domain.*/
ivalDb incidence::GetMonoDom()
{return MonoDom;
}//-----------------------------------------------------------------------------

/** \fn ivalDb GetDom()const
    \return Interval domain.*/
ivalDb& incidence::GetIvalVar()
{return *this->IvalVar;
}//-----------------------------------------------------------------------------

int incidence::GetMonoVar()
{return *this->MonoVar;
}//-----------------------------------------------------------------------------

bool incidence::GetTotalMonoVar()
{return *this->TotalMonoVar;
}//-----------------------------------------------------------------------------

bool incidence::GetUniIncVar()
{return *this->UniIncVar;
}//-----------------------------------------------------------------------------

/** \fn void SetMono(int Mono)
    Set monotony.
    (Mono=0 => Non monotonic; Mono=1 => Monotonic increasing; Mono=-1 => Monotonic decreasing)*/
void incidence::SetMono(int MN)
{Mono=MN;
}//-----------------------------------------------------------------------------

/** \fn void SetMonoDom(ivalDb& MD)
    Set monotony domain.*/
void incidence::SetMonoDom(ivalDb &MD)
{MonoDom=MD;
}//-----------------------------------------------------------------------------

/** \fn  void SetDom(ivalDb Dom)
    Set interval domain*/
void incidence::SetIvalVar(ivalDb* IV)
{IvalVar=IV;
}//-----------------------------------------------------------------------------

void incidence::SetMonoVar(int* MV)
{MonoVar=MV;
}//-----------------------------------------------------------------------------

void incidence::SetTotalMonoVar(bool* TMV)
{TotalMonoVar=TMV;
}//-----------------------------------------------------------------------------

void incidence::SetUniIncVar(bool* UIV)
{UniIncVar=UIV;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn bool IsProp(void)
    Return true if the interval domain is proper and false if it is improper*/
bool incidence::IsProp(void)
{return IvalVar->IsProp();
}//-----------------------------------------------------------------------------

/** \fn bool IsImpr(void)
    Return true if the interval domain is improper and false if it is proper*/
bool incidence::IsImpr(void)
{return IvalVar->IsImpr();
}//-----------------------------------------------------------------------------

/** \fn void AssignMono(void)
    Assign Mono (Non monotonic => Mono=0; Monotonic increasing => Mononotony=1;
    Monotonic decreasing => Mono=-1)*/
void incidence::AssignMono(void)
{if(IsNaN(Inf(MonoDom)) || IsNaN(Sup(MonoDom))) Mono=0;
 else if(Inf(MonoDom)>=0) Mono=1;
 else if(Sup(MonoDom)<=0) Mono=-1;
 else Mono=0;
}//-----------------------------------------------------------------------------

/** \fn bool IsMono(void)
    \return true if it is monotonic and false if it is not.*/
bool incidence::IsMono(void)
{if(Mono==1 || Mono==-1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsMonoInc(void)
    \return true if it is monotonic increasing and false if it is not.*/
bool incidence::IsMonoIncr(void)
{if(Mono==1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsMonoDec(void)
    \return true if it is monotonic decreasing and false if it is not.*/
bool incidence::IsMonoDecr(void)
{if(Mono==-1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn void MakeDu(void)
    Makes the dual to the incidence domain*/
/*
void incidence::MakeDu(void)
{*IvalVar=Du(*IvalVar);
}//-----------------------------------------------------------------------------
*/

/** \fn void MakeProp(void)
    Makes proper the incidence domain*/
/*
void incidence::MakeProp(void)
{*IvalVar=Prop(*IvalVar);
}//-----------------------------------------------------------------------------
*/

/** \fn void MakeImpr(void)
    Makes improper the incidence domain*/
/*
void incidence::MakeImpr(void)
{*IvalVar=Impr(*IvalVar);
}//-----------------------------------------------------------------------------
*/

////////////////////////////////////////////////////////////////////////////////
// Friend Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
// Assignation Operators
////////////////////////////////////////////////////////////////////////////////

/** \fn incidence& operator=(const incidence &x)
    Overloaded assignation operator*/
incidence& incidence::operator=(const incidence &Inc)
{MonoDom=Inc.MonoDom;
 Mono=Inc.Mono;
 IvalVar=Inc.IvalVar;
 MonoVar=Inc.MonoVar;
 TotalMonoVar=Inc.TotalMonoVar;
 return *this;
}//-----------------------------------------------------------------------------

