 /*!****************************************************************************
 *    \file incidence.h
 *    This file contains the definition of the incidence class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "ivalDb.h"

#ifndef INCIDENCE_HH
#define INCIDENCE_HH

/*!*****************************************************************************
*    \class incidence
*    \brief Class for the Star Algorithm.
*    This class represents an incidences of an interval variable inside an
*    interval function.
*******************************************************************************/
class incidence
{
private:

        ivalDb MonoDom;      ///< enum Monotony Domain.
        int Mono;            ///< enum Monotony (0 = Non Monotonic, 1 = Monotonic Increasing,-1 = Monotonic Decreasing)
        ivalDb* IvalVar;      ///< enum Pointer to the variable Interval Value.
        int* MonoVar;        ///< enum Pointer to the Monotony of the variable
        bool* TotalMonoVar;     ///< enum Pointer to the Total Monotony flag of the variable
        bool* UniIncVar;       ///< enum Uni Incidence Pointer to variable
public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

incidence();
incidence(const incidence&);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~incidence() {}

////////////////////////////////////////////////////////////////////////////////
// Acces Member Functions                                                     //
////////////////////////////////////////////////////////////////////////////////

ivalDb GetMonoDom();
int GetMono();
ivalDb& GetIvalVar();
int GetMonoVar();
bool GetTotalMonoVar();
bool GetUniIncVar();

void SetMonoDom(ivalDb&);
void SetMono(int);
void SetIvalVar(ivalDb*);
void SetMonoVar(int*);
void SetTotalMonoVar(bool*);
void SetUniIncVar(bool*);

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

bool IsProp(void);
bool IsImpr(void);
bool IsMono(void);
bool IsMonoIncr(void);
bool IsMonoDecr(void);

void AssignMono(void);

/*
void MakeProp(void);
void MakeImpr(void);
*/

////////////////////////////////////////////////////////////////////////////////
// Equality Operator
////////////////////////////////////////////////////////////////////////////////

incidence& operator=(const incidence&);

////////////////////////////////////////////////////////////////////////////////
// Friend Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

friend incidence Du(incidence &x);

}; // end Class incidence

#endif
