 /*!****************************************************************************
 *    \file model.cpp
 *    This file contains the mplementation of the model class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "model.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** \fn model::model().
    \brief Default Constructor.*/
model::model()
{Name="Function";
 Function=NULL;
 Satisfied=iperhaps;
 Tree=NULL;
}//-----------------------------------------------------------------------------

/** \fn model::model(const model &Model).
    \brief Copy Constructor.*/
model::model(const model &Model)
{Name=Model.Name;
 X.resize(Model.X.size());
 X=Model.X;
 Satisfied=Model.Satisfied;
 Function=Model.Function;
 Cell=Model.Cell;
 Tree=Model.Tree;
 FreeX=Model.FreeX;
 FreeNodeVars=Model.FreeNodeVars;
 ConstrsX.resize(Model.ConstrsX.size());
 ConstrsX=Model.ConstrsX;
}//-----------------------------------------------------------------------------

/** \fn model& model::operator=(const model &Model).
    \brief Assignation (=) overloaded operator.*/
model& model::operator=(const model &Model)
{Name=Model.Name;
 Satisfied=Model.Satisfied;
 X.resize(Model.X.size());
 X=Model.X;
 Function=Model.Function;
 Cell=Model.Cell;
 Tree=Model.Tree;
 FreeX=Model.FreeX;
 FreeNodeVars=Model.FreeNodeVars;
 ConstrsX.resize(Model.ConstrsX.size());
 ConstrsX=Model.ConstrsX;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members class                                                     //
////////////////////////////////////////////////////////////////////////////////

/** \fn char* model::GetName(void).
    \brief Get function name.*/
char* model::GetName(void)
{return Name;
}//-----------------------------------------------------------------------------

/** \fn iboolean model::GetSatisf(void).
    \brief Get Satisfiable flag.*/
iboolean model::GetSatisf(void)
{return Satisfied;
}//-----------------------------------------------------------------------------

/** \fn vector<variable>& model::GetVars(void).
    \brief Get vector of variables.*/
vector<variable>& model::GetX(void)
{return X;
}//-----------------------------------------------------------------------------

/** \fn int model::GetTOptimal(void).
    \brief Get tree optimality flag.*/
int model::GetTOptimal(void)
{return TreeOptimal;
}//-----------------------------------------------------------------------------

/** \fn int model::GetCell(void).
    \brief Get Cell.*/
cell& model::GetCell(void)
{return Cell;
}//-----------------------------------------------------------------------------

/** \fn tree& model::GetTree(void).
    \brief Get Tree.*/
tree& model::GetTree(void)
{return Tree;
}//-----------------------------------------------------------------------------

/** \fn list<tree>& model::GetConstraints(void)
    \brief Get Constraints.*/
list<model>& model::GetConstraints(void)
{return Constraints;
}//-----------------------------------------------------------------------------

vector<int>& model::GetConstrsX(void)
{return ConstrsX;
}//-----------------------------------------------------------------------------

string&  model::GetModality(void)
{return Modality;
}//-----------------------------------------------------------------------------


/** \fn void model::SetName(char* &NM).
    \brief Set function name.*/
void model::SetName(char* &NM)
{Name=NM;
}//-----------------------------------------------------------------------------

/** \fn void model::SetSatisf(iboolean Sat).
    \brief Set Satisfiable flag.*/
void model::SetSatisf(iboolean Sat)
{Satisfied=Sat;
}//-----------------------------------------------------------------------------

/** \fn void model::SetTOptimal(int TO).
    \brief Set Tree optimality flag.*/
void model::SetTOptimal(int TO)
{TreeOptimal=TO;
}//-----------------------------------------------------------------------------

/** \fn void model::SetCell(cell &C).
    \brief Set Cell.*/
void model::SetCell(cell &C)
{Cell=C;
}//-----------------------------------------------------------------------------

/** \fn void model::SetTree(tree &T).
    \brief Set Cell.*/
void model::SetTree(tree &T)
{Tree=T;
}//-----------------------------------------------------------------------------

/** \fn void model::SetConstraints(list<tree>& CNSTS)
    \brief Set Constraints.*/
void model::SetConstraints(list<model>& CNSTS)
{Constraints=CNSTS;
}//-----------------------------------------------------------------------------

void model::SetConstrsX(vector<int>& INDXS)
{ConstrsX=INDXS;
}//-----------------------------------------------------------------------------

void  model::SetModality(string& M)
{Modality=M;
}//-----------------------------------------------------------------------------

/** \fn void model::SetInit(void (*Initialize) (box &X,model &Model)).
    \brief Set Pointer to Initialization function.*/
void model::SetFunction(void (*Funct) (box&,model&,param&))
{Function=Funct;
}//-----------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
// Binary Inner Operators                                                     //
////////////////////////////////////////////////////////////////////////////////

// AssignIndexesToConstraints
void model::AssignXToConstraints(void)
{string NameVar1, NameVar2;
 list<model>::iterator Constraint;
 for(Constraint=this->GetConstraints().begin();Constraint!=this->GetConstraints().end();Constraint++)
 {
  Constraint->GetConstrsX().resize(Constraint->GetTree().GetNodeVars().size());

  for(int i=0;i<Constraint->GetCell().GetDim();i++)
  {NameVar1=Constraint->GetCell()[i].GetName();
   for(int j=0;j<this->GetCell().GetDim();j++)
    {
     NameVar2=this->GetCell()[j].GetName();
     if(NameVar1 == NameVar2)
     {Constraint->GetConstrsX()[i]=j;
      break;}
   }
  }
 }
}//-----------------------------------------------------------------------------

iboolean model::ConstraintsConsistency(cell& Cell)
{
 int Index;
 ivalDb Domain,LDomain;
 iboolean Consistency=itrue;
 string Operator;
 
 list<model>::iterator Constraint;
 for(Constraint=this->GetConstraints().begin();Constraint!=this->GetConstraints().end();Constraint++)
 {
  for(int i=0; i<Constraint->GetCell().GetDim();i++)    // Assign Domains to variables
  {
   Index=Constraint->GetConstrsX()[i];
   Constraint->GetCell()[i].SetIval(Cell[Index].GetIval());
  }

  AssignCellToTree(Constraint->GetTree(),Constraint->GetCell(),false);

  Constraint->GetTree().EvaluateFunct("Prop",false,false,itrue,'?');     // f* can be used !!!!!!!!!!!!!!!!!!

  Domain=Constraint->GetTree().GetResult();
  Operator=Constraint->GetTree().GetRoot()->GetType();
  LDomain=Constraint->GetTree().GetRoot()->GetLNode()->GetIval();

  if(IsIn(Domain,Zero()))
   {Consistency=itrue;}
  else
  {
   if(Operator=="=")
   {
     if(!IsIn(Zero(),LDomain))
     {Consistency=ifalse;
      break;}
     else
    {Consistency=iperhaps;
     break;}
   }

   else
   {
     if(Operator==">")
      {Domain=LDomain - Ival(0,MInfinity());}
     else if(Operator=="<")
      {Domain=LDomain - Ival(PInfinity(),0);}

     if(IsIn(Domain,Zero()))
      {Consistency=ifalse;
       break;}

     else
      {Consistency=iperhaps;
       break;}
   }
  }
 }
 return Consistency;
}//-----------------------------------------------------------------------------

// Assign Indexes of Free Variables
void model::AssignNodeVarsXIndexes(box& X)
{
  string NameVar1,NameVar2;
  for(int i=0;i<X.GetVars().size();i++)
  {NameVar1=X.GetVars()[i].GetName();
   for(int j=0;j<this->GetTree().GetNodeVars().size();j++)
    {NameVar2=this->GetTree().GetNodeVars()[j].GetNameVar();
     if(NameVar1 == NameVar2)
     {this->FreeNodeVars.push_back(j);
      this->FreeX.push_back(i);
      break;}
    }
  }
}//-----------------------------------------------------------------------------

// Assign Free Variables Interval Values
void model::AssignXToNodeVars(box& X,bool Mode)
{ivalDb Ival;
 int index1,index2;
 for(int i=0;i<this->FreeX.size();i++)
  {index1=this->FreeX[i];
   Ival=X[index1].GetIval();
   index2=this->FreeNodeVars[i];
   if(Mode) {this->GetTree().GetNodeVars()[index2].SetIval(Ival);}
   else {this->GetTree().GetNodeVars()[index2].SetIval(Du(Ival));}
  }
}//-----------------------------------------------------------------------------

void model::AssignXToNodeVars(box& X,int Dim)
{ivalDb Ival;
 int index1,index2;
 for(int i=0;i<this->FreeX.size();i++)
  {index1=this->FreeX[i];
   Ival=X[index1].GetIval();
   if(Dim==index1) {Ival.MakeProp();}
   else {Ival.MakeImpr();}
   index2=this->FreeNodeVars[i];
   this->GetTree().GetNodeVars()[index2].SetIval(Ival);
  }
}//-----------------------------------------------------------------------------

// Assign Indexes of Free Variables
void model::AssignXCellIndexes(box& X)
{
  string NameVar1,NameVar2;
  for(int i=0;i<X.GetVars().size();i++)
  {NameVar1=X.GetVars()[i].GetName();
   for(int j=0;j<this->GetCell().GetDim();j++)
    {NameVar2=this->GetCell()[j].GetName();
     if(NameVar1 == NameVar2)
     {this->CellIndexes.push_back(j);
      this->XIndexes.push_back(i);
      break;}
    }
  }
}//-----------------------------------------------------------------------------

// Assign Free Vars from Cell to X box
void model::AssignCellToX(box& X)
{ivalDb Ival;
 int index1,index2;
 for(int i=0;i<this->CellIndexes.size();i++)
  {index1=this->CellIndexes[i];
   index2=this->XIndexes[i];
   Ival=this->GetCell()[index1].GetIval();
   X[index2].SetIval(Ival);
   if(Ival.IsImpr()) {X.SetConsistent(ifalse);}
  }
}//-----------------------------------------------------------------------------

// Assign X to Cell
void model::AssignXToCell(box& X)
{ivalDb Ival;
 int index1,index2;
 for(int i=0;i<this->CellIndexes.size();i++)
  {
   index1=this->CellIndexes[i];
   index2=this->XIndexes[i];
   Ival=X[index2].GetIval();
   this->GetCell()[index1].SetIval(Ival);
  }
}//-----------------------------------------------------------------------------

