 /*!****************************************************************************
 *    \file mode.h
 *    This file contains the definition of the model class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "cell.h"
#include "iboolean.h"
#include "box.h"
#include "param.h"
#include "tree.h"

#ifndef MODEL_HH
#define MODEL_HH


/*!*****************************************************************************
*    \class model
*    \brief Model Class for the Star Algorithm.
*    This class represents the funtion to be analysed.
*******************************************************************************/
class model
{

private:

char *Name;                                       ///> Name of the function.
vector<variable> X;                               ///> Vector of variables.
iboolean Satisfied;                               ///> Satisfiable flag.
int TreeOptimal;                                  ///> Tree optimality flag.
cell Cell;                                        ///> Cell
tree Tree;                                        ///> Tree
vector<int> ConstrsX;                             //> Constraints variables variables (Indexes)
list<model> Constraints;                          ///> Constraints
string Modality;
deque<int> FreeX;                                 //> Indexes of Free Variables (QSI True Case)
deque<int> FreeNodeVars;
deque<int> CellIndexes;                           ///> Indexes of Free variables on the Cell.
deque<int> XIndexes;

public:

void (*Function)(box&,model&,param&);       ///> Pointer to Problem.

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

model();
model(const model &Model);

 ////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~model() {}

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

void SetFunction(void (*Initialize) (box&,model&,param&));

char* GetName(void);
iboolean GetSatisf(void);
vector<variable>& GetX(void);
int GetTOptimal(void);
cell& GetCell(void);
tree& GetTree(void);
list<model>& GetConstraints(void);
vector<int>& GetConstrsX(void);
string& GetModality(void);
cell& GetDXCell(void);

void SetName(char* &Name);
void SetSatisf(iboolean Sat);
void SetTOptimal(int TO);
void SetCell(cell &Cell);
void SetTree(tree &Tree);
void SetConstraints(list<model>&);
void SetConstrsX(vector<int>&);
void SetModality(string&);
void SetDXCell(cell &Cell);

model& operator=(const model &Model);

void AssignXToConstraints(void);
iboolean ConstraintsConsistency(cell& Cell);
void AssignNodeVarsXIndexes(box& X);
void AssignXToNodeVars(box& X,bool Mode);
void AssignXToNodeVars(box& X,int Dim);
void AssignCellToX(box& X);
void AssignXCellIndexes(box& X);
void AssignXToCell(box& X);

}; // end Class model

#endif

