/*******************************************************************************
 *    node.h: Syntactic Tree Node
 *    29/12/2002
 *    By Pau Herrero Vias
 ******************************************************************************/

#include <fstream.h>
#include <string.h>
#include <deque.h>
#include <vector.h>
#include <list.h>
#include <stack.h>
#include "incidence.h"
#include "iboolean.h"

#ifndef NODE_HH
#define NODE_HH

typedef deque<incidence*> var;
typedef deque<var> vars;


class node
{
 private:
        string Type;
        ivalDb Ival;
        int Const;
        vars* PVars;
        int VarIndex;
        incidence** PInc;
        string NameVar;
        deque<node>* PNodeVars;
        list<node*> NodesList;
        string Quantified;
        list<node*> Nodes;
        //list<node>* PNodesStock;
        node* LNode;
        node* RNode;

 public:

        int MeasureIndex;                       // SQT attribute

    //////////////////
    // Constructors //
    //////////////////

    node(void);
    node(int Const);
    node(double D);
    node(ivalDb& Ival);
    node(string& NameVar);
    node(const ivalDb& Ival, string NameVar);
    node(const ivalDb& Ival, string NameVar,string Quantified);
    node(incidence** I);
    node(const node &N);

    ////////////////
    // Destructor //
    ////////////////

    ~node(){}

    //////////////////////
    // Acces to members //
    //////////////////////

    string& GetType();
    ivalDb& GetIval();
    node* GetLNode();
    node* GetRNode();
    vars* GetPVars();
    int GetVarIndex();
    incidence** GetPInc();
    string& GetNameVar();
    deque<node>* GetPNodeVars();
    double GetConst();
    list<node*>& GetNodesList();
    string& GetQuantified();
    list<node*>& GetNodes();
    //list<node>* GetPNodesStock();

    void SetType(string&);
    void SetIval(ivalDb&);
    void SetLNode(node*);
    void SetRNode(node*);
    void SetPVars(vars*);
    void SetPInc(incidence**);
    void SetNameVar(string&);
    void SetPNodeVars(deque<node>* PNV);
    void SetConst(double);
    void SetNodesList(list<node*>&);
    void SetVarIndex(int);
    void SetQuantified(string& Q);
    void SetNodes(list<node*>&);
    //void SetPNodesStock(list<node>*);

    node* CreateNode(void);
    node* CreateVar(void);
    node* CreateConst(void);

    iboolean SelectNonUnifMonoNodes(list<node*>& NonUnifNodes);
    void Evaluate(string &Mode,bool StripTOptim,bool CellTOptim,iboolean &Consist,char Point);
    void Print(fstream &fout,bool Symbolic);
    node* Diff(incidence* Inc,int VarIndex,string &Scope);
    node* Copy(void);
    int GetIndex(incidence* Inc);
    bool IsProp(void);
    bool IsImpr(void);
    void AddToList(list<node*> &FunctList);
    void EvaluateNodesList(void);
    void CreatreNewIncidences(void);
    void ToLatex(fstream &fout);
    void EvaluateFPropagation(string &Mode);
    void EvaluateBPropagation(void);
    void PrintPrimitive(fstream &fout,int &Counter);
    void AssignNameAuxVar(void);
    node* NewNode(void);

    ////////////////////////////////////
    // SQT Functions                  //
    ////////////////////////////////////

    node* Replicate(int Step, deque< deque<node*> >& Measurements);
    void FindStateNodes(deque<node*>& States);
    void AssignName(int Step);

    ////////////////////////////////////
    // Monary Operators and Functions //
    ////////////////////////////////////

    node& operator=(const node& N);
    node& operator-();

    ////////////////////////////
    // Binary Operators //
    ////////////////////////////

    friend void Simplify(node *&N);
    friend void DeleteNode(node *&N);
    ////////////////////////////
    // Binary Operators //
    ////////////////////////////

    friend node& operator==(const node& LN,const node& RN);
    friend node& operator>>=(const node& LN,const node& RN);
    friend node& operator<<=(const node& LN,const node& RN);
    friend node& operator>>(const node& LN,const node& RN);
    friend node& operator<<(const node& LN,const node& RN);
    friend node& operator+(const node& LN,const node& RN);
    friend node& operator-(const node& LN,const  node& RN);
    friend node& operator*(const node& LN,const node& RN);
    friend node& operator/(const node &LN,const node &RN);
    friend node& operator^(node& LN, int n);

    ////////////////////////////
    // Monary Operators //
    ////////////////////////////

    friend node& Du(node& N);
    friend node& abs(node& N);
    friend node& pow(node& N,int n);
    friend node& sqr(node& N);
    friend node& sqrt(node& N);
    friend node& cbrt(node& N);
    friend node& root(node& N,int n);
    friend node& cos(node& N);
    friend node& sin(node& N);
    friend node& tan(node& N);
    friend node& acos(node& N);
    friend node& asin(node& N);
    friend node& atan(node& N);
    friend node& cosh(node& N);
    friend node& sinh(node& N);
    friend node& tanh(node& N);
    friend node& acosh(node& N);
    friend node& asinh(node& N);
    friend node& atanh(node& N);
    friend node& exp(node& N);
    friend node& log(node& N);
    friend node& log10(node& N);
    friend node& sign(node& N);
    friend node& max(const node& LN,const node& RN);
    friend node& min(const node& LN,const node& RN);
    friend node& minmax(const node& LN,const node& RN);
    friend node& max(list<node>& NODES);
    friend node& min(list<node>& NODES);

    //////////////////////////
    // I/O Stream Operators //
    //////////////////////////

    friend ostream& operator<<(ostream& out, node& N);

    friend bool operator<(const node &N1, const node &N2);

    
};



#endif
