 /*!****************************************************************************
 *    \file param.h
 *    This file contains the definition of the param class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include <fstream.h>
#include <vector.h>
#include "iboolean.h"

#ifndef PARAM_HH
#define PARAM_HH

/*!*****************************************************************************
*    \class fstar
*    \brief Class containig a set of parameters used for the Star Algorithm.
*******************************************************************************/
class param
{
private:
        double Eps;               ///> Limit size of the bisection procedure.
        double Tol;               ///> Precission between inner and outer approximation.
        int    Debug;             ///> Debug Output stream flag.
        int    Bisect;            ///> Number of bisectons at each iteration.
        double Time;              ///> Time limit for the algorithm.
        char*  Finish;            ///> Finish criterion.
        bool   ScreenOut;         ///> Screen Output flag
        int    Optim;             ///> Optimization flag (0=> None; 1=> MinMax,2==MaxMin; 3==f*).
        bool   Solution;          ///> Print Solution Flag.
        bool   Plot;              ///> Print Solution Flag.
        bool   Contract;          ///> Contraction flag.
        int    Inclusion;         ///> Inclusion stop conditions (0=> None; 1=> true,2==false).
        string PlotX;
        string PlotY;
        int    XPlotIndex;
        int    YPlotIndex;
        iboolean Consistency;     ///> Consistency Flag
        bool   Cells;             ///> Cells Flag
        string File;              ///> File name
        vector<double> XWidths;   ///> Variables Widths vector

public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

param();
param(param &P);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~param() {}

////////////////////////////////////////////////////////////////////////////////
// Copy Operator                                                                //
////////////////////////////////////////////////////////////////////////////////

param& operator=(const param &P);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

double GetEps() const;
double GetTol() const;
int GetDebug() const;
int GetBisect() const;
double GetTime() const;
int GetOptim() const;
char* GetFinish() const;
bool GetScreenOut() const;
bool GetSolution() const;
bool GetPlot() const;
int GetStrategy() const;
bool GetContract() const;
int GetInclusion() const;
string& GetPlotX();
string&  GetPlotY();
int GetXPlotIndex();
int  GetYPlotIndex();
iboolean&  GetConsistency();
bool GetCells() const;
string& GetFile();
//vector<double>& GetEpsilons();
vector<double>& GetXWidths();

void SetEps(double);
void SetTol(double);
void SetDebug(int);
void SetBisect(int);
void SetTime(double);
void SetOptim(int);
void SetFinish(char*);
void SetScreenOut(bool);
void SetSolution(bool);
void SetPlot(bool);
void SetContract(bool);
void SetInclusion(int);
void SetPlotX(string&);
void SetPlotY(string&);
void SetXPlotIndex(int);
void SetYPlotIndex(int);
void SetConsistency(iboolean&);
void SetCells(bool);
void SetFile(string&);
//void SetEpsilons(vector<double>&);
void SetXWidths(vector<double>&);

}; // End parameter class

#endif

