 /*!****************************************************************************
 *    \file strip.cpp
 *    This file contains the mplementation of the strip class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "plot.h"
#include "strip.h"
#include "Msgs.h"
#include "fstar.h"
//#include "functions.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
   \fn strip::strip(void)*/
strip::strip(void)
{Twin=twin();
 Xp.resize(0);
 XpDim=-1;
 Update=true;
 SelectedCell=NULL;
 Consistent=itrue;
 JoinFlag=false;
 TOptimal=false;
 //Penalty=0;
 BisectXp=true;
 ZeroIncluded=false;
 ZeroExcluded=false;
 }//-----------------------------------------------------------------------------

 /** Constructor 1.
    \fn strip::strip(vector<variable> &PV,vector<variable> &IV)
    \param PV Proper varibles vector.
    \param IV Imroper varibles vector. */
strip::strip(vector<variable> &PV,vector<variable> &IV)
{Twin=twin();
 Xp.resize(PV.size());
 Xp=PV;
 Xi.resize(IV.size());
 Xi=IV;
 Update=true;
 SelectedCell=NULL;
 Consistent=itrue;
 JoinFlag=false;
 TOptimal=false;
 //Penalty=0;
 BisectXp=true;
 ZeroIncluded=false;
 ZeroExcluded=false;
}//-----------------------------------------------------------------------------

/** Constructor 2.
    \fn strip::strip(const cell &Cell)
    \param Cell. */
strip::strip(const cell &Cell)
{Twin=Cell.GetTwin();
 Xp.resize(Cell.GetXp().size());
 Xp=Cell.GetXp();
 Xi.resize(Cell.GetXi().size());
 Xi=Cell.GetXi();
 Update=true;
 Consistent=itrue;
 Cells.push_back(Cell);
 SelectedCell=(--Cells.end());
 Cells.back().SetXpStrip(Xp);
 JoinFlag=false;
 TOptimal=false;
 //Penalty=0;
 BisectXp=true;
 this->AssignXpBigDim();
 ZeroIncluded=false;
 ZeroExcluded=false;
}//-----------------------------------------------------------------------------

/** Copy Constructor.
    \fn strip::strip(const strip &Strip)*/
strip::strip(const strip &Strip)
{Twin=Strip.Twin;
 Xp.resize(Strip.Xp.size());
 Xp=Strip.Xp;
 Xi.resize(Strip.Xi.size());
 Xi=Strip.Xi;
 SelectedCell=NULL;
 XpDim=Strip.XpDim;
 Cells=Strip.Cells;
 WareHouse=Strip.WareHouse;
 Update=Strip.Update;
 Consistent=Strip.Consistent;
 JoinFlag=Strip.JoinFlag;
 Excess=Strip.Excess;
 TOptimal=Strip.TOptimal;
 //Penalty=0;
 BisectXp=true;
 ZeroIncluded=false;
 ZeroExcluded=false;
 }//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator                                                        //
////////////////////////////////////////////////////////////////////////////////

/** \fn variable& strip::operator[] (int i)
    Overloaded Access Operator([])*/
variable& strip::operator[] (int i)
{int PropSize=Xp.size();
 int ImprSize=Xi.size();
 if(i< Xp.size())
  {
   return Xp[i];
  }
 else
 {int j=i-Xp.size();
  return Xi[j];}
}//-----------------------------------------------------------------------------

/** \fn strip& strip::operator=(const strip &Strip)
    Overloaded Assignation Operator(=)*/
strip& strip::operator=(const strip &Strip)
{Twin=Strip.Twin;
 Xp.resize(Strip.Xp.size());
 Xp=Strip.Xp;
 Xi.resize(Strip.Xi.size());
 Xi=Strip.Xi;
 SelectedCell=Strip.SelectedCell;
 XpDim=Strip.XpDim;
 Cells=Strip.Cells;
 WareHouse=Strip.WareHouse;
 Update=Strip.Update;
 Consistent=Strip.Consistent;
 JoinFlag=Strip.JoinFlag;
 Excess=Strip.Excess;
 //Penalty=0;
 BisectXp=true;
 ZeroIncluded=false;
 ZeroExcluded=false;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn twin& strip::GetTwin()
    \return Strip twin*/
twin& strip::GetTwin()
{return Twin;
}//-----------------------------------------------------------------------------

/** \fn vector<variable>& strip::GetXp()
    \return proper variables vector*/
vector<variable>& strip::GetXp()
{return Xp;
}//-----------------------------------------------------------------------------

/** \fn vector<variable>& strip::GetXi()
    \return improper variables vector*/
vector<variable>& strip::GetXi()
{return Xi;
}//-----------------------------------------------------------------------------

/** \fn bool& strip::GetUpdate()
    \return Update flag (true=>Update Strip twin; false=>No update Strip twin)*/
bool strip::GetUpdate()
{return Update;
}//-----------------------------------------------------------------------------

/** \fn list<cell>::iterator& strip::GetSelectCell()
    \return selected cell to bisect (it has to be previously selected)*/
list<cell>::iterator& strip::GetSelectCell()
{return SelectedCell;
}//-----------------------------------------------------------------------------

/** \fn list<cell>& strip::GetCells()
    \return Cells list (Candidate cells to be bisected). */
list<cell>& strip::GetCells()
{return Cells;
}//-----------------------------------------------------------------------------

/** \fn list<cell>& strip::GetWareHouse()
    \return Cells list not to be bisected*/
list<cell>& strip::GetWareHouse()
{return WareHouse;
}//-----------------------------------------------------------------------------

/** \fn int& strip::GetXpDim()
    \return selected proper dimention to bisect*/
int strip::GetXpDim()
{return XpDim;
}//-----------------------------------------------------------------------------

/** \fn int strip::GetDim()
    \return diemtion of the cartesian product of proper and improper variables*/
int strip::GetDim()
{return Xp.size() + Xi.size();
}//-----------------------------------------------------------------------------

/** \fn iboolean strip::GetConsist(void)
    \return consistency flag.*/
iboolean strip::GetConsist(void)
{return Consistent;
}//-----------------------------------------------------------------------------

/** \fn bool strip::GetJoinFlag()
    \brief  Flag which indicates if the Strip has to be joined with the global twin.
    \return Join flag (true=>Join Strip twin; false=>Not Join Strip twin)*/
bool strip::GetJoinFlag()
{return JoinFlag;
}//-----------------------------------------------------------------------------

/** \fn int cell::GetExcess(void)
    \return Excess approximation*/
ivalDb strip::GetExcess(void)
{return Excess;
}//-----------------------------------------------------------------------------

list< list<cell>::iterator>& strip::GetStock(void)
{return Stock;
}//-----------------------------------------------------------------------------

list< list<cell>::iterator>::iterator strip::GetPStock(void)
{return PStock;
}//-----------------------------------------------------------------------------

bool strip::GetTOptimal(void)
{return TOptimal;
}//-----------------------------------------------------------------------------

list< list<cell>::iterator >& strip::GetInsertedCells()
{return InsertedCells;
}//-----------------------------------------------------------------------------
/*
int strip::GetPenalty()
{return Penalty;
}//-----------------------------------------------------------------------------
*/

bool strip::GetBisectXp()
{return BisectXp;
}//-----------------------------------------------------------------------------

bool strip::GetZeroIncluded()
{return ZeroIncluded;
}//-----------------------------------------------------------------------------

bool strip::GetZeroExcluded()
{return ZeroExcluded;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetTwin(twin &Tw)
    \brief Set strip twin.
    \param Tw twin.*/
void strip::SetTwin(twin &Tw)
{Twin=Tw;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetXp(vector<variable> &Pvars)
    \brief Set the proper variables vector.
    \param Pvars proper variable vector.*/
void strip::SetXp(vector<variable> &Pvars)
{Xp.resize(Pvars.size());
 Xp=Pvars;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetXi(vector<variable> &Ivars)
    \brief Set improper variable vector.
    \param Ivars improper variable vector.*/
void strip::SetXi(vector<variable> &Ivars)
{Xi.resize(Ivars.size());
 Xi=Ivars;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetSelectCell(list<cell>::iterator &Cell)
    \brief Set selected cell.
    \param SCell selected cell.*/
void strip::SetSelectCell(list<cell>::iterator &SCell)
{SelectedCell=SCell;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetCells(list<cell> &CELLS)
    \brief Set list o cells.
    \param CELLS list of cells.*/
void strip::SetCells(list<cell> &CELLS)
{Cells.resize(CELLS.size());
 Cells=CELLS;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetWareHouse(list<cell> &WH)
    \brief Set the warehouse list of cells.
    \param WH list of cells.*/
void strip::SetWareHouse(list<cell> &WH)
{WareHouse.resize(WH.size());
 WareHouse=WH;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetXpDim(int Dim)
    \brief Set the selected proper dimention to bisect.
    \param Dim dimention to bisect.*/
void strip::SetXpDim(int Dim)
{XpDim=Dim;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetUpdate(bool UD)
    \brief Set update flag.
    \param UD (true=>Needs to be updated; flase=No needs;)*/
void strip::SetUpdate(bool UD)
{Update=UD;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetConsistent(iboolean &CST)
    \brief Set the consistency flag.
    \param CST consistency flag.*/
void strip::SetConsist(iboolean &CST)
{Consistent=CST;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetJoinFlag(bool JF)
    \brief Set Join flag.
    \param UD (true=>Needs to be Joined; flase=No needs;)*/
void strip::SetJoinFlag(bool JF)
{JoinFlag=JF;
}//-----------------------------------------------------------------------------

/** \fn void strip::SetExcess(ivalDb &ECSS)
    \brief.
    \param */
void strip::SetExcess(ivalDb& EXCSS)
{Excess=EXCSS;
}//-----------------------------------------------------------------------------

void strip::SetStock(list< list<cell>::iterator>& S)
{Stock=S;
}//-----------------------------------------------------------------------------

void strip::SetPStock(list< list<cell>::iterator>::iterator PS)
{PStock=PS;
}//-----------------------------------------------------------------------------

void strip::SetTOptimal(bool TO)
{TOptimal=TO;
}//-----------------------------------------------------------------------------

void strip::SetInsertedCells(list< list<cell>::iterator >& IC)
{InsertedCells=IC;
}//-----------------------------------------------------------------------------
/*
void strip::SetPenalty(int P)
{Penalty=P;
}//-----------------------------------------------------------------------------

void strip::Penalize()
{Penalty++;
}//-----------------------------------------------------------------------------
*/

void strip::SetBisectXp(bool BXp)
{BisectXp=BXp;
}//-----------------------------------------------------------------------------

void strip::SetZeroIncluded(bool ZI)
{ZeroIncluded=ZI;
}//-----------------------------------------------------------------------------

void strip::SetZeroExcluded(bool ZE)
{ZeroExcluded=ZE;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn bool IsMono(void)
    \return true if it is totaly monotonic*/
bool strip::IsTotalyMono(void)
{bool aux=true;
 int i=0;
 while(i<this->GetDim() && aux==true)
 {if(this->operator[](i).IsTotalyMono()==true)aux=true;
  else aux=false;
  i++;}
 return aux;
}//-----------------------------------------------------------------------------

/** \fn bool IsCellsEmpty(void)
    \return true if the Cells list is empty and false othewise.*/
bool strip::IsCellsEmpty(void)
{if(this->Cells.empty()) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn void strip::AddCellToCells(cell &CELL)
    \brief Add Cell To Cells and Update Strip Twin.*/
void strip::AddCellToCells(cell &CELL)
{
 Cells.push_back(CELL);
 Cells.back().SetXpStrip(this->Xp);
}//-----------------------------------------------------------------------------

/** \fn void strip::AddCellToWareHouse(cell &CELL)
    \brief Add Add Cell To WareHouse and Update Strip Twin.*/
void strip::AddCellToWareHouse(cell &CELL)
{WareHouse.push_back(CELL);
 WareHouse.back().SetXpStrip(this->Xp);
}//-----------------------------------------------------------------------------

/** \fn void strip::AssignCell(int Strategy)
    \brief Assign Cell to be bisected depending on the strategy.
    \param Update (Used for Strategy 1) True =>  Cell has not been selected;
    False => Cell has not been selected yet.
    \param Strategy Parameter selected by the user.
    Strategt==1 => From the whole list of cells List, select the Cell with the
    biggest tolerance. This task has already been done updating the Strip.
    Strategt==2 => From the last set of inserted cells corresponding,
    to the last iteration, select the one with the biggest tolerance.: */
void strip::AssignCell(fstream &fout,param &Pars)
{double Bigger=MInfinity();
 list<cell>::iterator Cell;
 list< list<cell>::iterator> Store;
 list< list<cell>::iterator> Trash;
 list<list<cell>::iterator>::iterator TCell;
 //int AuxPenalty=999999;

 if(this->Update || this->Stock.empty())
 {
    this->Stock.clear();

    for (Cell=this->WareHouse.begin(); Cell != this->WareHouse.end(); Cell++)  // Transfer Cells From WareHouse To Cells List
    {
     if(Cell->IsXiTotalyMono()){}                                // Xi Totally Monotonic => Do nothing
     //else if(Cell->GetNonMonoXiBigWidth()/Pars.GetXWidths()[Cell->GetXp().size()+Cell->GetXiDim()] <= Pars.GetEps()){}    // Smaller Than Epsilon  => Do nothing
     else if(Cell->GetNonMonoXiBigWidth() <= Pars.GetEps()){}    // Smaller Than Epsilon  => Do nothing
     else if(!this->IsIncluded(*Cell,fout,Pars))                 // IS Not Included => Put to Cells
        {Store.push_back(Cell);}
    }

    for (TCell=Store.begin(); TCell != Store.end(); TCell++)
    {this->Cells.splice(this->Cells.end(),this->WareHouse,*TCell);}

    Store.clear();


    for (Cell=this->Cells.begin(); Cell != this->Cells.end(); Cell++)
    {
      if(Pars.GetOptim() ==1) {Cell->GetTwin().AssignInfTol();}
      else if(Pars.GetOptim() ==2) {Cell->GetTwin().AssignSupTol();}

      // Add this->Stocked cells to a match list.
      if(Cell->GetConsist()==iperhaps)
      {PrintMsg("Eliminate Cell: Consistency",fout,Pars.GetDebug());
       this->Stock.push_back(Cell);}

      else if(Cell->GetConsist()==itrue && Cell->IsXiTotalyMono())
      {Store.push_back(Cell);
       PrintMsg("Accumulate Cell: Xi Monotony",fout,Pars.GetDebug());
       Cell->Print(fout,Pars.GetDebug());}

      else if(this->IsIncluded(*Cell,fout,Pars))     // Check Inclusion Criteria
        {Store.push_back(Cell);
         PrintMsg("Accumulate Cell: Inclusion Criterion",fout,Pars.GetDebug());
         Cell->Print(fout,Pars.GetDebug());}
      else if(this->IsEliminateCell(*Cell,fout,Pars))     // Check if the Cell can be eliminated
        {Trash.push_back(Cell);}

        /*
      else if(Pars.GetInclusion()==1 && Cell->GetZeroIncluded())     // Check if the Cell can be eliminated
        {Trash.push_back(Cell);}

      else if((Pars.GetInclusion()==1 || Pars.GetInclusion()==2) && Cell->GetZeroExcluded())     // Check Inclusion Criteria
        {Store.push_back(Cell);}
       */
      /*
      else if(Pars.GetOptim() !=2 &&  Cell->GetTwin().GetInf() <<= this->Twin.GetInf())
       {this->Stock.push_back(Cell);}
      else if(Pars.GetOptim()!=1 && Cell->GetTwin().GetSup() <<= this->Twin.GetSup())
        {this->Stock.push_back(Cell);}
      */
      

       else if(Pars.GetOptim() !=2 &&  (Cell->GetTwin().GetInn().GetInf() == this->Twin.GetInn().GetInf() ||
         Cell->GetTwin().GetOut().GetInf() == this->Twin.GetOut().GetInf()))
       {this->Stock.push_back(Cell);}
      else if(Pars.GetOptim()!=1 && (Cell->GetTwin().GetInn().GetSup() == this->Twin.GetInn().GetSup() ||
              Cell->GetTwin().GetOut().GetSup() == this->Twin.GetOut().GetSup()))
        {this->Stock.push_back(Cell);}
      


      //else {this->Stock.push_back(Cell);}

    }

  for (TCell=Store.begin(); TCell != Store.end(); TCell++)
    {this->WareHouse.splice(this->WareHouse.end(),this->Cells,*TCell);}
     Store.clear();
  for (TCell=Trash.begin(); TCell != Trash.end(); TCell++)
    {Cells.erase(*TCell);}
     Trash.clear();

 } // end if(this->Update || this->Stock.empty())

 // Choose the Cell with the biggest Tolerance from the match list
 list< list<cell>::iterator>::iterator ICell;
 if(this->Stock.empty())
     {this->SelectedCell=NULL;}
 else
    {
      for (ICell=this->Stock.begin(); ICell != this->Stock.end(); ICell++)
      {
        //if(((*ICell)->GetPenalty() <= AuxPenalty) && ((*ICell)->GetTwin().GetTol() > Bigger))  // Choose the Strip with the biggest Tolerance.
        if(((*ICell)->GetTwin().GetTol() > Bigger))  // Choose the Strip with the biggest Tolerance.
            {this->SelectedCell=*ICell;
             Bigger=(*ICell)->GetTwin().GetTol();
             this->PStock=ICell;
             //AuxPenalty=(*ICell)->GetPenalty();
             }                             // Point Selected Cell
      }
    }
}//-----------------------------------------------------------------------------

/** \fn  bool strip::EliminateCell(cell &Cell,fstream &fout,param &Pars)
    \brief Eliminate Cell from the Strip.*/
bool strip::IsEliminateCell(cell &Cell,fstream &fout,param &Pars)
{
 iboolean Consistent=Cell.GetConsist();     // Get Consistency.

 if(Consistent==ifalse)
  {
    if(Pars.GetPlot()) PlotCell(Cell,clGreen,clBlack,bsSolid,Pars);
    return true;
  }
 else
  {return false;}
}//-----------------------------------------------------------------------------

/** \fn  bool strip::IsIncluded(cell &Cell,fstream &fout,param &Pars)
    \brief Is Included Test.*/
bool strip::IsIncluded(cell &Cell,fstream &fout,param &Pars)
{
 int OptimLevel=Pars.GetOptim();

 twin GTW=this->GetTwin();
 twin CTW=Cell.GetTwin();
 
 // Eliminate:  Inclusion Criterion  ( Canvi important: Consistent==itrue && )
 if((OptimLevel==0 || OptimLevel==3) && IsIn(this->GetTwin().GetOut(),Cell.GetTwin().GetInn()) &&  this->GetJoinFlag())
  {
  //PrintMsg("Eliminate Cell: Inclusion Criterion",fout,Pars.GetDebug());
   Cell.Print(fout,Pars.GetDebug());
   if(Pars.GetPlot()) PlotCell(Cell,clBlack,clWhite,bsSolid,Pars);
   return true;
  }

 // Eliminate: MinMax Optimization
 else if(OptimLevel==1 && this->GetTwin().GetOut().GetInf() > Cell.GetTwin().GetInn().GetInf() && this->GetJoinFlag())
  {
   //PrintMsg("Eliminate Cell: MinMax Criterion",fout,Pars.GetDebug());
   if(Pars.GetPlot()) PlotCell(Cell,clSilver,clSilver,bsSolid,Pars);
   return true;
  }

 // Eliminate: MaxMin Optimization
else if(OptimLevel==2 && this->GetTwin().GetOut().GetSup() < Cell.GetTwin().GetInn().GetSup()  && this->GetJoinFlag())
  {
   //PrintMsg("Eliminate Cell: MaxMix Criterion",fout,Pars.GetDebug());
   if(Pars.GetPlot()) PlotCell(Cell,clTeal,clTeal,bsSolid,Pars);
   return true;
  }

else

{return false;}
}//-----------------------------------------------------------------------------

/** \fn double GetXpBigWidth(void)
    \brief Get biggest proper width and assign proper dimention to bisect.*/
void strip::AssignXpBigDim(void)
{double BWidth=MInfinity();
 ivalDb Domain;
 int Dim=-1;

 if(this->BisectXp)
 {
   for(int i=0 ;i<this->Xp.size();i++)
   {Domain=this->Xp[i].GetIval();
    if(Width(Domain) > BWidth)
          {BWidth=Width(Domain);
           Dim=i;}
   }
 }
 this->XpDim=Dim;
}//-----------------------------------------------------------------------------

/** \fn double GetNonMonoXpBigWidth(int TOptim)
    \brief Get Non monotonic biggest proper width and assign proper dimention to bisect.
    \param TOptim: Tree optimality flag.*/
/*
void strip::AssignNonMonoXpBigDim(void)
{double BWidth=MInfinity();
 int Dim= -1;
 bool Mono,UniInci;
 ivalDb Dom;
 int BPenalty=-1;
 int Penalty;

 if(this->BisectXp)
 {
   for(int i=0 ;i<this->Xp.size();i++)
   {
    Dom=this->Xp[i].GetIval();
    Mono=this->Xp[i].IsTotalyMono();
    UniInci=this->Xp[i].IsUniInci();
    Penalty=this->Xp[i].GetPenalty();
    if(this->Consistent==itrue && ((this->TOptimal && UniInci) || Mono)){}    // Do dot select (Tree Optimality + Unincident or Monotonic
    else if(Penalty > BPenalty)
          {BPenalty=Penalty;
           Dim=i;}
    else if(Penalty == BPenalty && Width(Dom) > BWidth)         // Select the Non Monotonic biggest width.
      {BWidth=Width(Dom);
       Dim=i;}
   }
 }
 this->XpDim=Dim;
}//-----------------------------------------------------------------------------
*/

void strip::AssignNonMonoXpBigDim(param& Pars)
{double BWidth=MInfinity();
 int Dim= -1;
 ivalDb Dom,MonDom;
 bool Mono,UniInci;
 double XWidth;

 if(this->BisectXp)
 {
   for(int i=0 ;i<this->Xp.size();i++)
   {
    Dom=this->Xp[i].GetIval();
    Mono=this->Xp[i].IsTotalyMono();
    UniInci=this->Xp[i].IsUniInci();
    //XWidth=Pars.GetXWidths()[i];
    if(this->Consistent==itrue && ((this->TOptimal && UniInci) || Mono)){}    // Do dot select (Tree Optimality + Unincident or Monotonic
    //else if(Width(Dom)/XWidth > BWidth)         // Select the Non Monotonic biggest width.
    else if(Width(Dom) > BWidth)         // Select the Non Monotonic biggest width.
          {
           //BWidth=Width(Dom)/XWidth;
           BWidth=Width(Dom);
           Dim=i;}
   }
 }
 this->XpDim=Dim;
}//-----------------------------------------------------------------------------

double strip::GetNonMonoXpBigWidth(void)
{if(this->XpDim>=0)
 {ivalDb Interval=this->Xp[this->XpDim].GetIval();
  return Width(Interval);}
 else {return MInfinity();}
}//-----------------------------------------------------------------------------

/** \fn void ComputeUpdateTwin(bool Compute,fstream &fout,model &Model)
    \param Compute (Compute==true => Compute & Update; Compute==false => Only Update)
    \param fout File stream output.
    \param Model Model Structure.
    \brief Compute the Strip twin  computing doing the meet of the cells twin contained
    by the Cells list and the warehouse list. Compute the Cells Twin if the parameter
    "Compute"==true.*/
void strip::ComputeUpdateTwin(bool Compute,fstream &fout,model &Model,param &Pars)
{
if(this->Update)           // If Update == true
{
   int Dim;
   int Print=Pars.GetDebug();
   list<cell>::iterator Cell;
   this->Consistent=empty;
   iboolean CellConsist;
   tree* Tree=&Model.GetTree();

   //PrintMsg("Update Strip",fout,Print);
   
   this->Twin.SetInn(NaN());
   this->Twin.SetOut(NaN());

   if(Compute)
    {
     AssignStripToTree(*Tree,*this);
     this->ComputeMonos(*Tree);
     this->AssignTreeOptim(*Tree);       // (NEW!!!!!!!!) Assign Tree Optimallity
     }

   // Cells List Meet (Compute==false => Only Update)
   for (Cell=this->Cells.begin(); Cell != this->Cells.end(); Cell++)
     {
      if(Compute)                        // Only if Compute==true (Bisected Strip)
      {
         Dim=this->GetXpDim();
         if(Dim>=0) {Cell->GetXp()[Dim].SetIval(this->Xp[Dim].GetIval());}

         Cell->SetXpStrip(this->Xp);     // Point Xp Cell pointer to Xp Strip vector

         CellConsist=Model.ConstraintsConsistency(*Cell);
         Cell->SetConsist(CellConsist);

         if(Cell->GetConsist()!=ifalse)
            {Cell->ComputeTwin(Model.GetTree(),this->TOptimal,Pars);}

         if(CellConsist==itrue)
            {this->JoinFlag=true;}    // Set Strip Join flag

      } // End if(Compute)

     this->MeetCell(*Cell);

    }  // End Cells iteration

    // WareHouse List Meet (Compute==false => Only Update)
   for (Cell=this->WareHouse.begin(); Cell != this->WareHouse.end(); Cell++)
     {
      if(Compute)
       {
         Dim=this->GetXpDim();
         if(Dim>=0) {Cell->GetXp()[Dim].SetIval(this->Xp[Dim].GetIval());}

         Cell->SetXpStrip(this->Xp);     // Point Xp Cell pointer to Xp Strip vector

         CellConsist=Model.ConstraintsConsistency(*Cell);
         Cell->SetConsist(CellConsist);

        if(CellConsist!=ifalse)
          {Cell->ComputeTwin(Model.GetTree(),this->TOptimal,Pars);}   // Compute Twin

        } // End if(Compute)

      this->MeetCell(*Cell);
     } // End WareHouse iteration

} // end if(this->Update)

 this->Twin.AssignTol();


 if(Compute) {this->AssignNonMonoXpBigDim(Pars);}
 
 this->AssignCell(fout,Pars);         // Assign Cell To Bisect
 /*
 if(IsIn(this->Twin.GetOut(),Zero()))    // For Testing
 {this->ZeroIncluded=true;}
 else
 {this->ZeroIncluded=false;}

 if(!IsIn(this->Twin.GetInn(),Zero()))    // For Testing
 {this->ZeroExcluded=true;}
 else
 {this->ZeroExcluded=false;}
*/
}//-----------------------------------------------------------------------------

// Meet Cell To Strip Twin and Set Consistency flag to Strip
void strip::MeetCell(cell &Cell)
{
if(Cell.GetConsist()!=ifalse)           // if the cell is itrue or iperhaps
{
    this->Twin.SetInn(this->Twin.GetInn() && Cell.GetTwin().GetInn());

    if(Cell.GetConsist()==itrue)
     {this->Twin.SetOut(this->Twin.GetOut() && Cell.GetTwin().GetOut());}
    else if(Cell.GetConsist()==iperhaps && this->JoinFlag==false)                          // Canvi!!!!!!
     {this->Twin.SetOut(this->Twin.GetOut() || Cell.GetTwin().GetOut());}
}

iboolean CellConsist=Cell.GetConsist();

if(CellConsist==iperhaps)
  {this->Consistent=iperhaps;}
else if(this->Consistent==iperhaps){}
else
 { if(this->Consistent==empty)
      {this->Consistent=CellConsist;}
   else if(this->Consistent==itrue && (CellConsist==ifalse || CellConsist==iperhaps))
      {this->Consistent=iperhaps;}
   else if(this->Consistent==itrue && CellConsist==itrue)
      {this->Consistent=itrue;}
   else if(this->Consistent==ifalse && (CellConsist==itrue || CellConsist==iperhaps))
      {this->Consistent=iperhaps;}
   else if(this->Consistent==ifalse && CellConsist==ifalse)
      {this->Consistent=ifalse;}
 }
}//-----------------------------------------------------------------------------


void strip::ComputeMonos(tree &T)
{
 string Mode="Full";
 ivalDb MonoDom=0;
 ivalDb MonoDomInc;
 int index;

 T.EvaluateDers(Mode);

 if(T.GetDers().size()>0)
 {
   for(int i=0;i<this->GetXp().size();i++)
   {MonoDom=0;
    index=T.GetXpIndexes()[i];
     for(int j=0;j<this->GetXp()[i].GetNumIncs();j++)
      {
       MonoDomInc=T.GetVars()[index][j]->GetMonoDom();
       if(IsIn(0,MonoDomInc))
        {MonoDom=Infinity(); break;}
       else
        {MonoDom=MonoDom + MonoDomInc;}
       }
     this->GetXp()[i].SetMonoDom(MonoDom);
   }

   for(int i=0;i<this->GetXi().size();i++)
   { MonoDom=0;
     index=T.GetXiIndexes()[i];
     for(int j=0;j<this->GetXi()[i].GetNumIncs();j++)
      {
       MonoDomInc=T.GetVars()[index][j]->GetMonoDom();
       if(IsIn(0,MonoDomInc))
        {MonoDom=Infinity(); break;}
       else
        {MonoDom=MonoDom + MonoDomInc;}
       }
     this->GetXi()[i].SetMonoDom(MonoDom);
   }

   this->AssignMonos();
 }
}//-----------------------------------------------------------------------------

// Assign Cell incidence pointers
void AssignStripToTree(tree &Tree,strip& Strip)
{int index;

 for(int i=0;i<Strip.GetXp().size();i++)
 { index=Tree.GetXpIndexes()[i];
   for(int j=0;j<Strip.GetXp()[i].GetNumIncs();j++)
    {Tree.GetVars()[index][j];
     Tree.GetVars()[index][j]=&Strip.GetXp()[i][j];}
 }
 for(int i=0;i<Strip.GetXi().size();i++)
 { index=Tree.GetXiIndexes()[i];
   for(int j=0;j<Strip.GetXi()[i].GetNumIncs();j++)
    {Tree.GetVars()[index][j]=&Strip.GetXi()[i][j];}
 }
}//-----------------------------------------------------------------------------

/** \fn void AssignMonos(void)
    \brief Assign Monotonies.*/
void strip::AssignMonos(void)
{for(int i=0 ;i<this->GetDim();i++)
 {this->operator[](i).AssignMono();
  for(int j=0 ;j<this->operator[](i).GetNumIncs();j++)
  {this->operator[](i)[j].AssignMono();}
  this->operator[](i).AssignTotalMono();
 }
}//-----------------------------------------------------------------------------

/** \fn void Print(void)
    \brief Print Strip to output stream.
    \param fout output stream.
    \param Print print flag (Print==1 => Prints)*/
void strip::Print(fstream &fout,int Print,bool PrintCells)
{
 if(Print==1)
 {
  fout << endl << "Strip#####################################################" << endl << endl;
   for(int i=0 ;i<this->GetXp().size();i++)
  {fout << this->GetXp()[i].GetName() << "[" << i << "].Dom=" << this->GetXp()[i].GetIval()  << " Mono=" << this->GetXp()[i].GetMono()<< endl;
   /*
   fout << "S[" << i << "].MonoDom=" << this->operator[](i).GetMonoDom()  << endl << endl;
   fout << "S[" << i << "].Mono=" << this->operator[](i).GetMono()<< endl << endl;
   for(int j=0 ;j<this->operator[](i).GetNumIncs();j++)
   {fout << "S[" << i << "][" << j << "].Dom=" << this->operator[](i)[j].GetIvalVar() << endl;
    fout << "S[" << i << "][" << j << "].MonoDom=" << this->operator[](i)[j].GetMonoDom() << endl;
    fout << "S[" << i << "][" << j << "].Mono=" << this->operator[](i)[j].GetMono()<< endl << endl;}
   */ 
  }

  fout << this->Twin << endl;

  if(PrintCells)
  {
   fout << "Cells List:"  << endl << endl;
   list<cell>::iterator Cell;
   for (Cell=this->Cells.begin(); Cell != this->Cells.end(); Cell++)
   {
    Cell->Print(fout,Print);
   }
   fout << endl << "WareHouse List:"  << endl << endl;
   for (Cell=this->WareHouse.begin(); Cell != this->WareHouse.end(); Cell++)
   {
    Cell->Print(fout,Print);
   }
  }
  fout << "Join Flag: " << this->JoinFlag << endl;
  //fout << "Penalty: " << this->Penalty << endl << endl;
  fout << "#######################################################Strip" << endl << endl;
 }
}//-----------------------------------------------------------------------------

void strip::AssignTreeOptim(tree &T)
{
 if(T.GetTOptimal())
 {this->TOptimal=true;}
 else
 {
   if(T.GetUniIncident())
   {this->TOptimal=T.IsDomainTOptimal();}
   else
   {this->TOptimal=false;}
 }
}//-----------------------------------------------------------------------------

/** \fn bool IsProp(void)
    \return Return if it is proper*/
bool strip::IsProp(void)
{if(this->Xi.size()==0) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn void strip::Contract(ivalDb &Out, int Dim)
    \brief Contract the Strip dimention "Dim"*/
void strip::Contract(ivalDb &Out, int Dim)
{
 list<cell>::iterator Cell;
 ivalDb NewDom=this->operator [](Dim).GetIval() && Out;

 this->operator [](Dim).SetIval(NewDom);

 for (Cell=this->Cells.begin(); Cell != this->Cells.end(); Cell++)
 {
  Cell->operator [](Dim).SetIval(NewDom);
 }  // End Cells iteration

 for (Cell=this->WareHouse.begin(); Cell != this->WareHouse.end(); Cell++)
 {
  Cell->operator [](Dim).SetIval(NewDom);
 } // End WareHouse iteration

}//-----------------------------------------------------------------------------

bool strip::IsThereImprovement(twin& OLDTW)
{
 list< list<cell>::iterator >::iterator Cell;
 ivalDb Ival=ivalDb(NaN());
 bool Improved;

 twin NEWTW=this->GetTwin();

 if(NEWTW.GetTol() < OLDTW.GetTol())
 {Improved=true;}
 else
 {
  for(Cell=this->InsertedCells.begin(); Cell != this->InsertedCells.end(); Cell++)
  {this->Cells.erase(*Cell);}
  Improved=false;
  //this->SelectedCell->Penalize();
  //this->SelectedCell->GetXi()[this->SelectedCell->GetXiDim()].Penalize();
 }

 this->InsertedCells.clear();

 return Improved;
}//-----------------------------------------------------------------------------

