/*******************************************************************************
 *    tree.h: Syntactic Tree
 *    29/12/2002
 *    By Pau Herrero Vias
 ******************************************************************************/

#include <fstream.h>
#include "node.h"
#include "param.h"
#include <vector.h>

#ifndef TREE_HH
#define TREE_HH

typedef vector< vector<node*> > ders;
typedef vector< vector< list<node*> > > derlist;

class tree
{
 private:
        node* Root;
        vars Vars;
        ders Ders;
        deque<node> NodeVars;
        ivalDb Result;
        list<node*> FunctList;
        derlist DersLists;
        list<node*> NonUnifMonoNodes;           //> List of Non Uniformaly Monotone nodes
        bool UniIncident;                       //> UniIncidence Flag
        vector<int> XiUniIndexes;               //> Vector of improper Uni-Incident variables (Indexes)
        vector<int> XpUniIndexes;               //> Vector of proper Uni-Incident variables (Indexes)
        bool UniModal;                          //> Uni-Modal Flag
        bool TOptimal;                          //> Tree Optimality Flag
        deque<int> XpIndexes;                   //> Indexes of the Xp vector
        deque<int> XiIndexes;                   //> Indexes of the Xi vector
        vector< deque<int> > XCrossDers;      //> Indexes of the Xp derivatives dependencies
        list<node> NodesStock;                  // List of nodes used by the tree

 public:

    //////////////////
    // SQT Attributs//
    //////////////////

      string State;                     // State variable
      node* StateVar;
      deque< deque<node*> > Measurements;
      vector<int> MeasurementsIndexes;
      int StateIndex;


    //////////////////
    // Constructors //
    //////////////////

    tree(void);
    tree(node* N);
    tree(const tree &T);

    ////////////////
    // Destructor //
    ////////////////

    ~tree(){}

    //////////////////////
    // Acces to members //
    //////////////////////

    node* GetRoot();
    vars& GetVars();
    ders& GetDers();
    deque<node>& GetNodeVars();
    ivalDb& GetResult(void);
    list<node*>& GetFunctList(void);
    derlist& GetDersLists(void);
    list<node*>& GetNonUnifMonoNodes(void);
    vector<int>& GetXpUniIndexes(void);
    vector<int>& GetXiUniIndexes(void);
    bool GetUniIncident(void);
    bool GetUniModal(void);
    bool GetTOptimal(void);
    deque<int>& GetXpIndexes(void);
    deque<int>& GetXiIndexes(void);
    vector< deque<int> >& GetXCrossDers(void);
    list<node>& GetNodesStock();

    void SetRoot(node*);
    void SetVars(vars&);
    void SetDers(ders&);
    void SetNodeVars(deque<node>&);
    void SetResult(ivalDb&);
    void SetFunctList(list<node*>&);
    void SetDersLists(derlist&);
    void SetNonUnifMonoNodes(list<node*>&);
    void SetXpUniIndexes(vector<int>&);
    void SetXiUniIndexes(vector<int>&);
    void SetUniIncident(bool);
    void SetUniModal(bool);
    void SetTOptimal(bool);
    void SetXCrossDers(vector< deque<int> >&);
    void SetNodesStock(list<node>&);

    //////////////////////
    // Member functions //
    //////////////////////

    ivalDb EvaluateFunct(string& Mode,bool STOpt,bool CTOpt,iboolean &Consist,char Point);
    void EvaluateDers(string &Mode);
    void AddNodeVar(node&);
    node* Diff(incidence* Inc,string &Type);
    void Diff(void);
    node* Diff(string &VarName);
    void DeleteTree(void);
    void PrintTrees(void);
    bool IsDomainTOptimal(void);
    void ConvertTreesToLists(void);
    void PrintLists(void);
    void AssignNonUnifNodes(void);
    void AssignTOptimality(void);
    void CreateBoxContractor(string &VarName);
    int GetVarIndex(string &NameVar);
    void NormalizeConstraint(param&);
    void Negate(void);
    void AssignModes(bool Mode,param &Pars);
    void ToLatex(char * filename);
    void InitializeNodes();
    void ForwardPropagation(fstream &fout,int &Counter,string &Mode);
    void BackwardPropagation(fstream &fout,int &Counter);

    void AssignStateMeasurements(void);
    void BuildWindowTree(int WindowLength);


    ////////////////////////////////////
    // Monary Operators and Functions //
    ////////////////////////////////////

    tree& operator=(const tree& N);

    ////////////////////////////
    // Binary Operators //
    ////////////////////////////

    ////////////////////////////
    // Binary Operators //
    ////////////////////////////

    ////////////////////////////
    // Monary Operators //
    ////////////////////////////

    //////////////////////////
    // I/O Stream Operators //
    //////////////////////////

    friend ostream& operator<<(ostream& out, tree& N);

};

#endif
