 /*!****************************************************************************
 *    \file twin.cpp
 *    This file contains the mplementation of the twin class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "twin.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
  \fn twin(void)*/
twin::twin()
{Inner=DuInfinity();
 Outer=Infinity();
 Tolerance=PInfinity();
}//-----------------------------------------------------------------------------

/** Constructor 1.
    \fn twin(ivalDb &Inn,ivalDb &Out)
    \param Inn Inner approximation
    \param Out Outer approximation */
twin::twin(ivalDb &Inn,ivalDb &Out)
{Inner=Inn;
 Outer=Out;
 this->AssignTol();
}//-----------------------------------------------------------------------------

/** Constructor 2.
    \fn twin(string c)
    \param Type="xy". x==n -> Inner=[-Inf,+Inf]; x==d -> Inner=[+Inf,-Inf]
    y==n -> Outer=[-Inf,+Inf]; y==d -> Outer=[+Inf,-Inf]*/
twin::twin(string Type)
{if(Type=="nn")
 {Inner=Infinity();
  Outer=Infinity();}
 else if(Type=="dd")
 {Inner=DuInfinity();
  Outer=DuInfinity();}
 else if(Type=="nd")
 {Inner=Infinity();
  Outer=DuInfinity();}
 else if(Type=="dn")
 {Inner=DuInfinity();
  Outer=Infinity();}
}//-----------------------------------------------------------------------------

/** Copy Constructor.
    \fn twin(const twin &Twin)*/
twin::twin(const twin &Twin)
{Inner=Twin.Inner;
 Outer=Twin.Outer;
 Tolerance=Twin.Tolerance;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn int GetInn() const
    \return Inner approximation.*/
ivalDb twin::GetInn() const
{return Inner;
}//-----------------------------------------------------------------------------

/** \fn int GetOut() const
    \return Outer approximation.*/
ivalDb twin::GetOut() const
{return Outer;
}//-----------------------------------------------------------------------------

/** \fn int GetInf() const
    \return the interval composed by [Outer.Inf,Inner.Inf]*/
ivalDb twin::GetInf() const
{return Ival(Outer.GetInf(),Inner.GetInf());
}//-----------------------------------------------------------------------------

/** \fn int GetSup() const
    \return the interval composed by [Inner.Sup,Outer.Sup]*/
ivalDb twin::GetSup() const
{return Ival(Inner.GetSup(),Outer.GetSup());
}//-----------------------------------------------------------------------------

/** \fn int GetTol() const
    \return Tolerance between approximations*/
double twin::GetTol() const
{return Tolerance;
}//-----------------------------------------------------------------------------

/** \fn int GetInfTol() const
    \return Tolerance correaponding to [Outer.Inf,Inner.Inf]*/
double twin::GetInfTol() const
{
 double Tol;
 if(IsInfinity(Inner.GetInf()) || IsInfinity(Outer.GetInf())) Tol=PInfinity();
 else Tol=FDLIBM::fabs(Inner.GetInf()-Outer.GetInf());
 return Tol;

// return this->InfTol;
}//-----------------------------------------------------------------------------

/** \fn int GetSupTol() const
    \return Tolerance correaponding to [Inner.Sup,Outer.Sup]*/
double twin::GetSupTol() const
{
 double Tol;
 if(IsInfinity(Inner.GetSup()) || IsInfinity(Outer.GetSup())) Tol=PInfinity();
 else Tol=FDLIBM::fabs(Inner.GetSup()-Outer.GetSup());
 return Tol;

// return this->SupTol;
}//-----------------------------------------------------------------------------

/** \fn void SetInn(ivalDb& Inn)
    Set Inner approximation.*/
void twin::SetInn(ivalDb& Inn)
{Inner=Inn;
}//-----------------------------------------------------------------------------

/** \fn void SetOut(ivalDb& Out)
    Set Outerer approximation.*/
void twin::SetOut(ivalDb& Out)
{Outer=Out;
}//-----------------------------------------------------------------------------

/** \fn void SetTol(double& Tol)
    Set Tolerance.*/
void twin::SetTol(double& Tol)
{Tolerance=Tol;
}//-----------------------------------------------------------------------------

void twin::SetInf(ivalDb& I)
{Outer.SetInf(I.GetInf());
 Inner.SetInf(I.GetSup());
}//-----------------------------------------------------------------------------

void twin::SetSup(ivalDb& S)
{Outer.SetSup(S.GetSup());
 Inner.SetSup(S.GetInf());
}//-----------------------------------------------------------------------------


/** \fn void Print(fstream &fout,int PrintFlag)
    Print twin to stream fout if PrintFlag>0.*/
void twin::Print(fstream &fout,int PrintFlag)
{if(PrintFlag>0)
 {
 fout << "Twin: " << "Inn:" << Inner << endl;
 fout << "      " << "Out:" << Outer << endl;
 fout << "Tol:" << Tolerance << endl;
 }
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Binary Inner Operators                                                     //
////////////////////////////////////////////////////////////////////////////////

/** \fn twin& twin::operator=(const twin &Twin)
    Overloaded assignation operator*/
twin& twin::operator=(const twin &Twin)
{Inner=Twin.Inner;
 Outer=Twin.Outer;
 Tolerance=Twin.Tolerance;
 return *this;
}//-----------------------------------------------------------------------------

/** \fn twin operator&&(const twin &x,const twin &y)
 Overloaded Meet operator */
twin operator&&(const twin &x,const twin &y)
{ivalDb Inner,Outer;
 Inner=x.GetInn() && y.GetInn();
 Outer=x.GetOut() && y.GetOut();
 return twin(Inner,Outer);
}//-----------------------------------------------------------------------------

/** \fn twin operator||(const twin &x,const twin &y)
 Overloaded Join Operator operator */
twin operator||(const twin &x,const twin &y)
{ivalDb Inner,Outer;
 Inner=x.GetInn() || y.GetInn();
 Outer=x.GetOut() || y.GetOut();
 return twin(Inner,Outer);
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn void twin::AssignTol(void)
    Assign Tolerance*/
void twin::AssignTol(void)
{
 if(this->IsInfty()) this->Tolerance=PInfinity();
 else
 {double Tol1=Outer.GetInf()-Inner.GetInf();
  double Tol2=Outer.GetSup()-Inner.GetSup();
  this->Tolerance=max(FDLIBM::fabs(Tol1),FDLIBM::fabs(Tol2));}
}//-----------------------------------------------------------------------------


/** \fn void twin::AssignInfTol(void)
    Assign Infimmum Tolerance*/
void twin::AssignInfTol(void)
{
 if(IsInfinity(Inner.GetInf()) || IsInfinity(Outer.GetInf()))
  {this->Tolerance=PInfinity();}
 else
  {this->Tolerance=FDLIBM::fabs(Inner.GetInf()-Outer.GetInf());}
}//-----------------------------------------------------------------------------

/** \fn void twin::AssignSupTol(void)
    Assign Supremmum Tolerance*/
void twin::AssignSupTol(void)
{
 if(IsInfinity(Inner.GetSup()) || IsInfinity(Outer.GetSup())) this->Tolerance=PInfinity();
 else this->Tolerance=FDLIBM::fabs(Inner.GetSup()-Outer.GetSup());
}//-----------------------------------------------------------------------------

/** \fn bool twin::IsInfinity(void)
    Return true if one of the bounds of the approximations is infinity*/
bool twin::IsInfty(void)
{if(IsInfinity(Inner.Sup) || IsInfinity(Inner.Inf) ||
 IsInfinity(Outer.Sup) || IsInfinity(Outer.Inf) )
 return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool twin::IsRelTol(double Tol)
    Check if the twin tolerance is smaller than the relative tolerance Tol.*/
bool twin::IsRelTol(double Tol)
{ double Inf=min(FDLIBM::fabs(Outer.Inf),FDLIBM::fabs(Inner.Inf));
  double Sup=min(FDLIBM::fabs(Outer.Sup),FDLIBM::fabs(Inner.Sup));
  if (Inf==0) Inf=1;
  if (Sup==0) Sup=1;
  if(!this->IsInfty() && this->GetInfTol() <= Tol*Inf && this->GetSupTol() <= Tol*Sup)
  return true;
  else return false;
}//-----------------------------------------------------------------------------

bool twin::IsImproved(twin &TW)
{
 if((TW.Inner.GetInf() < this->Inner.GetInf()) ||
    (TW.Inner.GetSup() > this->Inner.GetSup()) ||
    (TW.Outer.GetInf() > this->Outer.GetInf()) ||
    (TW.Outer.GetSup() < this->Outer.GetSup()))
 {return true;}
 else
 {return false;}
}//-----------------------------------------------------------------------------


ostream& operator<<(ostream & out, const twin &TW)
{out << "Twin:" << ": " << "Inn:" << TW.GetInn() << endl;
 out << "       " <<         "Out:" << TW.GetOut() << endl;
 out << "Tol:" << TW.GetTol() << endl;
 return out;
}//-----------------------------------------------------------------------------

//##############################################################################

