 /*!****************************************************************************
 *    \file twin.h
 *    This file contains the definition of the twin class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "ivalDb.h"
#include <fstream.h>

#ifndef TWIN_HH
#define TWIN_HH

/*!*****************************************************************************
*    \class twin
*    \brief Class for the Star Algorithm.
*    This class represents an inner and an outer approximation of an interval
*    function for a given domain.
*******************************************************************************/
class twin
{
private:
         ivalDb Inner;          ///< Inner approximation.
         ivalDb Outer;          ///< Outer approximation.
         double Tolerance;      ///< Distance between inner and outer approximations.

public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

twin();
twin(ivalDb &Inner,ivalDb &Outer);
twin(const twin &Twin);
twin(string c);

 ////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~twin() {}

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

ivalDb GetInn() const;
ivalDb GetOut() const;
double GetTol() const;
double GetInfTol() const;
ivalDb GetInf() const;
ivalDb GetSup() const;
double GetSupTol() const;
void SetInn(ivalDb&);
void SetOut(ivalDb&);
void SetTol(double&);
void Print(fstream &fout,int PrintFlag);
void SetInf(ivalDb&);
void SetSup(ivalDb&);

void AssignTol(void);
void AssignInfTol(void);
void AssignSupTol(void);
bool IsInfty(void);
bool IsRelTol(double Tol);
bool IsImproved(twin &Twin);

////////////////////////////////////////////////////////////////////////////////
// Binary Inner Operators                                                          //
////////////////////////////////////////////////////////////////////////////////

twin& operator=(const twin &Twin);
friend twin operator&&(const twin &x,const twin &y);
friend twin operator||(const twin &x,const twin &y);

friend ostream& operator<<(ostream& out, const twin& x);

////////////////////////////////////////////////////////////////////////////////
// Friend Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

}; // end Class twin

#endif




