 /*!****************************************************************************
 *    \file variable.cpp
 *    This file contains the mplementation of the variable class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

 #include "variable.h"

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

/** Default Constructor.
  \fn variable(void)*/
variable::variable(void)
{Incs.resize(0);
 Ival=Infinity();
 Mono=0;
 MonoDom=Infinity();
 TotalMono=false;
 Penalty=0;
}//-----------------------------------------------------------------------------

/** Constructor 1.
    \fn variable(const int &NI)
    \param NI Number of incidences */
variable::variable(const int &NI)
{Incs.resize(NI);
 Ival=Infinity();
 Mono=0;
 MonoDom=Infinity();
 TotalMono=false;
 if(NI==1) {UniIncident=true;}
 else {UniIncident=false;}
 this->SetIncsPointers();
 Penalty=0;
}//-----------------------------------------------------------------------------

/** Constructor 2.
    \fn variable::variable(const int &NI,const ivalDb &Dom)
    \param NI Number of incidences.
    \param Dom Interval domian. */
variable::variable(const int &NI,const ivalDb &Dom)
{Ival=Dom;
 Incs.resize(NI);
 Mono=0;
 MonoDom=Infinity();
 TotalMono=false;
 if(NI==1) {UniIncident=true;}
 else {UniIncident=false;}
 this->SetIncsPointers();
 Penalty=0;
}//-----------------------------------------------------------------------------

/** Constructor 3.
    \fn variable::variable(const int &NI,const ivalDb &Dom)
    \param NI Number of incidences.
    \param Dom Interval domian.
    \param Nm Name of the variable (syntactic use)*/
variable::variable(const int &NI,const ivalDb &Dom,string Nm)
{Ival=Dom;
 Incs.resize(NI);
 Mono=0;
 MonoDom=Infinity();
 Name=Nm;
 TotalMono=false;
 if(NI==1) {UniIncident=true;}
 else {UniIncident=false;}
 this->SetIncsPointers();
 Penalty=0;
}//-----------------------------------------------------------------------------

/** Copy Constructor.
    \fn variable(const variable &Var)*/
variable::variable(const variable &Var)
{Ival=Var.Ival;
 Incs.resize(Var.GetNumIncs());
 Mono=Var.Mono;
 MonoDom=Var.MonoDom;
 Name=Var.Name;
 TotalMono=Var.TotalMono;
 UniIncident=Var.UniIncident;
 this->SetIncsPointers();
 Penalty=0;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operators                                                        //
////////////////////////////////////////////////////////////////////////////////

/** \fn incidence& variable::operator[] (int i)
    \return incidence i*/
incidence& variable::operator[] (int i)
{return Incs[i];
}//-----------------------------------------------------------------------------

/** \fn variable& variable::operator=(const variable &Var)
    Overloaded assignation operator*/
variable& variable::operator=(const variable &Var)
{
 Incs.resize(Var.Incs.size());
 Ival=Var.Ival;
 Mono=Var.Mono;
 MonoDom=Var.MonoDom;
 Incs.resize(Var.GetNumIncs());
 Name=Var.Name;
 TotalMono=Var.TotalMono;
 UniIncident=Var.UniIncident;
 this->SetIncsPointers();
 Penalty=Var.Penalty;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Acces to member attributes                                                 //
////////////////////////////////////////////////////////////////////////////////

/** \fn int GetNumIncs() const
    \return Number of incidences.*/
int variable::GetNumIncs() const
{return Incs.size();
}//-----------------------------------------------------------------------------

/** \fn ivalDb GetIval() const
    \return Ival of the variable.*/
ivalDb variable::GetIval() const
{return Ival;
}//-----------------------------------------------------------------------------

/** \fn int GetMono() const
    \return */
int variable::GetMono() const
{return Mono;
}//-----------------------------------------------------------------------------

/** \fn int GetTotalMono() const
    \return */
bool variable::GetTotalMono() const
{return TotalMono;
}//-----------------------------------------------------------------------------

/** \fn ivalDb GetMonoDom() const
    \return Mono.
    (0 => Non monotonic; 1 => Monotonic increasing; -1 => Monotonic decreasing)*/
ivalDb variable::GetMonoDom() const
{return MonoDom;
}//-----------------------------------------------------------------------------

/** \fn char* GetName() const
    \return Name of the variable (syntactic use)*/
string& variable::GetName()
{return Name;
}//-----------------------------------------------------------------------------

bool variable::GetUniIncident() const
{return UniIncident;
}//-----------------------------------------------------------------------------

int variable::GetPenalty()
{return Penalty;
}//-----------------------------------------------------------------------------

/** \fn void SetDoms(ivalDb Dom)
    Set the Ival of the variable and its incidences*/
void variable::SetIncsPointers(void)
{
 for(int i=0 ;i<this->Incs.size();i++)
 {this->Incs[i].SetIvalVar(&this->Ival);
  this->Incs[i].SetMonoVar(&this->Mono);
  this->Incs[i].SetTotalMonoVar(&this->TotalMono);
  this->Incs[i].SetUniIncVar(&this->UniIncident);
 }
}//-----------------------------------------------------------------------------

/** \fn void SetDom(ivalDb Dom)
    Set the Ival of the variable*/
void variable::SetIval(ivalDb& IV)
{Ival=IV;
}//-----------------------------------------------------------------------------

/** \fn void SetMono(int Mono)
    Set monotiny
    (Mono=0 => Non monotonic; Mono=1 => Monotonic increasing; Mono=-1 => Monotonic decreasing)*/
void variable::SetMono(int Mono)
{Mono=Mono;
}//-----------------------------------------------------------------------------

/** \fn void SetTotalMono(int Mono)*/
void variable::SetTotalMono(bool Mono)
{TotalMono=Mono;
}//-----------------------------------------------------------------------------

/** \fn void SetMono(int Mono)
    Set monotiny Ival*/
void variable::SetMonoDom(ivalDb& MD)
{MonoDom=MD;
}//-----------------------------------------------------------------------------

/** \fn void SetName(char* Nm)
    Set name of the vairable*/
void variable::SetName(string& Nm)
{Name=Nm;
}//-----------------------------------------------------------------------------

void variable::SetUniIncident(bool UI)
{UniIncident=UI;
}//-----------------------------------------------------------------------------

void variable::SetPenalty(int P)
{Penalty=P;
}//-----------------------------------------------------------------------------

void variable::Penalize()
{Penalty++;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Member Functions                                                           //
////////////////////////////////////////////////////////////////////////////////

/** \fn void AssignMono(void)
    Assign Mono to the variable*/
void variable::AssignMono(void)
{double Infimum=Inf(MonoDom);
 double Supremum=Sup(MonoDom);
 if(IsNaN(Infimum) || IsNaN(Supremum)) Mono=0;
 else if(Infimum>=0) Mono=1;
 else if(Supremum<=0) Mono=-1;
 else Mono=0;
}//-----------------------------------------------------------------------------

/** \fn void AssignTotalMono(void)
    Assign Mono to the variable*/
void variable::AssignTotalMono(void)
{bool aux=true;
 int i=0;
 if(this->Mono!=0)
  {while(i< Incs.size() && aux==true)
   {if(Incs[i].IsMono()==true) aux=true;
    else aux=false;
    i++;}}
 else aux=false;
 this->TotalMono=aux;
}//-----------------------------------------------------------------------------

/** \fn bool IsMono(void)
    \return true if it is monotonic and false if it is not*/
bool variable::IsMono(void)
{if(Mono==1 || Mono==-1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsMonoIncr(void)
    \return true if it is monotonic increasing and false if it is not*/
bool variable::IsMonoIncr(void)
{if(Mono==1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsMonoDecr(void)
    \return true if it is monotonic decreasing and false if it is not*/
bool variable::IsMonoDecr(void)
{if(Mono==-1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn  bool IsMultiInci(void)
    \return true if it is multi-incident and false if it is not*/
bool variable::IsMultiInci(void)
{if(Incs.size() >1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn  bool IsUniInci(void)
    \return true if it is uni-incident and false if it is not*/
bool variable::IsUniInci(void)
{if(Incs.size()==1) return true;
 else return false;
}//-----------------------------------------------------------------------------

/** \fn bool IsTotalyMono(void)
    \return true if it is totaly monotonic and false if it is not*/
bool variable::IsTotalyMono(void)
{
 return this->TotalMono;
}//-----------------------------------------------------------------------------

/** \fn bool IsImpr(void)
    \return true if it is improper and false if it is not*/
bool variable::IsImpr(void)
{return Ival.IsImpr();
}//-----------------------------------------------------------------------------

/** \fn bool IsProp(void)
    \return true if it is proper and false if it is not*/
bool variable::IsProp(void)
{return Ival.IsProp();
}//-----------------------------------------------------------------------------

void variable::Print(int i,fstream &fout,int PrintFlag)
{
  fout << this->Name << "[" << i << "].Dom="  << this->GetIval()  << "   ";
  fout << Name << "[" << i << "].MonoDom=" << this->GetMonoDom()  << endl;
  fout << Name << "[" << i << "].Mono=" << this->GetMono()<< endl << endl;
  for(int j=0 ;j<this->GetNumIncs();j++)
  {fout << Name << "[" << i << "][" << j << "].Dom=" << this->operator[](j).GetIvalVar() << endl;
   fout << Name << "[" << i << "][" << j << "].MonoDom=" << this->operator[](j).GetMonoDom() << endl;
   fout << Name << "[" << i << "][" << j << "].Mono=" << this->operator[](j).GetMono()<< endl << endl;}
}//-----------------------------------------------------------------------------


//##############################################################################
