 /*!****************************************************************************
 *    \file variable.h
 *    This file contains the definition of the variable class belonging
 *    to the Star Algorithm.
 *    Date: 27/01/2003
 *    Author: Pau Herrero Vias.
 *    MiceLab - Universitat de Girona.
 ******************************************************************************/

#include "incidence.h"
#include <vector.h>
#include <fstream.h>

#ifndef VARIABLE_HH
#define VARIABLE_HH

/*!*****************************************************************************
*    \class variable
*    \brief Class for the Star Algorithm.
*    This class represents a variable of an interval function.
*******************************************************************************/
class variable
{
private:
        string Name;     ///< Name of the variable (for a syntactic use).
        ivalDb Ival;  ///< enum Domain of the variable.
        vector<incidence> Incs;  ///< enum Vector of incidences of the variable.
        int Mono;     ///< Mono of the variable.
        bool TotalMono; ///< Total Monotony of the variable (with incidence).
        ivalDb MonoDom;  ///< Monotony domain.
        bool UniIncident; ///< Uni incidence flag
        int Penalty;

public:

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

variable();
variable(const int &NI);
variable(const int &NI,const ivalDb &Ival);
variable(const int &NI,const ivalDb &Ival,string Name);
variable(const variable &Var);

////////////////////////////////////////////////////////////////////////////////
// Destructor                                                                 //
////////////////////////////////////////////////////////////////////////////////

~variable() {}

////////////////////////////////////////////////////////////////////////////////
// Overloaded Operator                                                        //
////////////////////////////////////////////////////////////////////////////////

incidence& operator[] (int);
variable& operator=(const variable &var);

////////////////////////////////////////////////////////////////////////////////
// Acces to members                                                           //
////////////////////////////////////////////////////////////////////////////////

int GetNumIncs() const;
ivalDb GetIval() const;
int GetMono() const;
bool GetTotalMono() const;
ivalDb GetMonoDom() const;
string& GetName();
bool GetUniIncident() const;
int GetPenalty();

void SetMono(int);
void SetTotalMono(bool);
void SetMonoDom(ivalDb&);
void SetIval(ivalDb&);
void SetName(string&);
void SetUniIncident(bool);
void SetPenalty(int);

void Penalize();

void SetIncsPointers(void);

bool IsMono(void);
bool IsMonoIncr(void);
bool IsMonoDecr(void);
bool IsMultiInci(void);
bool IsUniInci(void);
bool IsTotalyMono(void);
bool IsImpr(void);
bool IsProp(void);

void AssignMono(void);
void AssignTotalMono(void);

void Print(int i,fstream &fout,int PrintFlag);

}; // End of Class variable

#endif


