#pragma hdrstop
#include <vcl.h>

#include "ColorHLS.h"

#define min(a, b)  (((a) < (b)) ? (a) : (b))
#define max(a, b)  (((a) > (b)) ? (a) : (b))  

TColorHLS::TColorHLS()
{
  Hue = 0.0f;
  Luminance = 0.0f;
  Saturation = 0.0f;
  Red = 0;
  Green = 0;
  Blue = 0;
}

TColorHLS::TColorHLS(int r, int g, int b)
{
  Red = r;
  Green = g;
  Blue = b;
}


TColorHLS::TColorHLS(double hue, double luminance, double saturation)
{
  Hue=hue;
  Luminance=luminance;
  Saturation=saturation;
}

/*long GetTcolor(const TColorHLS & cl) {
  return Tcolor(RGB(cl.Red,cl.Green,cl.Blue));
}
*/
void TColorHLS::SetRed(int red) {
  Red=red;
}

void TColorHLS::SetGreen(int green) {
  Green=green;
}

void TColorHLS::SetBlue(int blue) {
  Blue=blue;
}

void TColorHLS::SetRGB(int red, int green, int blue) {
  Red=red;
  Green=green;
  Blue=blue;
}


void TColorHLS::SetHue(double hue) {
  Hue=hue;
}

void TColorHLS::SetLuminance(double lum) {
  Luminance=lum;
}

void TColorHLS::SetSaturation(double sat) {
  Saturation=sat;
}

double TColorHLS::GetHue() const
{
  return Hue;
}

double TColorHLS::GetLuminance() const
{
  return Luminance;
}

double TColorHLS::GetSaturation() const
{
  return Saturation;
}

int ToRGB1(double rm1, double rm2, double rh)
{
  while (rh > 360.0f)
    rh -= 360.0f;
  while (rh < 0.0f)
    rh += 360.0f;

  if (rh <  60.0f)
    rm1 = rm1 + (rm2 - rm1) * rh / 60.0f;
  else if (rh < 180.0f)
    rm1 = rm2;
  else if (rh < 240.0f)
    rm1 = rm1 + (rm2 - rm1) * (240.0f - rh) / 60.0f;

  return (int)(rm1 * 255.0f);
}



TColorHLS RGBToHLS(int r, int g, int b)
{
  int red    = r;
  int green  = g;
  int blue   = b;
  int minval = min(red, min(green, blue));
  int maxval = max(red, max(green, blue));
  double mdiff  = double(maxval) - double(minval);
  double msum   = double(maxval) + double(minval);

  TColorHLS colorHLS;
  colorHLS.SetLuminance(msum / 510.0f);

  if (maxval == minval)
  {
    colorHLS.SetSaturation(0.0f);
    colorHLS.SetHue(0.0f);
  }
  else
  {
    double rnorm = (maxval - r  ) / mdiff;
    double gnorm = (maxval - g) / mdiff;
    double bnorm = (maxval - b ) / mdiff;

    double saturation = ((colorHLS.GetLuminance() <= 0.5f) ?
      (mdiff / msum) : (mdiff / (510.0f - msum)));
    colorHLS.SetSaturation(saturation);

    double hue;
    if (r == maxval)   hue = 60.0f * (6.0f + bnorm - gnorm);
    if (g == maxval) hue = 60.0f * (2.0f + rnorm - bnorm);
    if (b == maxval)  hue = 60.0f * (4.0f + gnorm - rnorm);
    colorHLS.SetHue(hue);
  }
  return colorHLS;
}

int HLSToRGB(const TColorHLS& colorHLS)
{
//  TColorHLS colorF;
  if (colorHLS.GetSaturation() == 0.0f)
  {
/*    colorF.Red=(int)(colorHLS.GetLuminance() * 255.0f);
    colorF.Green=(int)(colorHLS.GetLuminance() * 255.0f);
    colorF.Blue=(int)(colorHLS.GetLuminance() * 255.0f);
    return colorF;
*/
    return TColor(RGB((int)(colorHLS.GetLuminance() * 255.0f),
                      (int)(colorHLS.GetLuminance() * 255.0f),
                      (int)(colorHLS.GetLuminance() * 255.0f)));

  }
  else
  {
    double rm1, rm2;

    if (colorHLS.GetLuminance() <= 0.5f)
      rm2 = colorHLS.GetLuminance() +
            colorHLS.GetLuminance() * colorHLS.GetSaturation();
    else
      rm2 = colorHLS.GetLuminance() + colorHLS.GetSaturation() -
            colorHLS.GetLuminance() * colorHLS.GetSaturation();
    rm1 = 2.0f * colorHLS.GetLuminance() - rm2;

/*    colorF.Red=ToRGB1(rm1, rm2, colorHLS.GetHue() + 120.0f);
    colorF.Green=ToRGB1(rm1, rm2, colorHLS.GetHue());
    colorF.Blue=ToRGB1(rm1, rm2, colorHLS.GetHue() - 120.0f);
*/


    return TColor(RGB(ToRGB1(rm1, rm2, colorHLS.GetHue() + 120.0f),
                      ToRGB1(rm1, rm2, colorHLS.GetHue()),
                      ToRGB1(rm1, rm2, colorHLS.GetHue() - 120.0f)));
//     return colorF;
  }
}



// *************************** TColorHLS **************************************


