/*******************************************************************************
 *    ivalDb.h: Modal Interval Library using FLIBM
 *    v2.0
 *    17/12/2002
 *    By Pau Herrero Vias
 ******************************************************************************/
/*
    *COPYRIGHT

    Copyright (c) 2000-2003, MiceLab (Modal Interval and Control
    Engineering Laboratory - University of Girona - Catalonia , Spain.
    All rights reserved.

    The authors hereby grant you non-exclusive and non-transferable
    license to use, copy and modify the Software and its documentation
    for non-commercial purposes.

    The Software and derived products of the Software shall not be
    redistributed without specific prior written permission.

    *LIMITATIONS

    IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR DIRECT,
    INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
    OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
    DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,BUT
    NOT LIMITED TO, THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
    PURPOSE, AND NON-INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS
    IS" BASIS, AND THE AUTHORS HAVE NO OBLIGATION TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

//Uncomment the following line to use with Borland C++
#define BORLAND
//Uncomment the following line to use with Microsoft Visual C++
//#define VISUAL
//Uncomment the following line to use with g++ compiler for Linux
//#define LINUX

#include <iostream.h>

//#include <fstream.h>

#ifndef IVALDB_HH
#define IVALDB_HH

extern "C"
{
	namespace  FDLIBM
	{
		#include "fdlibm.h"
	}
}

#ifdef VISUAL
        #define min(a,b) (((a) < (b))? (a):(b))
        #define max(a,b) (((a) > (b))? (a):(b))
        #include <limits>
        using namespace std;
        #define _plusinfinity numeric_limits<double>::infinity()
        #define _minusinfinity -(numeric_limits<double>::infinity())
        #undef __STDC__
#endif

#ifdef BORLAND
        #define _plusinfinity 1.0/0.0
        #define _minusinfinity -1.0/0.0
        #define _nan NaN;
#endif

#ifdef LINUX
        #define _plusinfinity 1.0/0.0
        #define _minusinfinity -1.0/0.0
        #undef __P
        #undef HUGE
#endif

class ivalDb
{
private:


public:

    double Inf;           // Infimum
    double Sup;           // Supremum

    //////////////////
    // Constructors //
    //////////////////

    ivalDb();
    ivalDb(const double &i, const double &s);
    ivalDb(const double &x);
    ivalDb(const ivalDb &x);

    ////////////////
    // Destructor //
    ////////////////

    ~ivalDb() {}

    //////////////////////
    // Acces to members //
    //////////////////////

    double GetInf()const;
    double GetSup()const;
    double GetMid()const;
    void   SetInf(double);
    void   SetSup(double);
    void   SetBounds(double,double);
    bool   IsEmpty()const;

    //new function members
    bool IsProp()const;
    bool IsImpr()const;
    bool IsInterval()const;
    bool IsZero()const;
    bool IsInfty()const;
    bool IsIn(const ivalDb &x)const;
    bool IsBig(const ivalDb &x)const;
    bool IsSmall(const ivalDb &x)const;
    bool IsBigEq(const ivalDb &x)const;
    bool IsSmallEq(const ivalDb &x)const;
    void MakeDu(void);
    void MakeProp(void);
    void MakeImpr(void);
    ivalDb AddAbsError(double Error);
    ivalDb AddRelError(double Percent);
    ////////////////////////////////////
    // Monary Operators and Functions //
    ////////////////////////////////////

    ivalDb& operator=(const ivalDb &x);
    ivalDb operator-()const;

    friend double Inf(const ivalDb &x); //sobra
    friend double Sup(const ivalDb &x); //sobra
    friend ivalDb Prop(const ivalDb &x);
    friend ivalDb Impr(const ivalDb &x);

    friend ivalDb Du(const ivalDb &x);
    friend ivalDb Du1(const ivalDb &x, bool &flag);
    friend ivalDb Du2(const ivalDb &x, bool flag);

    ////////////////////////////
    // Binary Inner Operators //
    ////////////////////////////
    friend ivalDb operator+=(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator-=(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator*=(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator/=(const ivalDb &x, const ivalDb &y);

    friend ivalDb operator+(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator-(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator*(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator/(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator^(const ivalDb &x, const int &n);
    friend ivalDb operator&&(const ivalDb &x, const ivalDb &y);
    friend ivalDb operator||(const ivalDb &x, const ivalDb &y);

    friend ivalDb abs(const ivalDb &x);
    friend ivalDb pow(const ivalDb &x,long n);
    friend ivalDb sqr(const ivalDb &x);
    friend ivalDb sqrt(const ivalDb &x);
    friend ivalDb cbrt(const ivalDb &x);
    friend ivalDb root(const ivalDb &x,const int &n);
    friend ivalDb cos(const ivalDb &x);
    friend ivalDb sin(const ivalDb &x);
    friend ivalDb tan(const ivalDb &x);
    friend ivalDb acos(const ivalDb &x);
    friend ivalDb asin(const ivalDb &x);
    friend ivalDb atan(const ivalDb &x);
    friend ivalDb atan2(const ivalDb &y, const ivalDb &x);
    friend ivalDb mod(const ivalDb &y, const ivalDb &x);
    friend ivalDb cosh(const ivalDb &x);
    friend ivalDb sinh(const ivalDb &x);
    friend ivalDb tanh(const ivalDb &x);
    friend ivalDb acosh(const ivalDb &x);
    friend ivalDb asinh(const ivalDb &x);
    friend ivalDb atanh(const ivalDb &x);
    friend ivalDb exp(const ivalDb &x);
    friend ivalDb log(const ivalDb &x);
    friend ivalDb log10(const ivalDb &x);

    //////////////////////////
    // Relational Operators //
    //////////////////////////

    friend bool operator<=(const ivalDb &x, const ivalDb &y);
    friend bool operator>=(const ivalDb &x, const ivalDb &y);
    friend bool operator<(const ivalDb &x, const ivalDb &y);
    friend bool operator>(const ivalDb &x, const ivalDb &y);

    friend bool operator<<(const ivalDb &x, const ivalDb &y);
    friend bool operator>>(const ivalDb &x, const ivalDb &y);
    friend bool operator<<=(const ivalDb &x, const ivalDb &y);
    friend bool operator>>=(const ivalDb &x, const ivalDb &y);
    friend bool operator==(const ivalDb &x, const ivalDb &y);

    //////////////////////
    // Auxiliar functions
    //////////////////////

    friend ivalDb RoundLB(ivalDb &x);
    friend ivalDb RoundHB(ivalDb &x);
    friend ivalDb Round(ivalDb &x);
    friend ivalDb Ival(double a,double b);

    friend double Width(ivalDb &x);
    friend double Center(ivalDb &x);

    friend ivalDb PI(void);
    friend ivalDb PI2(void);
    friend ivalDb PI05(void);
    friend ivalDb PI025(void);
    friend ivalDb LN2(void);
    friend ivalDb Zero(void);
    friend ivalDb Infinity(void);
    friend ivalDb DuInfinity(void);
    friend bool IsIn(ivalDb &x,ivalDb &y);
    friend bool IsIntersect(ivalDb &x,ivalDb &y);
    friend bool IsEmpty(ivalDb &x,ivalDb &y);

//    friend double   Distance(const ivalDb &x,const ivalDb &y);

    //////////////////////////
    // I/O Stream Operators //
    //////////////////////////

    friend iostream& operator>>(iostream& os, ivalDb& x);
    friend ostream& operator<<(ostream& out, const ivalDb& x);

};

    ////////////////////////
    // Not Member Functions
    ////////////////////////

     bool   IsNaN(double x);
     double NaN(void);
     double PInfinity(void);
     double MInfinity(void);
     double Integer(double x);
     bool   IsPInfinity(const double x);
     bool   IsMInfinity(const double x);
     bool   IsInfinity(const double x);
     double AddULP(double x);
     double RestULP(double x);
     double AddULPs(double x,int n);
     double RestULPs(double x,int n);
     void secureFpu();

#endif
