/*******************************************************************************
 *    ivalDb_vc.cpp: Modal Interval Library using FLIBM
 *    v2.0
 *    17/12/2002
 *    By Pau Herrero Vias
 ******************************************************************************/
/*
    *COPYRIGHT

    Copyright (c) 2000-2003, MiceLab (Modal Interval and Control
    Engineering Laboratory - University of Girona - Catalonia , Spain.
    All rights reserved.

    The authors hereby grant you non-exclusive and non-transferable
    license to use, copy and modify the Software and its documentation
    for non-commercial purposes.

    The Software and derived products of the Software shall not be
    redistributed without specific prior written permission.

    *LIMITATIONS

    IN NO EVENT SHALL THE AUTHORS BE LIABLE TO ANY PARTY FOR DIRECT,
    INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
    OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
    DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
    POSSIBILITY OF SUCH DAMAGE.

    THE AUTHORS SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,BUT
    NOT LIMITED TO, THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR
    PURPOSE, AND NON-INFRINGEMENT. THIS SOFTWARE IS PROVIDED ON AN "AS
    IS" BASIS, AND THE AUTHORS HAVE NO OBLIGATION TO PROVIDE
    MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
*/

#include <float.h>                            
#include "ivalDb.h"

/*
extern "C"
{
	namespace  FDLIBM
	{
		#include "fdlibm.h"
	}
}
*/

#ifdef LINUX
double min(double a, double b) {
	if (a>b) return b; else return a;
}
double max(double a, double b) {
	if (a>b) return a; else return b;
}
#endif

const double N_PI=3.1415926535897932384626433832795;
const double N_PI_L=3.141592653589793238462643383279;
const double N_PI_U=3.141592653589793238462643383280;
const double N_PI_05=1.5707963267948966192313216916398;
const double N_PI_025=0.78539816339744830961566084581988;
const double N_LN2=0.69314718055994530941723212145818;

////////////////////////////////////////////////////////////////////////////////
// Constructors                                                               //
////////////////////////////////////////////////////////////////////////////////

ivalDb::ivalDb()
{Inf=MInfinity();
 Sup=PInfinity();
}//-----------------------------------------------------------------------------

ivalDb::ivalDb(const double &i, const double &s)
{
  if (!IsInfinity(i) && !IsNaN(i) && i!=0) Inf=RestULP(i);
        else Inf=i;
  if (!IsInfinity(s) && !IsNaN(s) && s!=0) Sup=AddULP(s);
        else Sup=s;
}//-----------------------------------------------------------------------------

ivalDb::ivalDb(const double &x)
{
 if(!IsInfinity(x) && !IsNaN(x) && x!=0)
 {Inf=RestULP(x);
  Sup=AddULP(x);}
 else
 {Inf=x;
  Sup=x;}
}//-----------------------------------------------------------------------------

ivalDb::ivalDb(const ivalDb &x)
{
 Inf=x.Inf;
 Sup=x.Sup;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Access to members                                                           //
////////////////////////////////////////////////////////////////////////////////

// Get Infimum Bound
double ivalDb::GetInf()const
{return Inf;
}//-----------------------------------------------------------------------------

// Get Supremum Bound
double ivalDb::GetSup()const
{return Sup;
}//-----------------------------------------------------------------------------

// Get Middle Point
double ivalDb::GetMid()const
{return (Sup+Inf)/2;
}//-----------------------------------------------------------------------------

// Set Infimum Bound: y.SetInf(x)
void ivalDb::SetInf(double x)
{Inf=x;}
//------------------------------------------------------------------------------

// Set Supremum Bound: y.SetSup(x)
void ivalDb::SetSup (double x)
{Sup=x;}
//------------------------------------------------------------------------------

// Set Bounds: y.SetBounds(x)
void ivalDb::SetBounds (double i,double s)
{Inf=i; Sup=s;}
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Operators                                                                  //
////////////////////////////////////////////////////////////////////////////////

ivalDb& ivalDb::operator=(const ivalDb &x)
{Inf = x.Inf;
 Sup = x.Sup;
 return *this;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Monary Operators and Functions                                             //
////////////////////////////////////////////////////////////////////////////////

ivalDb ivalDb::operator-()const
{ivalDb res;
 res.SetBounds(-Sup, -Inf);
 return res;
}//-----------------------------------------------------------------------------

// Is Empty Interval Test: y.IsEmpty()
bool ivalDb::IsEmpty()const
{if (IsNaN(Inf) || IsNaN(Sup)) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsProp()const
{if ( Inf <= Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsImpr()const
{if ( Inf >= Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

// IsInterval Test: IsInterval(x)
bool ivalDb::IsInterval()const
{if ( Inf!=Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

// IsZero Test: IsZero(x)
bool ivalDb::IsZero()const
{if(Inf==0 && Sup==0) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsIn(const ivalDb &x)const
{if (Inf>x.Inf && Sup<x.Sup) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsBig(const ivalDb &x)const
{if (Inf>x.Sup) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsSmall(const ivalDb &x)const
{if (Sup<x.Inf) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsBigEq(const ivalDb &x)const
{if (Inf>=x.Sup) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsSmallEq(const ivalDb &x)const
{if (Sup<=x.Inf) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool ivalDb::IsInfty(void) const
{if (IsMInfinity(this->Inf) || IsPInfinity(this->Sup)) return true;
 else return false;
}//-----------------------------------------------------------------------------

// Dual
void ivalDb::MakeDu(void)
{double ASup;
 ASup=Sup;
 Sup=Inf;
 Inf=ASup;
}//-----------------------------------------------------------------------------

// Make proper
void ivalDb::MakeProp(void)
{if (Sup<Inf) this->MakeDu();
}//-----------------------------------------------------------------------------

// Make Impr
void ivalDb::MakeImpr(void)
{if (Inf<Sup) this->MakeDu();
}//-----------------------------------------------------------------------------

// Add Absolute Error
ivalDb ivalDb::AddAbsError(double Error)
{if(this->IsProp())
 {
  this->Sup=this->Sup+Error;
  this->Inf=this->Inf-Error;
 }
 else
 {
  this->Sup=this->Sup-Error;
  this->Inf=this->Inf+Error;
 }
 return *this;
}//-----------------------------------------------------------------------------

// Add Relative Error
ivalDb ivalDb::AddRelError(double Percent)
{if(this->IsProp())
 {
  this->Sup=this->Sup + Percent*Sup;
  this->Inf=this->Inf - Percent*Sup;
 }
 else
 {
  this->Sup=this->Sup - Percent*Sup;
  this->Inf=this->Inf + Percent*Sup;
 }
 return *this;
}//-----------------------------------------------------------------------------

// Imfimum
double Inf(const ivalDb &x)
{return x.Inf;
}//-----------------------------------------------------------------------------

// Suprem
double Sup(const ivalDb &x)
{return x.Sup;
}//-----------------------------------------------------------------------------

// A third
double AQuarter(ivalDb &x)
{ivalDb a;
 double Quarter=Width(x)/4;
 return x.Inf+Quarter;
}//-----------------------------------------------------------------------------

// Four Third
double ThreeQuarters(ivalDb &x)
{double Quarter=Width(x)/4;
 return x.Sup-Quarter;
}//-----------------------------------------------------------------------------

// Dual
ivalDb Du(const ivalDb &x)
{return Ival(x.Sup, x.Inf);
}//-----------------------------------------------------------------------------

// Dual 1
ivalDb Du1(const ivalDb &x, bool &flag)
{if(x.IsProp()) {flag=false; return x;}
 else {flag=true; return Du(x);}
}//-----------------------------------------------------------------------------

// Dual 2
ivalDb Du2(const ivalDb &x, bool flag)
{if(flag==true) return Du(x);
 else return x;
}//-----------------------------------------------------------------------------

// Proper
ivalDb Prop(const ivalDb &x)
{if ( x.Inf <= x.Sup ) return x;
 else return Du(x);
}//-----------------------------------------------------------------------------

// Improper
ivalDb Impr(const ivalDb &x)
{if ( x.Inf <= x.Sup ) return Du(x);
 else return x;
}//-----------------------------------------------------------------------------

// IsProp
bool IsProp(const ivalDb &x)
{if ( x.Inf <= x.Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

// IsImpr
bool IsImpr(const ivalDb &x)
{if ( x.Inf >= x.Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

// IsInterval Test: IsInterval(x)
bool IsInterval(const ivalDb &x)
{if ( x.Inf!=x.Sup ) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsEmpty(const ivalDb &x)
{if ( IsNaN(x.Inf) || IsNaN(x.Sup)) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsIn(ivalDb &x,ivalDb &y)
{if (x.Inf>y.Inf && x.Sup<y.Sup) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsEmpty(ivalDb &x,ivalDb &y)
{if (x.Inf>y.Sup || x.Sup<y.Inf) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsIntersect(ivalDb &x,ivalDb &y)
{if(x.Inf>y.Sup || x.Sup<y.Inf)return false;
 else return true;
}//-----------------------------------------------------------------------------

bool IsBig(const ivalDb &x,const ivalDb &y)
{if (x.Inf>y.Sup) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsSmall(const ivalDb &x,const ivalDb &y)
{if (x.Sup<y.Inf) return true;
 else return false;
}//-----------------------------------------------------------------------------

bool IsIn(const double &d,const ivalDb &x)
{if (d > x.GetInf() && d <x.GetSup()) return true;
 else return false;
}//-----------------------------------------------------------------------------

/*##############################################################################
  Binary Operators
/#############################################################################*/

//Linear operators
//linear sum
ivalDb operator+=(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb,hb;
 lb=x.Inf+y.Inf;
 try {
         hb=x.Sup+y.Sup;
 } catch ( ... ) {
//        cout<<"Error found\n";
 }
 res.SetBounds(lb,hb);
 return Round(res);
}// ----------------------------------------------------------------------------

//linear substract
ivalDb operator-=(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb,hb;
 lb=x.Inf-y.Inf;
 try {
         hb=x.Sup-y.Sup;
 } catch ( ... ) {
//        cout<<"Error found\n";
 }
 res.SetBounds(lb,hb);
 return Round(res);
}// ----------------------------------------------------------------------------

//linear product
ivalDb operator*=(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb,hb;
 lb=x.Inf*y.Inf;
 try {
         hb=x.Sup*y.Sup;
 } catch ( ... ) {
//        cout<<"Error found\n";
 }
 res.SetBounds(lb,hb);
 return Round(res);
}// ----------------------------------------------------------------------------

//linear sum
ivalDb operator/=(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb,hb;
 lb=x.Inf/y.Inf;
 try {
         hb=x.Sup/y.Sup;
 } catch ( ... ) {
//        cout<<"Error found\n";
 }
 res.SetBounds(lb,hb);
 return Round(res);
}// ----------------------------------------------------------------------------

// Sum
ivalDb operator+(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb,hb;

 lb=x.Inf+y.Inf;
 try {
         hb=x.Sup+y.Sup;
 } catch ( ... ) {
//        cout<<"Error found\n";
 }
 res.SetBounds(lb,hb);

 return Round(res);
}// ----------------------------------------------------------------------------

// Rest
ivalDb operator-(const ivalDb &x, const ivalDb &y)
{ivalDb res;
 double lb=x.Inf-y.Sup;
 double hb=x.Sup-y.Inf;
 res.SetBounds(lb,hb);
 return Round(res);
}// ----------------------------------------------------------------------------


ivalDb operator*(const ivalDb &x, const ivalDb &y)
{double inf, sup;

//// Exceptions
    if ((((x.Inf<=0 && x.Sup>=0)||(x.Inf>=0 && x.Sup<=0)) && ( IsInfinity(y.Inf) || IsInfinity(y.Sup))) ||
       (((y.Inf<=0 && y.Sup>=0)||(y.Inf>=0 && y.Sup<=0)) && ( IsInfinity(x.Inf) || IsInfinity(x.Sup))))
       {
         return Ival(NaN(),NaN());
       }
///////////////

    if(x.Sup>=0)
    {
        if(x.Inf>=0)
        {
            if(y.Sup>=0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf*y.Inf;
                    sup = x.Sup*y.Sup;
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf< 0.
                    inf = x.Sup*y.Inf;
                    sup = x.Sup*y.Sup;
                }
            }
            else
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf>=0.
                    inf = x.Inf*y.Inf;
                    sup = x.Inf*y.Sup;
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup*y.Inf;
                    sup = x.Inf*y.Sup;
                }
            }
        }

        else
        {
            if(y.Sup>=0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf*y.Sup;
                    sup = x.Sup*y.Sup;
                }
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    inf=min(x.Sup*y.Inf,x.Inf*y.Sup);
                    sup=max(x.Sup*y.Sup,x.Inf*y.Inf);
                }
            }
            else
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf>=0.
                    if(IsNaN(x.Inf) || IsNaN(x.Sup) || IsNaN(y.Inf) || IsNaN(y.Sup))
                    {//cout << "\n" << "Not defined" << "\n";
                     inf = NaN();
                     sup = NaN();
                    }
                    else
                    {
                     inf = double(0.);
                     sup = double(0.);
                    }
                }
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup*y.Inf;
                    sup = x.Inf*y.Inf;
                }
            }
        }
    }
    else
    {
        if(x.Inf>=0)
        {
            if(y.Sup>=0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf*y.Inf;
                    sup = x.Sup*y.Inf;
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf< 0.

                    if(IsNaN(x.Inf) || IsNaN(x.Sup) || IsNaN(y.Inf) || IsNaN(y.Sup) )
                    {
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                    }
                    else
                    {
                        inf = double(0.);      // zero
                        sup = double(0.);      // zero
                    }
                }
            }
            else
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf>=0.
                    inf=max(x.Inf*y.Inf,x.Sup*y.Sup);
                    sup=min(x.Sup*y.Inf,x.Inf*y.Sup);
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup*y.Sup;
                    sup = x.Inf*y.Sup;
                }
            }
        }
        else
        {
            if(y.Sup>=0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf*y.Sup;
                    sup = x.Sup*y.Inf;
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    inf = x.Inf*y.Sup;
                    sup = x.Inf*y.Inf;
                }
            }
            else
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf>=0.
                    inf = x.Sup*y.Sup;
                    sup = x.Sup*y.Inf;
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup*y.Sup;
                    sup = x.Inf*y.Inf;
                }
            }
        }
    }
    ivalDb res;
    res.SetBounds(inf,sup);
    return Round(res);
}// ----------------------------------------------------------------------------

/*
ivalDb operator/(const ivalDb &x, const ivalDb &y1)
{
    double  inf, sup;
	ivalDb y;
	y = y1;
        ivalDb Zero=Ival(0,0);
//// Exceptions
    if ((IsInfinity(y.Inf) || IsInfinity(y.Sup))
    &&  (IsInfinity(x.Inf) || IsInfinity(x.Sup))) {
         return Ival(NaN(),NaN());
         //return (MInfinity(),PInfinity());
    }

    if(IsIn(Zero,y) || IsIn(y,Zero))
     {return Infinity();}

    if (y.Inf==0 && y.Sup==0) return (MInfinity(),PInfinity());

// Denominator with 0 in a bound
// set tendency
    if (y.Inf==0) {
        if (y.Sup<0) y.SetInf(-0.0);
    }
    if (y.Sup==0) {
        if (y.Inf<0) y.SetSup(-0.0);
    }

///////////////


    if(x.Sup>=0)
    {
        if(x.Inf>=0)
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Sup;
                    sup = x.Sup/y.Inf;
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Sup;
                    sup = x.Inf/y.Inf;
                }
            }
        }
        else
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Inf;
                    sup = x.Sup/y.Inf;
                }
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();}
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Sup;
                    sup = x.Inf/y.Sup;
                }
            }
        }
    }
    else
    {
        if(x.Inf>=0)
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Sup;
                    sup = x.Sup/y.Sup;
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = x.Sup/y.Inf;
                    sup = x.Inf/y.Inf;
                }
            }
        }
        else
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Inf;
                    sup = x.Sup/y.Sup;
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf>=0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Inf;
                    sup = x.Inf/y.Sup;
                }
            }
        }
    }
   ivalDb res;
   res.SetBounds(inf,sup);
   return Round(res);
}
// ----------------------------------------------------------------------------
*/

ivalDb operator/(const ivalDb &x, const ivalDb &y1)
{
    double  inf, sup;
	ivalDb y;
	y = y1;
        ivalDb Zero=Ival(0,0);
//// Exceptions
    if ((IsInfinity(y.Inf) || IsInfinity(y.Sup))
    &&  (IsInfinity(x.Inf) || IsInfinity(x.Sup))) {
         return Ival(NaN(),NaN());
         //return (MInfinity(),PInfinity());
    }

//    if(IsIn(Zero,y) || IsIn(y,Zero))
//     {return Infinity();}

    if (y.Inf==0 && y.Sup==0) return (MInfinity(),PInfinity());

// Denominator with 0 in a bound
// set tendency
    if (y.Inf==0) {
        if (y.Sup<0) y.SetInf(-0.0);
    }
    if (y.Sup==0) {
        if (y.Inf<0) y.SetSup(-0.0);
    }

///////////////


    if(x.Sup>=0)
    {
        if(x.Inf>=0)
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Sup;
                    sup = x.Sup/y.Inf;
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup>=0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Sup;
                    sup = x.Inf/y.Inf;
                }
            }
        }
        else
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Inf;
                    sup = x.Sup/y.Inf;
                }
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();}
                else
                {
                    // x.Sup>=0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Sup;
                    sup = x.Inf/y.Sup;
                }
            }
        }
    }
    else
    {
        if(x.Inf>=0)
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Sup;
                    sup = x.Sup/y.Sup;
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf>0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup< 0, x.Inf>=0, y.Sup< 0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = x.Sup/y.Inf;
                    sup = x.Inf/y.Inf;
                }
            }
        }
        else
        {
            if(y.Sup>0)
            {
                if(y.Inf>=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf>=0.
                    inf = x.Inf/y.Inf;
                    sup = x.Sup/y.Sup;
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup>=0, y.Inf< 0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
            }
            else
            {
                if(y.Inf>0 && y.Sup!=0)
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf>=0.
                    //cout << "\n" << "Not defined" << "\n";
                    inf = NaN();
                    sup = NaN();
                }
                else
                {
                    // x.Sup< 0, x.Inf< 0, y.Sup< 0, y.Inf< 0.
                    inf = x.Sup/y.Inf;
                    sup = x.Inf/y.Sup;
                }
            }
        }
    }
   ivalDb res;
   res.SetBounds(inf,sup);
   return Round(res);
}
// ----------------------------------------------------------------------------

// ----------------------------------------------------------------------------
// ^ Operator
// ----------------------------------------------------------------------------
ivalDb operator^(const ivalDb &x, const int &n)
{
return pow(x,n);
}// ----------------------------------------------------------------------------

// Meet
ivalDb operator&&(const ivalDb &x, const ivalDb &y)
{    double Max,Min;
     //find minimum
     if (IsNaN(x.Sup))
          Min=y.Sup;
     else
          if (IsNaN(y.Sup))
               Min=x.Sup;
          else
               Min=min(x.Sup, y.Sup);
     //find maximum
     if (IsNaN(x.Inf))
          Max=y.Inf;
     else
          if (IsNaN(y.Inf))
               Max=x.Inf;
          else
               Max=max(x.Inf, y.Inf);

     return Ival(Max, Min);
}// ----------------------------------------------------------------------------

// Join
ivalDb operator||(const ivalDb &x, const ivalDb &y)
{    double Max,Min;
     //find minimum
     if (IsNaN(x.Inf))
          Min=y.Inf;
     else
          if (IsNaN(y.Inf))
               Min=x.Inf;
          else
               Min=min(x.Inf, y.Inf);
     //find maximum
     if (IsNaN(x.Sup))
          Max=y.Sup;
     else
          if (IsNaN(y.Sup))
               Max=x.Sup;
          else
               Max=max(x.Sup, y.Sup);

     return Ival(Min,Max);
}// ----------------------------------------------------------------------------

/*##############################################################################
  Functions
/#############################################################################*/

// Absolute Value: Abs(x)
ivalDb abs(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::fabs(x1.Inf);
 double hb=FDLIBM::fabs(x1.Sup);
 if (x1.Inf >= 0 && x1.Sup >= 0) {res.SetBounds(lb,hb);}
 else if(x1.Inf < 0 && x1.Sup >= 0){res.SetBounds(0,max(lb,hb));}
 else{res.SetBounds(hb,lb);}
 return Du2(res,flag);
}//-----------------------------------------------------------------------------

// Power: Pow(x,n)
ivalDb pow(const ivalDb &x, long n)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::pow(x1.Inf,n);
 double hb=FDLIBM::pow(x1.Sup,n);
 if (n%2 == 1){res.SetBounds(lb,hb);}
 else
 {if (x1.Inf >= 0 && x1.Sup >= 0) {res.SetBounds(lb,hb);}
  else if(x1.Inf < 0 && x1.Sup >= 0){res.SetBounds(0,max(lb,hb));}
  else{res.SetBounds(hb,lb);}}
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Square: Sqr(x)
ivalDb sqr(const ivalDb &x)
{return pow(x,2);}
//-----------------------------------------------------------------------------

// Square Root: Sqrt(x)
ivalDb sqrt(const ivalDb &x)
{
 ivalDb res;
 double lb,hb;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 if(x1.Inf<0) lb=NaN();
 else lb=FDLIBM::sqrt(x1.Inf);
 if(x1.Sup<0) hb=NaN();
 else hb=FDLIBM::sqrt(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

 // Cubic Root: Cbrt(x)
ivalDb cbrt(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::cbrt(x1.Inf);
 double hb=FDLIBM::cbrt(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Generic Root: Root(x)
ivalDb root(const ivalDb &x,const int &n)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);

 if(n==2){res=sqrt(x1);}                // (n==2)

 else if(n==3) return cbrt(x1);         // (n==3)

 else if(n%2==0)                       // (n%2==0)
 {ivalDb aux;
  aux=log(x1)/n;
  aux=exp(aux);
  res.SetBounds(aux.Inf,aux.Sup);}

  else                                  // (n%2!=0)
  {ivalDb aux,aux1;
   double alb,ahb;
   alb=min(FDLIBM::fabs(x1.Inf),FDLIBM::fabs(x1.Sup));
   ahb=max(FDLIBM::fabs(x1.Inf),FDLIBM::fabs(x1.Sup));
   aux1.SetBounds(alb,ahb);
   aux=log(aux1)/n;
   aux=exp(aux);
   if(x1.Inf<=0 && x1.Sup<=0) res.SetBounds(-aux.Sup,-aux.Inf);
   else if(x1.Inf<0 && x1.Sup>0)
        {if(x1.Sup>=FDLIBM::fabs(x1.Inf)) res.SetBounds(-aux.Inf,aux.Sup);
        else res.SetBounds(-aux.Sup,aux.Inf);}
   else res.SetBounds(aux.Inf,aux.Sup);}
  return Du2(res,flag);
}//-----------------------------------------------------------------------------

// Cosine: Cos(x)
ivalDb cos(const ivalDb &x)
{
  ivalDb res;
  double klb,khb;
  bool flag=false;
  //X1 gets the proper value of x
  ivalDb x1=Du1(x,flag);

  double cosi = FDLIBM::cos(x1.Inf);
  double coss = FDLIBM::cos(x1.Sup);

  //Determinar el rango de la funcin
  if(x1.Inf>0) klb=Integer((x1/PI()).Inf);
  else klb=Integer((x1/PI()).Inf)-1;
  if(x1.Sup>0) khb=Integer((x1/PI()).Sup);
  else khb=Integer((x1/PI()).Sup)-1;

  if((khb-klb)>=2) { //el rango es mayor a 2*pi
        res.SetBounds(-1,1);
        return Du2(res,flag);
  } else if((khb-klb)>=1) { //el rango est entre 0 y pi
        if(int(khb)%2==0) {
                res.SetBounds(min(cosi,coss),1);
                return Du2(RoundLB(res),flag);
        } else {
                res.SetBounds(-1,max(cosi,coss));
                return Du2(RoundHB(res),flag);
        }
  } else {
        res.SetBounds(min(cosi,coss),max(cosi,coss));
        return Du2(Round(res),flag);
  }
}//-----------------------------------------------------------------------------

// Sine: Sin(x)
ivalDb sin(const ivalDb &x)
{ivalDb aux;
 aux=x-PI05();
 return cos(aux);
}//-----------------------------------------------------------------------------

// Tangent: Tan(x)
ivalDb tan(const ivalDb &x)
{ivalDb res;
 int klb,khb;
 bool flag=false;
 ivalDb x1=Du1(x,flag);

 double tani = FDLIBM::tan(x1.Inf);
 double tans = FDLIBM::tan(x1.Sup);

 if(x1.Inf>=0) klb=int((x1/PI05()).Inf);
 else klb=int((x1/PI05()).Inf)-1;
 if(x1.Sup>=0) khb=int((x1/PI05()).Sup);
 else khb=int((x1/PI05()).Sup)-1;

 if((khb-klb)>=2) res.SetBounds(MInfinity(),PInfinity());
 else if((khb-klb)==0)
          res.SetBounds(tani,tans);
      else if((khb-klb)>=1) {
          if(klb%2!=0 && khb%2==0 )
                res.SetBounds(min(tani,tans),max(tani,tans));
          else
                res.SetBounds(MInfinity(),PInfinity());
      }
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Acosine: Acos(x)
ivalDb acos(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::acos(x1.Inf);
 double hb=FDLIBM::acos(x1.Sup);
 res.SetBounds(min(lb,hb),max(lb,hb));
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Asine: Asin(x)
ivalDb asin(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::asin(x1.Inf);
 double hb=FDLIBM::asin(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Atangent: Atan(x)
ivalDb atan(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::atan(x1.Inf);
 double hb=FDLIBM::atan(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Atangent2: Atan2(y,x)
ivalDb atan2(const ivalDb &y, const ivalDb &x)
{ivalDb res,tem;
 double r1,r2;
 if ((x.Inf<0 && x.Sup>0)||(x.Inf>0 && x.Sup<0)) {
     if (y.Inf>0 && y.Sup>0) {
          tem.SetBounds(RestULP(N_PI_05),AddULP(N_PI_05));
          res=tem-atan2(x,y);
     } else
     if (y.Inf<0 && y.Sup<0) {
          tem.SetBounds(RestULP(5*N_PI/2),AddULP(5*N_PI/2));
          res=tem-atan2(x,y);
     } else {
          //extra case
          if (IsImpr(x) && IsImpr(y)) {
               res.SetBounds(RestULP(2*N_PI),0);
          } else {
               res.SetBounds(0,AddULP(2*N_PI));
          }
     }
 } else {
       if (y.Inf>0) {
          if (y.Sup>0) {
               if (x.Inf>0 && x.Sup>0) {
                    r1=FDLIBM::atan2(y.Inf,x.Sup);
                    r2=FDLIBM::atan2(y.Sup,x.Inf);
               } else {
                    r1=FDLIBM::atan2(y.Sup,x.Sup);
                    r2=FDLIBM::atan2(y.Inf,x.Inf);
               }
          } else { //y.Sup<0
               if (x.Inf>0 && x.Sup>0) {
                    r1=FDLIBM::atan2(y.Inf,x.Sup);
                    r2=FDLIBM::atan2(y.Sup,x.Sup);
               } else {
                    r1=FDLIBM::atan2(y.Sup,x.Inf);
                    r2=FDLIBM::atan2(y.Inf,x.Inf);
               }
          }
       } else { //y.Inf<0
          if (y.Sup>0) {
               if (x.Inf>0 && x.Sup>0) {
                    r1=FDLIBM::atan2(y.Inf,x.Inf);
                    r2=FDLIBM::atan2(y.Sup,x.Inf);
               } else {
                    r1=FDLIBM::atan2(y.Sup,x.Sup);
                    r2=FDLIBM::atan2(y.Inf,x.Sup);
               }
          } else { //y.Sup<0
               if (x.Inf>0 && x.Sup>0) {
                    r1=FDLIBM::atan2(y.Inf,x.Inf);
                    r2=FDLIBM::atan2(y.Sup,x.Sup);
               } else {
                    r1=FDLIBM::atan2(y.Sup,x.Inf);
                    r2=FDLIBM::atan2(y.Inf,x.Sup);
               }
          }
       }
       r1=RestULP(r1);
       r2=AddULP(r2);
       res.SetBounds(r1,r2);
 }
 return res;
}//-----------------------------------------------------------------------------

// Atangent2: Atan2(y,x)
ivalDb mod(const ivalDb &y, const ivalDb &x)
{ivalDb res;
     res=sqrt(pow(x,2)+pow(y,2));
     return res;
}

// Hyperbolic Cosine: Cosh(x)
ivalDb cosh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::cosh(x1.Inf);
 double hb=FDLIBM::cosh(x1.Sup);
 double alb=FDLIBM::cosh(FDLIBM::fabs(x1.Inf));
 double ahb=FDLIBM::cosh(FDLIBM::fabs(x1.Sup));
 if (x1.Inf >= 0)
  {if (x1.Sup>=0) res.SetBounds(lb,hb);
   else {res.SetBounds(max(lb,hb),1);}
  }
  else
   {if (x1.Sup >= 0){res.SetBounds(1,max(alb,ahb));}
    else res.SetBounds(hb,lb);}
  return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Hyperbolic Sine: Sinh(x)
ivalDb sinh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::sinh(x1.Inf);
 double hb=FDLIBM::sinh(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Hyperbolic Tangent: Tanh(x)
ivalDb tanh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::tanh(x1.Inf);
 double hb=FDLIBM::tanh(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Hyperbolic Acos: Acosh(x)
ivalDb acosh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::acosh(x1.Inf);
 double hb=FDLIBM::acosh(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Hyperbolic Asin: Asinh(x)
ivalDb asinh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::asinh(x1.Inf);
 double hb=FDLIBM::asinh(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Hyperbolic Atan: Atanh(x)
ivalDb atanh(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::atanh(x1.Inf);
 double hb=FDLIBM::atanh(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Exponential: Exp(x)
ivalDb exp(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 ivalDb x1=Du1(x,flag);
 double lb=FDLIBM::exp(x1.Inf);
 double hb=FDLIBM::exp(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Logarithm: Log(x)
ivalDb log(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 double lb,hb;
 ivalDb x1=Du1(x,flag);
 //if(IsMInfinity(x1.Inf) || x1.Inf<=0) lb=NaN();
 if(x1.Inf<=0) lb=NaN();
 else lb=FDLIBM::log(x1.Inf);
 //if(IsPInfinity(x1.Sup) || x1.Sup<=0) hb=PInfinity();
 if(IsPInfinity(x1.Sup)) hb=PInfinity();
 else if(x1.Sup<0) hb=NaN();
 else if(x1.Sup==0) hb=MInfinity();
 else hb=FDLIBM::log(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

// Logarithm 10: Log10(x)
ivalDb log10(const ivalDb &x)
{ivalDb res;
 bool flag=false;
 double lb,hb;
 ivalDb x1=Du1(x,flag);
 if(IsMInfinity(x1.Inf) || x1.Inf<0) lb=NaN();
 else lb=FDLIBM::log10(x1.Inf);
 if(IsPInfinity(x1.Sup) || x1.Sup<0) hb=PInfinity();
 else hb=FDLIBM::log10(x1.Sup);
 res.SetBounds(lb,hb);
 return Du2(Round(res),flag);
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Relational Operators                                                       //
////////////////////////////////////////////////////////////////////////////////

bool operator<=(const ivalDb &x, const ivalDb &y)
{return ((x.Inf<=y.Inf) && (x.Sup<=y.Sup));}
//------------------------------------------------------------------------------

bool operator>=(const ivalDb &x, const ivalDb &y)
{return ((x.Inf>=y.Inf) && (x.Sup>=y.Sup));}
//------------------------------------------------------------------------------

bool operator<(const ivalDb &x, const ivalDb &y)
{return x.Sup < y.Inf;}
//------------------------------------------------------------------------------

bool operator>(const ivalDb &x, const ivalDb &y)
{return x.Inf > y.Sup;}
//------------------------------------------------------------------------------

/*
bool operator<(const ivalDb &x, const ivalDb &y)
{return min(x.Inf,x.Sup) < min(y.Inf,y.Sup);}
//------------------------------------------------------------------------------

bool operator>(const ivalDb &x, const ivalDb &y)
{return max(x.Inf,x.Sup) > max(y.Inf,y.Sup);}
//------------------------------------------------------------------------------
*/

bool operator<<(const ivalDb &x, const ivalDb &y)
{return ((x.Inf>=y.Inf) && (x.Sup<=y.Sup));
}//-----------------------------------------------------------------------------

bool operator>>(const ivalDb &x, const ivalDb &y)
{return ((x.Inf<=y.Inf) && (x.Sup>=y.Sup));}
//------------------------------------------------------------------------------

bool operator==(const ivalDb &x, const ivalDb &y)
{return ((x.Inf==y.Inf) && (x.Sup==y.Sup));}
//------------------------------------------------------------------------------

bool operator<<=(const ivalDb &x, const ivalDb &y)  // Intersection
{ivalDb meet=x && y;
 return meet.IsProp();
}//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Auxiliar functions
////////////////////////////////////////////////////////////////////////////////

// Round Low Bound: RoundLB(x)
ivalDb RoundLB(ivalDb &x) {

/* if(IsNaN(x.Inf) || IsNaN(x.Sup)) return Ival(NaN(),NaN());
 else
 {ivalDb res;
 double lb,hb;
 if(x.Inf==0)lb=0; else lb=RestULP(x.Inf);
 hb=x.Sup;
 res.SetBounds(lb,hb);
 return res;}
  */

 ivalDb res;
 double lb,hb;
 lb=x.Inf;
 hb=x.Sup;
 if (x.Inf==0) return ivalDb(lb,hb);
 if (!IsInfinity(x.Inf) && !IsNaN(x.Inf)) {
      lb=RestULP(x.Inf);
 }
 res.SetBounds(lb,hb);
 return res;
}//-----------------------------------------------------------------------------

// Round High Bound: RoundHB(x)
ivalDb RoundHB(ivalDb &x) { 
 ivalDb res;
 double lb,hb;
 hb=x.Sup;
 lb=x.Inf;
 if(x.Sup==0) return ivalDb(lb,hb);
 if (!IsInfinity(x.Sup) && !IsNaN(x.Sup)) {
         hb=AddULP(x.Sup);
 }
 res.SetBounds(lb,hb);
 return res;
}//-----------------------------------------------------------------------------

// Round Both Bounds: Round(x)
ivalDb Round(ivalDb &x)
{
 double lb,hb;
 ivalDb res;
 res=x;
 res=RoundLB(res);
 res=RoundHB(res);
 return res;
}//-----------------------------------------------------------------------------

// Return Empty Interval: EmptyInterval()
ivalDb EmptyInterval(void)
{ivalDb res;
 res.SetBounds(NaN(),NaN());
 return res;
}//-----------------------------------------------------------------------------

// Ival
ivalDb Ival(double a,double b)
{ivalDb res;
 res.SetBounds(a,b);
 return res;
}//-----------------------------------------------------------------------------

double Width(ivalDb &x)
{double width;
 width=FDLIBM::fabs(x.Sup-x.Inf);
 return width;
}//-----------------------------------------------------------------------------

double Center(ivalDb &x)
{double center;
 center=(x.Sup+x.Inf)/2;
 return center;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// PI Interval
////////////////////////////////////////////////////////////////////////////////

// Pi: PI()
ivalDb PI(void)
{ivalDb res;
 res.SetBounds(RestULP(N_PI),AddULP(N_PI));
 return res;
}//------------------------------------------------------------------------------

// 2*Pi: PI2()
ivalDb PI2(void)
{return ivalDb(2,2)*PI();
}//------------------------------------------------------------------------------

// Pi/2: PI05()
ivalDb PI05(void)
{ivalDb res;
 res.SetBounds(RestULP(N_PI_05),AddULP(N_PI_05));
 return res;
}//------------------------------------------------------------------------------

// Pi/4: PI025()
ivalDb PI025(void)
{ivalDb res;
 res.SetBounds(RestULP(N_PI_025),AddULP(N_PI_025));
 return res;
}//------------------------------------------------------------------------------

// Ln2: LN2()
ivalDb LN2(void)
{ivalDb res;
 res.SetBounds(RestULP(N_LN2),AddULP(N_LN2));
 return res;
}//------------------------------------------------------------------------------

// Zero: Return Zero Interval
ivalDb Zero(void)
{return Ival(0,0);
}// ----------------------------------------------------------------------------

ivalDb Infinity(void)
{return Ival(MInfinity(),PInfinity());
}// ----------------------------------------------------------------------------

ivalDb DuInfinity(void)
{return Ival(PInfinity(),MInfinity());
}// ----------------------------------------------------------------------------

// Disjoint
bool Disjoint(const ivalDb& x, const ivalDb& y)
{if (IsEmpty(x) || IsEmpty(x)) return true;
 return (x.Sup<y.Inf)||(y.Sup<x.Inf);
}//-----------------------------------------------------------------------------

// In
/*iboolean In(const ivalDb& x, const ivalDb& y)
{if (Disjoint (x,y)) return false;
 else if (y.Inf<=x.Inf && x.Sup<=y.Sup) return true;
 else return iboolean(iperhaps);
}//------------------------------------------------------------------------------
*/

// Distance
double Distance(const ivalDb& x, const ivalDb& y)
{if(IsEmpty(x)||IsEmpty(y)) return PInfinity();
 double r=max(0.0,x.Inf-y.Sup);
 r=max(r,y.Inf-x.Sup);
 return r;
} //----------------------------------------------------------------------------


////////////////////////////////////////////////////////////////////////////////
// Not Member Functions
////////////////////////////////////////////////////////////////////////////////

// Is Not a Number: IsNaN(x)
bool IsNaN(double x)
{int result;
 result=FDLIBM::isnan(x);
 if(result==1) return true;
 else return false;
}//-----------------------------------------------------------------------------


// Not defined number: NaN(x)
double NaN(void)
{
 //double result=FDLIBM::sqrt(-1);
 double result =0.0/0.0;
 return result;
}//-----------------------------------------------------------------------------


// Maximal Floting Point Number: PInfinity()
double PInfinity(void)
{
return _plusinfinity;
}
//------------------------------------------------------------------------------

// Minimal Floting Point Number: MInfinity()
double MInfinity(void)
{
return _minusinfinity;
}
//------------------------------------------------------------------------------

// Convert Double To "Integer": Integer(x)
double Integer(double x)
{if(IsMInfinity(x)) return MInfinity();
  else if(IsPInfinity(x)) return PInfinity();
  return double(int(x));
}//-----------------------------------------------------------------------------

// Infinity Positive ivalDb Test: IsPInfinity(x)
bool IsPInfinity(const double x)
{double y=PInfinity();
 if (x>=y) return true;
 else return false;
}//-----------------------------------------------------------------------------

// Infinity Negative Interval: IsMInfinity(x)
bool IsMInfinity(const double x)
{double y=MInfinity();
 if (x<=y) return true;
 else return false;

}//-----------------------------------------------------------------------------

// Infinity ivalDb Test: IsInfinity()
bool IsInfinity(const double x)
{if (IsMInfinity(x)|| IsPInfinity(x)) return true;
else return false;
}//-----------------------------------------------------------------------------

// Add ULP
double AddULP(double x)
{return FDLIBM::nextafter(x, PInfinity());
}//-----------------------------------------------------------------------------

// Rest ULP
double RestULP(double x)
{return FDLIBM::nextafter(x, MInfinity());
}//-----------------------------------------------------------------------------

double AddULPs(double x,int n)
{for(int i=0;i<n;i++)
 {x=AddULP(x);}
 return x;
}//-----------------------------------------------------------------------------

double RestULPs(double x,int n)
{for(int i=0;i<n;i++)
 {x=RestULP(x);}
 return x;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// I/O Stream Operators                                                       //
////////////////////////////////////////////////////////////////////////////////

ostream& operator<<(ostream & fout, const ivalDb &x)
{fout << "[" << x.GetInf() << ", " << x.GetSup() << "]";
 return fout;
}//-----------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
// Disable Floating Exception                                                 //
////////////////////////////////////////////////////////////////////////////////

void secureFpu()
{_control87(PC_64|MCW_EM,MCW_PC|MCW_EM);
}//-----------------------------------------------------------------------------
